@@ -31,12 +31,12 @@ int main( int /*argc*/, char ** /*argv*/)
     null.size( "70x70" );
     null.read( "null:black" );
 
-    Image model( "model.gif" );
+    Image model( "model.miff" );
     model.label( "Magick++" );
     model.borderColor( "black" );
     model.backgroundColor( "black" );
 
-    Image smile( "smile.gif" );
+    Image smile( "smile.miff" );
     smile.label( "Smile" );
     smile.borderColor( "black" );
 
@@ -45,11 +45,17 @@ int main( int /*argc*/, char ** /*argv*/)
     //
     cout << "Transform image..." << endl;
 
-    // Construct list containing five copies of null image
+    // Construct initial list containing five copies of null image
     list<Image> images( 5, null );
 
     Image example = model;
 
+    // Each of the following follow the pattern
+    //  1. obtain reference to (own copy of) image
+    //  2. apply label to image
+    //  3. apply operation to image
+    //  4. append image to container
+
     cout << "  annotate ..." << endl;
     example.label( "Annotate" );
     example.font( "@Generic.ttf" );
@@ -228,17 +234,7 @@ int main( int /*argc*/, char ** /*argv*/)
     cout << "  segment ..." << endl;
     example = model;
     example.label( "Segment" );
-    // Segment is *extremely* slow at 16-bit color so skip
-    // it in that case.
-    if ( example.depth() == 16 )
-      {
-	example.penColor( "gold" );
-	example.annotate( "Skipped", CenterGravity );
-      }
-    else
-      {
-	example.segment( );
-      }
+    example.segment( );
     images.push_back( example );
 
     cout << "  shade ..." << endl;
@@ -283,26 +279,13 @@ int main( int /*argc*/, char ** /*argv*/)
     example.label( "Wave" );
     example.wave( 25, 150 );
     images.push_back( example );
-
+    
     cout << "  zoom ..." << endl;
     example = model;
     example.label( "Zoom" );
     example.zoom( "50%" );
     images.push_back( example );
 
-//     {
-//       list<Image> temp = images;
-//       animateImages( temp.begin(), temp.end() );
-//     }
-//     {
-//       list<Image> temp = images;
-//       displayImages( temp.begin(), temp.end() );
-//     }
-//     {
-//       list<Image> temp = images;
-//       writeImages( temp.begin(), temp.end(), "demo-anim.gif", true );
-//     }
-
     //
     // Create title.
     //
@@ -361,10 +344,11 @@ int main( int /*argc*/, char ** /*argv*/)
 		    "Magick++ and ImageMagick libraries.",
 		    "+20+175" );
 
-    cout << "Write image \"demo.jpg\" ..." << endl;
+    cout << "Writing image \"demo.miff\" ..." << endl;
     final.matte( false );
-    final.write( "demo.jpg" );
+    final.write( "demo.miff" );
 
+    // Uncomment following lines to display image to screen
 //    cout <<  "Display image..." << endl;
 //    final.display();
 
