@@ -12,11 +12,11 @@
 #include <algorithm>
 #include <functional>
 #include <iterator>
-#include <Magick++/Drawable.h>
-#include <Magick++/Montage.h>
-#include <Magick++/LastError.h>
-#include <Magick++/Include.h>
-#include <Magick++/Functions.h>
+#include "Magick++/Drawable.h"
+#include "Magick++/Montage.h"
+#include "Magick++/LastError.h"
+#include "Magick++/Include.h"
+#include "Magick++/Functions.h"
 
 namespace Magick
 {
@@ -98,6 +98,7 @@ namespace Magick
   void readImages( Container *sequence_,
 		   const std::string &imageSpec_ );
   template <class Container>
+  // Read images from BLOB
   void readImages( Container *sequence_,
 		   const Blob &blob_ );
 
@@ -110,9 +111,10 @@ namespace Magick
 		    const std::string &imageSpec_,
 		    bool adjoin_ );
   template <class InputIterator>
+  // Write images to a BLOB
   void writeImages( InputIterator first_,
 		    InputIterator last_,
-		    const Blob &blob_,
+		    Blob *blob_,
 		    bool adjoin_ );
 
   //
@@ -256,6 +258,21 @@ namespace Magick
     const Color _penColor;
   };
 
+  // Convert the image colorspace representation
+  class colorSpaceImage : public std::unary_function<Image&,void>
+  {
+  public:
+    colorSpaceImage( ColorspaceType colorSpace_ )
+      : _colorSpace( colorSpace_ ) { }
+
+    void operator()( Image &image_ )
+      {
+	image_.colorSpace( _colorSpace );
+      }
+  private:
+    const ColorspaceType _colorSpace;
+  };
+
   // Comment image (add comment string to image)
   class commentImage : public std::unary_function<Image&,void>
   {
@@ -303,19 +320,6 @@ namespace Magick
     const CompositeOperator _compose;
   };
 
-  // Condense image (Re-run-length encode image in memory)
-  class condenseImage : public std::unary_function<Image&,void>
-  {
-  public:
-    condenseImage( void ) { }
-
-    void operator()( Image &image_ )
-      {
-	image_.condense( );
-      }
-  private:
-  };
-
   // Contrast image (enhance intensity differences in image)
   class contrastImage : public std::unary_function<Image&,void>
   {
@@ -470,7 +474,7 @@ namespace Magick
     // Flood-fill color across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    floodFillColorImage( int x_, int y_,
+    floodFillColorImage( unsigned int x_, unsigned int y_,
 			 const Color &fillColor_ )
       : _x(x_),
 	_y(y_),
@@ -486,7 +490,7 @@ namespace Magick
     // Flood-fill color across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    floodFillColorImage( int x_, int y_,
+    floodFillColorImage( unsigned int x_, unsigned int y_,
 			 const Color &fillColor_,
 			 const Color &borderColor_ )
       : _x(x_),
@@ -513,10 +517,10 @@ namespace Magick
 	  }
       }
   private:
-    const int   _x;
-    const int   _y;
-    const Color _fillColor;
-    const Color _borderColor;
+    const unsigned int   _x;
+    const unsigned int   _y;
+    const Color          _fillColor;
+    const Color          _borderColor;
   };
 
   // Flood-fill image with texture
@@ -526,7 +530,7 @@ namespace Magick
     // Flood-fill texture across pixels that match the color of the
     // target pixel and are neighbors of the target pixel.
     // Uses current fuzz setting when determining color match.
-    floodFillTextureImage( int x_, int y_,
+    floodFillTextureImage( unsigned int x_, unsigned int y_,
 			   const Image &texture_ )
       : _x(x_),
 	_y(y_),
@@ -542,7 +546,7 @@ namespace Magick
     // Flood-fill texture across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    floodFillTextureImage( int x_, int y_,
+    floodFillTextureImage( unsigned int x_, unsigned int y_,
 			   const Image &texture_,
 			   const Color &borderColor_ )
       : _x(x_),
@@ -569,10 +573,10 @@ namespace Magick
 	  }
       }
   private:
-    const int     _x;
-    const int     _y;
-    const Image   _texture;
-    const Color   _borderColor;
+    const unsigned int  _x;
+    const unsigned int  _y;
+    const Image         _texture;
+    const Color         _borderColor;
   };
 
   // Flop image (reflect each scanline in the horizontal direction)
@@ -741,6 +745,22 @@ namespace Magick
     PaintMethod   _method;
   };
 
+  // Filter image by replacing each pixel component with the median
+  // color in a circular neighborhood
+  class medianFilterImage : public std::unary_function<Image&,void>
+  {
+    public:
+    medianFilterImage( unsigned int radius_ )
+      : _radius( radius_ ) { }
+
+    void operator()( Image &image_ )
+      {
+	image_.medianFilter( _radius );
+      }
+  private:
+    const unsigned int _radius;
+  };
+
   // Reduce image by integral size
   class minifyImage : public std::unary_function<Image&,void>
   {
@@ -912,21 +932,15 @@ namespace Magick
   class rotateImage : public std::unary_function<Image&,void>
   {
   public:
-    rotateImage( double degrees_,
-		 bool crop_ = false,
-		 unsigned int sharpen_ = false )
-      : _degrees( degrees_ ),
-	_crop( crop_ ),
-	_sharpen( sharpen_ ) { }
+    rotateImage( double degrees_ )
+      : _degrees( degrees_ ) { }
 
     void operator()( Image &image_ )
       {
-	image_.rotate( _degrees, _crop, _sharpen );
+	image_.rotate( _degrees );
       }
   private:
     const double       _degrees;
-    const bool         _crop;
-    const unsigned int _sharpen;
   };
 
   // Resize image by using pixel sampling algorithm
@@ -1018,20 +1032,17 @@ namespace Magick
   {
   public:
     shearImage( double xShearAngle_,
-		double yShearAngle_,
-		bool crop_ = false )
+		double yShearAngle_ )
       : _xShearAngle( xShearAngle_ ),
-	_yShearAngle( yShearAngle_ ),
-	_crop( crop_ ) { }
+	_yShearAngle( yShearAngle_ ) { }
 
     void operator()( Image &image_ )
       {
-	image_.shear( _xShearAngle, _yShearAngle, _crop );
+	image_.shear( _xShearAngle, _yShearAngle );
       }
   private:
     const double _xShearAngle;
     const double _yShearAngle;
-    const bool   _crop;
   };
 
   // Solarize image (similar to effect seen when exposing a
@@ -1165,21 +1176,6 @@ namespace Magick
     const Geometry _cropGeometry;
   };
 
-  // Convert the image colorspace representation
-  class transformColorSpaceImage : public std::unary_function<Image&,void>
-  {
-  public:
-    transformColorSpaceImage( ColorspaceType colorSpace_ )
-      : _colorSpace( colorSpace_ ) { }
-
-    void operator()( Image &image_ )
-      {
-	image_.transformColorSpace( _colorSpace );
-      }
-  private:
-    const ColorspaceType _colorSpace;
-  };
-
   // Set image color to transparent
   class transparentImage : public std::unary_function<Image&,void>
   {
@@ -1728,15 +1724,15 @@ namespace Magick
   };
 
   // Postscript page size.
-  class psPageSizeImage : public std::unary_function<Image&,void>
+  class pageImage : public std::unary_function<Image&,void>
   {
   public:
-    psPageSizeImage( const Geometry &pageSize_ )
+    pageImage( const Geometry &pageSize_ )
       : _pageSize( pageSize_ ) { }
 
     void operator()( Image &image_ )
       {
-	image_.psPageSize( _pageSize );
+	image_.page( _pageSize );
       }
   private:
     const Geometry _pageSize;
@@ -2103,15 +2099,12 @@ namespace Magick
     MagickLib::GetMontageInfo( montageInfo );
 
     // Set some default options based on current imageInfo settings.
-    if ( first_->backgroundColor().isValid() )
-      Magick::CloneString( &montageInfo->background_color,
-			   first_->backgroundColor() );
-    if ( first_->borderColor().isValid() )
-      Magick::CloneString ( &montageInfo->border_color,
-			    first_->borderColor() );
-    if ( first_->matteColor().isValid() )
-      Magick::CloneString ( &montageInfo->matte_color,
-			    first_->matteColor() );
+    montageInfo->background_color = first_->backgroundColor();
+
+    montageInfo->border_color = first_->borderColor();
+
+    montageInfo->matte_color = first_->matteColor();
+
     if ( first_->penColor().isValid() )
       Magick::CloneString ( &montageInfo->pen,
 			    first_->penColor() );
@@ -2172,9 +2165,8 @@ namespace Magick
 
     insertImages( morphedImages_, images, options );
 
-    LastError* errPtr = LastError::instance();
-    if ( errPtr->isError() )
-      errPtr->throwException();
+    LastError lastError;
+    lastError.throwException();
   }
 
   // Quantize colors in images using current quantization settings
@@ -2213,10 +2205,9 @@ namespace Magick
     options.fileName( imageSpec_ );
     MagickLib::Image* images =  MagickLib::ReadImage( options.imageInfo() );
     insertImages( sequence_, images, options );
-  
-    LastError* errPtr = LastError::instance();
-    if ( errPtr->isError() )
-      errPtr->throwException();
+
+    LastError lastError;
+    lastError.throwException();
   }
   template <class Container>
   void readImages( Container *sequence_,
@@ -2225,16 +2216,10 @@ namespace Magick
     MagickLib::Image *images = MagickLib::BlobToImage( options.imageInfo(),
 						       static_cast<const char *>(blob_.data()),
 						       blob_.length() );
-
-    // Reset-blob struct in Image.
-    // FIXME: requirement for this is eliminated in final 4.2.8
-    MagickLib::GetBlobInfo( &(images->blob) );
-
     insertImages( sequence_, images, options );
 
-    LastError* errPtr = LastError::instance();
-    if ( errPtr->isError() )
-      errPtr->throwException();
+    LastError lastError;
+    lastError.throwException();
   }
 
   // Write
@@ -2266,15 +2251,15 @@ namespace Magick
     if ( errorStat != false )
       return;
 
-    LastError* errPtr = LastError::instance();
-    if ( errPtr->isError() )
-      errPtr->throwException();
+    LastError lastError;
+    lastError.throwException();
   }
+  // Write images to BLOB
   template <class InputIterator>
   void writeImages( InputIterator first_,
 		    InputIterator last_,
-		    const Blob &blob_,
-		    bool adjoin_ ) {
+		    Blob *blob_,
+		    bool adjoin_ = true) {
     // Save original image settings
     std::string origMagick = first_->magick();
     bool origAdjoin = first_->adjoin();
@@ -2283,14 +2268,11 @@ namespace Magick
 
     linkImages( first_, last_ );
 
-    unsigned long length = 24576; // 64 x 64 x 6
+    size_t length = 2048; // Efficient size for small images
     void* data = MagickLib::ImageToBlob( first_->imageInfo(),
 					 first_->image(),
 					 &length );
     blob_->updateNoCopy( data, length );
-    // Reset-blob struct in Image.
-    // FIXME: requirement for this should be temporary
-    MagickLib::GetBlobInfo( &(first_->image()->blob) );
 
     unlinkImages( first_, last_ );
 
@@ -2298,9 +2280,8 @@ namespace Magick
     first_->magick( origMagick );
     first_->adjoin( origAdjoin );
 
-    LastError* errPtr = LastError::instance();
-    if ( errPtr->isError() )
-      errPtr->throwException();
+    LastError lastError;
+    lastError.throwException();
   }
 
   template <class InputIterator>
@@ -2308,8 +2289,8 @@ namespace Magick
 		   InputIterator last_ ) {
 
     MagickLib::Image* previous = (MagickLib::Image*) NULL;
-
-    for ( InputIterator iter = first_; iter != last_; iter++ )
+    int scene = 0;
+    for ( InputIterator iter = first_; iter != last_; ++iter )
       {
 	// Unless we reduce the reference count to one, the same image
 	// structure may occur more than once in the container, causing
@@ -2320,10 +2301,14 @@ namespace Magick
 
 	current->previous = previous;
 	current->next     = (MagickLib::Image*) NULL;
+	current->orphan   = (int)false; // In a list
 
 	if ( previous != (MagickLib::Image*) NULL)
 	  previous->next = current;
 
+	current->scene=scene;
+	++scene;
+
 	previous = current;
       }
   }
@@ -2331,11 +2316,12 @@ namespace Magick
   template <class InputIterator>
   void unlinkImages( InputIterator first_,
 		     InputIterator last_ ) {
-    for( InputIterator iter = first_; iter != last_; iter++ )
+    for( InputIterator iter = first_; iter != last_; ++iter )
       {
 	MagickLib::Image* image = iter->image();
 	image->previous = (MagickLib::Image*) NULL;
 	image->next = (MagickLib::Image*) NULL;
+	image->orphan = (int)true; // Stand-alone
       }
   }
 
