@@ -2,6 +2,9 @@
 /* Define to empty if the keyword does not work.  */
 #undef const
 
+/* Define to empty if the keyword does not work.  */
+#undef inline
+
 /* Define if you don't have vprintf but do have _doprnt.  */
 #undef HAVE_DOPRNT
 
@@ -32,6 +35,10 @@
 /* Define if you have the ANSI C header files.  */
 #undef STDC_HEADERS
 
+/* Define if your processor stores words with the most significant
+   byte first (like Motorola and SPARC, unlike Intel and VAX).  */
+#undef WORDS_BIGENDIAN
+
 /* Define on System V Release 4.  */
 #undef SVR4
 
@@ -41,10 +48,13 @@
 /* Define if the X Window System is missing or not being used.  */
 #undef X_DISPLAY_MISSING
 
+/* Define if you have Posix thread methods.  */
+#undef HasPTHREADS
+
 /* Define if you have X11 library */
 #undef HasX11
 
-/* Include patented LZW compression */
+/* Include LZW compression */
 #undef HasLZW
 
 /* X11 server supports shape extension */
@@ -209,6 +219,12 @@
 /* Define location of the Postscript support program 'ps' */
 #undef PSDelegate
 
+/* Define location of the ILBM support program 'ilbmtoppm' */
+#undef ILBMDecodeDelegate
+
+/* Define location of the ILBM support program 'ppmtoilbm' */
+#undef ILBMEncodeDelegate
+
 /* Define location of the MPEG support program 'mpeg2decode' */
 #undef MPEGDecodeDelegate
 
@@ -263,3 +279,14 @@
 /* Define location of 'enscript' program */
 #undef TXTDelegate
 
+/* Define location of the XCF support program 'xcftopnm' */
+#undef XCFDecodeDelegate
+
+/* Define if you have the gettimeofday function.  */
+#undef HAVE_GETTIMEOFDAY
+
+/* Define if you have the POSIX.4 clock_gettime function.  */
+#undef HAVE_CLOCK_GETTIME
+
+/* Pixel cache threshold (default 80MB)*/
+#undef PixelCacheThreshold
