@@ -18,7 +18,8 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -65,6 +66,7 @@
 %
 %  Where options include:
 %    -backdrop            display image centered on a backdrop
+%    -cache threshold     megabytes of memory available to the pixel cache
 %    -colormap type       Shared or Private
 %    -colors value        preferred number of colors in the image
 %    -colorspace type     alternate image colorspace
@@ -108,15 +110,9 @@
 /*
   Include declarations.
 */
-#if !defined(macintosh)
 #include "magick/magick.h"
 #include "magick/defines.h"
 #include "magick/xwindows.h"
-#else
-#include "magick.h"
-#include "defines.h"
-#include "xwindows.h"
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -131,9 +127,9 @@
 %
 %  Method Usage displays the program command syntax.
 %
-%  The format of the Usage routine is:
+%  The format of the Usage method is:
 %
-%      Usage(client_name)
+%      void Usage(const char *client_name)
 %
 %  A description of each parameter follows:
 %
@@ -156,6 +152,7 @@ static void Usage(const char *client_name)
     *options[]=
     {
       "-backdrop            display image centered on a backdrop",
+      "-cache threshold     megabytes of memory available to the pixel cache",
       "-colormap type       Shared or Private",
       "-colors value        preferred number of colors in the image",
       "-colorspace type     alternate image colorspace",
@@ -392,26 +389,26 @@ int main(int argc,char **argv)
             }
           if (strncmp("background",option+1,5) == 0)
             {
-              resource_info.background_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
                   (void) CloneString(&resource_info.background_color,argv[i]);
-                  (void) CloneString(&image_info->background_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],
+                    &image_info->background_color);
                 }
               break;
             }
           if (strncmp("bordercolor",option+1,7) == 0)
             {
-              resource_info.border_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
                   (void) CloneString(&resource_info.border_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],&image_info->border_color);
                 }
               break;
             }
@@ -432,6 +429,17 @@ int main(int argc,char **argv)
         }
         case 'c':
         {
+          if (strncmp("cache",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+                    MagickError(OptionError,"Missing threshold",option);
+                }
+              SetCacheThreshold(atoi(argv[i]));
+              break;
+            }
           if (strncmp("colormap",option+1,6) == 0)
             {
               resource_info.colormap=PrivateColormap;
@@ -591,7 +599,6 @@ int main(int argc,char **argv)
             }
           if (strncmp("foreground",option+1,3) == 0)
             {
-              resource_info.foreground_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
@@ -702,13 +709,13 @@ int main(int argc,char **argv)
             break;
           if (strncmp("mattecolor",option+1,6) == 0)
             {
-              resource_info.matte_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
                   (void) CloneString(&resource_info.matte_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],&image_info->matte_color);
                 }
               break;
             }
@@ -952,16 +959,16 @@ int main(int argc,char **argv)
         Animate image to X server.
       */
       loaded_image=XAnimateImages(display,&resource_info,argv,argc,image);
-      DestroyImages(image);
       while (loaded_image != (Image *) NULL)
       {
         image=loaded_image;
         MogrifyImage(image_info,argc-1,argv,&image);
         loaded_image=XAnimateImages(display,&resource_info,argv,argc,image);
-        DestroyImages(image);
       }
     }
   DestroyDelegateInfo();
+  DestroyMagickInfo();
+  FreeMemory(argv);
   Exit(0);
 #endif
   return(False);
