@@ -19,7 +19,8 @@
 %                               January 1993                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -34,15 +35,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,6 +53,7 @@
 %
 %  Where options include:
 %    -blend value        blend the two images a given percent
+%    -cache threshold    number of megabytes available to the pixel cache
 %    -colors value       preferred number of colors in the image
 %    -compose operator   composite operator
 %    -colorspace type    alternate image colorspace
@@ -86,13 +88,8 @@
 /*
   Include declarations.
 */
-#if !defined(macintosh)
 #include "magick/magick.h"
 #include "magick/defines.h"
-#else
-#include "magick.h"
-#include "defines.h"
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -125,6 +122,7 @@ static void Usage(const char *client_name)
     *options[]=
     {
       "-blend value        blend the two images a given percent",
+      "-cache threshold    number of megabytes available to the pixel cache",
       "-colors value       preferred number of colors in the image",
       "-colorspace type    alternate image colorspace",
       "-comment string     annotate image with comment",
@@ -185,6 +183,7 @@ int main(int argc,char **argv)
   char
     *client_name,
     *displacement_geometry,
+    *watermark_geometry,
     *filename,
     *geometry,
     *option,
@@ -204,7 +203,7 @@ int main(int argc,char **argv)
     *mask_image;
 
   ImageInfo
-    image_info;
+    *image_info;
 
   int
     gravity,
@@ -235,10 +234,11 @@ int main(int argc,char **argv)
   compose=ReplaceCompositeOp;
   composite_image=(Image *) NULL;
   displacement_geometry=(char *) NULL;
+  watermark_geometry=(char *) NULL;
   geometry=(char *) NULL;
   gravity=NorthWestGravity;
   image=(Image *) NULL;
-  GetImageInfo(&image_info);
+  image_info=CloneImageInfo((ImageInfo *) NULL);
   mask_image=(Image *) NULL;
   stegano=0;
   stereo=False;
@@ -257,10 +257,10 @@ int main(int argc,char **argv)
           Read input images.
         */
         filename=argv[i];
-        (void) strcpy(image_info.filename,filename);
+        (void) strcpy(image_info->filename,filename);
         if (image == (Image *) NULL)
           {
-            image=ReadImage(&image_info);
+            image=ReadImage(image_info);
             if (image == (Image *) NULL)
               MagickError(OptionError,"Missing an image file name",
                 (char *) NULL);
@@ -270,13 +270,13 @@ int main(int argc,char **argv)
           MagickError(OptionError,"input images already specified",filename);
         if (composite_image == (Image *) NULL)
           {
-            composite_image=ReadImage(&image_info);
+            composite_image=ReadImage(image_info);
             if (composite_image == (Image *) NULL)
               MagickError(OptionError,"Missing an image file name",
                 (char *) NULL);
             continue;
           }
-        mask_image=ReadImage(&image_info);
+        mask_image=ReadImage(image_info);
         if (mask_image == (Image *) NULL)
           MagickError(OptionError,"Missing an image file name",(char *) NULL);
       }
@@ -285,6 +285,18 @@ int main(int argc,char **argv)
       {
         case 'b':
         {
+          if (strncmp("background",option+1,5) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if (i == argc)
+                    MagickError(OptionError,"Missing background color",option);
+                  (void) QueryColorDatabase(argv[i],
+                    &image_info->background_color);
+                }
+              break;
+            }
           if (strncmp("blend",option+1,3) == 0)
             {
               blend=0.0;
@@ -303,6 +315,17 @@ int main(int argc,char **argv)
         }
         case 'c':
         {
+          if (strncmp("cache",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+                    MagickError(OptionError,"Missing threshold",option);
+                }
+              SetCacheThreshold(atoi(argv[i]));
+              break;
+            }
           if (strncmp("colors",option+1,7) == 0)
             {
               if (*option == '-')
@@ -315,39 +338,39 @@ int main(int argc,char **argv)
             }
           if (strncmp("colorspace",option+1,7) == 0)
             {
-              image_info.colorspace=RGBColorspace;
+              image_info->colorspace=RGBColorspace;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.colorspace=UndefinedColorspace;
+                  image_info->colorspace=UndefinedColorspace;
                   if (Latin1Compare("cmyk",option) == 0)
-                    image_info.colorspace=CMYKColorspace;
+                    image_info->colorspace=CMYKColorspace;
                   if (Latin1Compare("gray",option) == 0)
-                    image_info.colorspace=GRAYColorspace;
+                    image_info->colorspace=GRAYColorspace;
                   if (Latin1Compare("ohta",option) == 0)
-                    image_info.colorspace=OHTAColorspace;
+                    image_info->colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
-                    image_info.colorspace=RGBColorspace;
+                    image_info->colorspace=RGBColorspace;
                   if (Latin1Compare("srgb",option) == 0)
-                    image_info.colorspace=sRGBColorspace;
+                    image_info->colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
-                    image_info.colorspace=TransparentColorspace;
+                    image_info->colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
-                    image_info.colorspace=XYZColorspace;
+                    image_info->colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
-                    image_info.colorspace=YCbCrColorspace;
+                    image_info->colorspace=YCbCrColorspace;
                   if (Latin1Compare("ycc",option) == 0)
-                    image_info.colorspace=YCCColorspace;
+                    image_info->colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
-                    image_info.colorspace=YIQColorspace;
+                    image_info->colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
-                    image_info.colorspace=YPbPrColorspace;
+                    image_info->colorspace=YPbPrColorspace;
                   if (Latin1Compare("yuv",option) == 0)
-                    image_info.colorspace=YUVColorspace;
-                  if (image_info.colorspace == UndefinedColorspace)
+                    image_info->colorspace=YUVColorspace;
+                  if (image_info->colorspace == UndefinedColorspace)
                     MagickError(OptionError,"Invalid colorspace type",option);
                 }
               break;
@@ -411,31 +434,31 @@ int main(int argc,char **argv)
             }
           if (strncmp("compress",option+1,5) == 0)
             {
-              image_info.compression=NoCompression;
+              image_info->compression=NoCompression;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.compression=UndefinedCompression;
+                  image_info->compression=UndefinedCompression;
                   if (Latin1Compare("None",option) == 0)
-                    image_info.compression=NoCompression;
+                    image_info->compression=NoCompression;
                   if (Latin1Compare("BZip",option) == 0)
-                    image_info.compression=BZipCompression;
+                    image_info->compression=BZipCompression;
                   if (Latin1Compare("Fax",option) == 0)
-                    image_info.compression=FaxCompression;
+                    image_info->compression=FaxCompression;
                   if (Latin1Compare("Group4",option) == 0)
-                    image_info.compression=Group4Compression;
+                    image_info->compression=Group4Compression;
                   if (Latin1Compare("JPEG",option) == 0)
-                    image_info.compression=JPEGCompression;
+                    image_info->compression=JPEGCompression;
                   if (Latin1Compare("LZW",option) == 0)
-                    image_info.compression=LZWCompression;
+                    image_info->compression=LZWCompression;
                   if (Latin1Compare("RunlengthEncoded",option) == 0)
-                    image_info.compression=RunlengthEncodedCompression;
+                    image_info->compression=RunlengthEncodedCompression;
                   if (Latin1Compare("Zip",option) == 0)
-                    image_info.compression=ZipCompression;
-                  if (image_info.compression == UndefinedCompression)
+                    image_info->compression=ZipCompression;
+                  if (image_info->compression == UndefinedCompression)
                     MagickError(OptionError,"Invalid compression type",option);
                 }
               break;
@@ -447,13 +470,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("density",option+1,3) == 0)
             {
-              image_info.density=(char *) NULL;
+              image_info->density=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  (void) CloneString(&image_info.density,argv[i]);
+                  (void) CloneString(&image_info->density,argv[i]);
                 }
               break;
             }
@@ -472,31 +495,31 @@ int main(int argc,char **argv)
             }
           if (Latin1Compare("display",option+1) == 0)
             {
-              image_info.server_name=(char *) NULL;
+              image_info->server_name=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing server name",option);
-                  (void) CloneString(&image_info.server_name,argv[i]);
+                  (void) CloneString(&image_info->server_name,argv[i]);
                 }
               break;
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=(char *) NULL;
+              image_info->dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing method",option);
-                  image_info.dispose=PostscriptGeometry(argv[i]);
+                  image_info->dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
           if (strncmp("dither",option+1,3) == 0)
             {
-              image_info.dither=(*option == '-');
+              image_info->dither=(*option == '-');
               break;
             }
           MagickError(OptionError,"Unrecognized option",option);
@@ -504,13 +527,13 @@ int main(int argc,char **argv)
         }
         case 'f':
         {
-          image_info.font=(char *) NULL;
+          image_info->font=(char *) NULL;
           if (*option == '-')
             {
               i++;
               if (i == argc)
                 MagickError(OptionError,"Missing font name",option);
-              (void) CloneString(&image_info.font,argv[i]);
+              (void) CloneString(&image_info->font,argv[i]);
             }
           break;
         }
@@ -579,23 +602,23 @@ int main(int argc,char **argv)
         {
           if (strncmp("interlace",option+1,3) == 0)
             {
-              image_info.interlace=NoInterlace;
+              image_info->interlace=NoInterlace;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.interlace=UndefinedInterlace;
+                  image_info->interlace=UndefinedInterlace;
                   if (Latin1Compare("None",option) == 0)
-                    image_info.interlace=NoInterlace;
+                    image_info->interlace=NoInterlace;
                   if (Latin1Compare("Line",option) == 0)
-                    image_info.interlace=LineInterlace;
+                    image_info->interlace=LineInterlace;
                   if (Latin1Compare("Plane",option) == 0)
-                    image_info.interlace=PlaneInterlace;
+                    image_info->interlace=PlaneInterlace;
                   if (Latin1Compare("Partition",option) == 0)
-                    image_info.interlace=PartitionInterlace;
-                  if (image_info.interlace == UndefinedInterlace)
+                    image_info->interlace=PartitionInterlace;
+                  if (image_info->interlace == UndefinedInterlace)
                     MagickError(OptionError,"Invalid interlace type",option);
                 }
               break;
@@ -638,13 +661,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("page",option+1,3) == 0)
             {
-              image_info.page=(char *) NULL;
+              image_info->page=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing page geometry",option);
-                  image_info.page=PostscriptGeometry(argv[i]);
+                  image_info->page=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -655,13 +678,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("quality",option+1,2) == 0)
             {
-              image_info.quality=75;
+              image_info->quality=75;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing quality",option);
-                  image_info.quality=atoi(argv[i]);
+                  image_info->quality=atoi(argv[i]);
                 }
               break;
             }
@@ -682,13 +705,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("size",option+1,2) == 0)
             {
-              image_info.size=(char *) NULL;
+              image_info->size=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  (void) CloneString(&image_info.size,argv[i]);
+                  (void) CloneString(&image_info->size,argv[i]);
                 }
               break;
             }
@@ -737,7 +760,25 @@ int main(int argc,char **argv)
         {
           if (strncmp("verbose",option+1,2) == 0)
             {
-              image_info.verbose=(*option == '-');
+              image_info->verbose=(*option == '-');
+              break;
+            }
+          MagickError(OptionError,"Unrecognized option",option);
+          break;
+        }
+        case 'w':
+        {
+          if (Latin1Compare("watermark",option+1) == 0)
+            {
+              watermark_geometry=(char *) NULL;
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+                    MagickError(OptionError,"Missing geometry",option);
+                  (void) CloneString(&watermark_geometry,argv[i]);
+                  compose=ModulateCompositeOp;
+                }
               break;
             }
           MagickError(OptionError,"Unrecognized option",option);
@@ -764,37 +805,51 @@ int main(int argc,char **argv)
     }
   if (compose == BlendCompositeOp)
     {
-      register RunlengthPacket
-        *p;
+      Quantum
+        opacity;
 
-      unsigned short
-        index;
+      register PixelPacket
+        *q;
 
       /*
         Create mattes for blending.
       */
-      index=(unsigned short)
-        (DownScale(MaxRGB)-(((int) DownScale(MaxRGB)*blend)/100));
+      opacity=Opaque-(Opaque*blend)/100;
       image->class=DirectClass;
       image->matte=True;
-      p=image->pixels;
-      for (i=0; i < (int) image->packets; i++)
+      for (y=0; y < (int) image->rows; y++)
       {
-        p->index=index;
-        p++;
+        q=GetPixelCache(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (int) image->columns; x++)
+        {
+          q->opacity=opacity;
+          q++;
+        }
+        if (!SyncPixelCache(image))
+          break;
       }
-      index=(unsigned short) (DownScale(MaxRGB)-index);
       composite_image->class=DirectClass;
       composite_image->matte=True;
-      p=composite_image->pixels;
-      for (i=0; i < (int) composite_image->packets; i++)
+      for (y=0; y < (int) composite_image->rows; y++)
       {
-        p->index=index;
-        p++;
+        q=GetPixelCache(composite_image,0,y,composite_image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (int) composite_image->columns; x++)
+        {
+          q->opacity=Opaque-opacity;
+          q++;
+        }
+        if (!SyncPixelCache(composite_image))
+          break;
       }
     }
   if (compose == DisplaceCompositeOp)
     composite_image->geometry=displacement_geometry;
+  if (compose == ModulateCompositeOp)
+    composite_image->geometry=watermark_geometry;
   /*
     Combine image.
   */
@@ -902,16 +957,20 @@ int main(int argc,char **argv)
     Transmogrify image as defined by the image processing options.
   */
   CommentImage(combined_image,composite_image->comments);
-  MogrifyImage(&image_info,argc,argv,&combined_image);
+  MogrifyImage(image_info,argc,argv,&combined_image);
   /*
     Write image.
   */
   (void) strcpy(combined_image->filename,write_filename);
-  SetImageInfo(&image_info,True);
-  status=WriteImage(&image_info,combined_image);
-  if (image_info.verbose)
-    DescribeImage(combined_image,(FILE *) NULL,False);
+  SetImageInfo(image_info,True);
+  status=WriteImage(image_info,combined_image);
+  if (image_info->verbose)
+    DescribeImage(combined_image,stderr,False);
+  DestroyImages(combined_image);
+  DestroyImageInfo(image_info);
   DestroyDelegateInfo();
+  DestroyMagickInfo();
+  FreeMemory(argv);
   Exit(status ? 0 : errno);
   return(False);
 }
