@@ -19,7 +19,8 @@
 %                            September 1994                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -34,15 +35,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,9 +55,10 @@
 %  Usage: identify [options ...] file [ files... ]
 %
 %  Where options include:
-%    -ping           efficiently determine image characteristics
-%    -size geometry  width and height of image
-%    -verbose        print detailed information about the image
+%    -cache threshold  megabytes of memory available to the pixel cache
+%    -ping             efficiently determine image characteristics
+%    -size geometry    width and height of image
+%    -verbose          print detailed information about the image
 %
 %
 */
@@ -64,13 +66,8 @@
 /*
   Include declarations.
 */
-#if !defined(macintosh)
 #include "magick/magick.h"
 #include "magick/defines.h"
-#else
-#include "magick.h"
-#include "defines.h"
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -102,9 +99,10 @@ static void Usage(const char *client_name)
   static const char
     *options[]=
     {
-      "-ping           efficiently determine image characteristics",
-      "-size geometry  width and height of image",
-      "-verbose        print detailed information about the image",
+      "-cache threshold  megabytes of memory available to the pixel cache",
+      "-ping             efficiently determine image characteristics",
+      "-size geometry    width and height of image",
+      "-verbose          print detailed information about the image",
       (char *) NULL
     };
 
@@ -136,12 +134,15 @@ int main(int argc,char **argv)
     *client_name,
     *option;
 
+  double
+    sans;
+
   Image
     *image,
     *p;
 
   ImageInfo
-    image_info;
+    *image_info;
 
   int
     number_images;
@@ -164,7 +165,7 @@ int main(int argc,char **argv)
     Set defaults.
   */
   count=0;
-  GetImageInfo(&image_info);
+  image_info=CloneImageInfo((ImageInfo *) NULL);
   number_images=0;
   /*
     Identify an image.
@@ -176,6 +177,22 @@ int main(int argc,char **argv)
       {
         switch(*(option+1))
         {
+          case 'c':
+          {
+            if (strncmp("cache",option+1,3) == 0)
+              {
+                if (*option == '-')
+                  {
+                    i++;
+                    if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+                      MagickError(OptionError,"Missing threshold",option);
+                  }
+                SetCacheThreshold(atoi(argv[i]));
+                break;
+              }
+            MagickError(OptionError,"Unrecognized option",option);
+            break;
+          }
           case 'h':
           {
             if (strncmp("help",option+1,2) == 0)
@@ -190,7 +207,7 @@ int main(int argc,char **argv)
           {
             if (strncmp("ping",option+1,2) == 0)
               {
-                image_info.ping=(*option == '-');
+                image_info->ping=(*option == '-');
                 break;
               }
             MagickError(OptionError,"Unrecognized option",option);
@@ -200,13 +217,13 @@ int main(int argc,char **argv)
           {
             if (strncmp("size",option+1,2) == 0)
               {
-                image_info.size=(char *) NULL;
+                image_info->size=(char *) NULL;
                 if (*option == '-')
                   {
                     i++;
                     if ((i == argc) || !IsGeometry(argv[i]))
                       MagickError(OptionError,"Missing geometry",option);
-                    (void) CloneString(&image_info.size,argv[i]);
+                    (void) CloneString(&image_info->size,argv[i]);
                   }
                 break;
               }
@@ -217,7 +234,7 @@ int main(int argc,char **argv)
           {
             if (strncmp("verbose",option+1,2) == 0)
               {
-                image_info.verbose=(*option == '-');
+                image_info->verbose=(*option == '-');
                 break;
               }
             MagickError(OptionError,"Unrecognized option",option);
@@ -239,29 +256,32 @@ int main(int argc,char **argv)
     /*
       Identify image.
     */
-    (void) strcpy(image_info.filename,argv[i]);
-    if (image_info.ping)
+    (void) strcpy(image_info->filename,argv[i]);
+    if (image_info->ping)
       {
-        image_info.verbose=True;
-        image=PingImage(&image_info);
+        image_info->verbose=True;
+        image=PingImage(image_info);
         number_images++;
         continue;
       }
-    image=ReadImage(&image_info);
+    image=ReadImage(image_info);
     if (image == (Image *) NULL)
       continue;
     for (p=image; p != (Image *) NULL; p=p->next)
     {
       if (p->scene == 0)
         p->scene=count++;
-      DescribeImage(p,(FILE *) NULL,image_info.verbose);
+      DescribeImage(p,stdout,image_info->verbose);
     }
     DestroyImages(image);
     number_images++;
   }
   if (number_images == 0)
     MagickError(OptionError,"Missing an image file name",(char *) NULL);
+  DestroyImageInfo(image_info);
   DestroyDelegateInfo();
+  DestroyMagickInfo();
+  FreeMemory(argv);
   Exit(0);
   return(False);
 }
