@@ -19,7 +19,8 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -34,15 +35,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -61,6 +62,7 @@
 %  Where options include:
 %    -adjoin             join images into a single multi-image file
 %    -border             include image borders in the output image
+%    -cache threshold    megabytes of memory available to the pixel cache
 %    -colors value       preferred number of colors in the image
 %    -colorspace type    alternate image colorspace
 %    -comment string     annotate image with comment
@@ -103,15 +105,9 @@
 /*
   Include declarations.
 */
-#if !defined(macintosh)
 #include "magick/magick.h"
 #include "magick/defines.h"
 #include "magick/xwindows.h"
-#else
-#include "magick.h"
-#include "defines.h"
-#include "xwindows.h"
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -145,6 +141,7 @@ static void Usage(const char *client_name)
     {
       "-adjoin             join images into a single multi-image file",
       "-border             include image borders in the output image",
+      "-cache threshold    megabytes of memory available to the pixel cache",
       "-colors value       preferred number of colors in the image",
       "-colorspace type    alternate image colorspace",
       "-comment string     annotate image with comment",
@@ -222,6 +219,9 @@ int main(int argc,char **argv)
   Display
     *display;
 
+  double
+    sans;
+
   Image
     *image,
     *next_image;
@@ -378,13 +378,12 @@ int main(int argc,char **argv)
             }
           if (strncmp("bordercolor",option+1,7) == 0)
             {
-              image_info->border_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing border color",option);
-                  (void) CloneString(&image_info->border_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],&image_info->border_color);
                 }
               break;
             }
@@ -393,6 +392,17 @@ int main(int argc,char **argv)
         }
         case 'c':
         {
+          if (strncmp("cache",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+                    MagickError(OptionError,"Missing threshold",option);
+                }
+              SetCacheThreshold(atoi(argv[i]));
+              break;
+            }
           if (strncmp("colors",option+1,7) == 0)
             {
               quantize_info->number_colors=0;
@@ -694,7 +704,7 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing size",option);
-                  image_info->pointsize=atoi(argv[i]);
+                  image_info->pointsize=atof(argv[i]);
                 }
               break;
             }
@@ -842,6 +852,7 @@ int main(int argc,char **argv)
   */
   MogrifyImages(image_info,argc,argv,&image);
   SetImageInfo(image_info,True);
+  status=0;
   for (p=image; p != (Image *) NULL; p=p->next)
   {
     status=WriteImage(image_info,p);
@@ -849,8 +860,11 @@ int main(int argc,char **argv)
       break;
   }
   if (image_info->verbose)
-    DescribeImage(image,(FILE *) NULL,False);
+    DescribeImage(image,stderr,False);
+  DestroyImages(image);
   DestroyDelegateInfo();
+  DestroyMagickInfo();
+  FreeMemory(argv);
   Exit(status ? 0 : errno);
 #endif
   return(False);
