@@ -18,7 +18,8 @@
 %                                December 1994                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -56,7 +57,7 @@
 
 #if defined(PRE_R6_ICCCM)
 /*
-  Compatibility routines for pre X11R6 ICCCM.
+  Compatibility methods for pre X11R6 ICCCM.
 */
 Status XInitImage(XImage *ximage)
 {
@@ -98,7 +99,7 @@ Status XInitImage(XImage *ximage)
 
 #if defined(PRE_R5_ICCCM)
 /*
-  Compatibility routines for pre X11R5 ICCCM.
+  Compatibility methods for pre X11R5 ICCCM.
 */
 void XrmCombineDatabase(XrmDatabase source,XrmDatabase *target,
   Bool override)
@@ -137,7 +138,7 @@ Bool XSupportsLocale()
 
 #if defined(PRE_R4_ICCCM)
 /*
-  Compatibility routines for pre X11R4 ICCCM.
+  Compatibility methods for pre X11R4 ICCCM.
 */
 XClassHint *XAllocClassHint)
 {
