@@ -18,7 +18,8 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,10 +54,10 @@
 */
 #include "magick.h"
 #include "defines.h"
-
 #if defined(HasX11)
 #include "xwindows.h"
 #include "animate.h"
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -71,9 +72,11 @@
 %  Method XMagickCommand makes a transform to the image or Image window as
 %  specified by a user menu button or keyboard command.
 %
-%  The format of the XMagickCommand routine is:
+%  The format of the XMagickCommand method is:
 %
-%    XMagickCommand(display,resource_info,windows,image,command_type,state);
+%      Image *XMagickCommand(Display *display,XResourceInfo *resource_info,
+%        XWindows *windows,const CommandType command_type,Image **image,
+%        unsigned int *state)
 %
 %  A description of each parameter follows:
 %
@@ -184,7 +187,7 @@ static Image *XMagickCommand(Display *display,XResourceInfo *resource_info,
         /* Something's wrong here.  No matching AllocMemory except for
            filelist[0] .. glennrp Jun 1999 */
         if (filelist[i] != filenames)
-          FreeMemory((char *) filelist[i]);
+          FreeMemory(filelist[i]);
 #endif
         if (next_image != (Image *) NULL)
           {
@@ -228,7 +231,10 @@ static Image *XMagickCommand(Display *display,XResourceInfo *resource_info,
       FormatString(windows->image.name,"ImageMagick: %.1024s",
         BaseFilename((*image)->filename));
       if (resource_info->title != (char *) NULL)
-        (void) strcpy(windows->image.name,(*image)->label);
+        {
+          LabelImage(*image,resource_info->title);
+          (void) strcpy(windows->image.name,(*image)->label);
+        }
       status=XStringListToTextProperty(&windows->image.name,1,&window_name);
       if (status == 0)
         break;
@@ -390,9 +396,10 @@ static Image *XMagickCommand(Display *display,XResourceInfo *resource_info,
 %  Method XAnimateBackgroundImage animates an image sequence in the
 %  background of a window.
 %
-%  The format of the XAnimateBackgroundImage routine is:
+%  The format of the XAnimateBackgroundImage method is:
 %
-%      XAnimateBackgroundImage(display,resource_info,image)
+%      void XAnimateBackgroundImage(Display *display,
+%        XResourceInfo *resource_info, Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -426,7 +433,7 @@ Export void XAnimateBackgroundImage(Display *display,
     visual_type[MaxTextExtent];
 
   static XPixelInfo
-    pixel_info;
+    pixel;
 
   static XStandardColormap
     *map_info;
@@ -512,8 +519,8 @@ Export void XAnimateBackgroundImage(Display *display,
         MagickError(XServerError,"Unable to create standard colormap",
           "Memory allocation failed");
       map_info->colormap=(Colormap) NULL;
-      pixel_info.pixels=(unsigned long *) NULL;
-      pixel_info.gamma_map=(XColor *) NULL;
+      pixel.pixels=(unsigned long *) NULL;
+      pixel.gamma_map=(XColor *) NULL;
       /*
         Initialize visual info.
       */
@@ -536,7 +543,17 @@ Export void XAnimateBackgroundImage(Display *display,
   if (window_info.id == root_window)
     XDestroyWindowColors(display,root_window);
   if (image->next != (Image *)NULL)
-    CoalesceImages(image);
+    {
+      Image
+        *coalesce_image;
+
+      coalesce_image=CoalesceImages(image);
+      if (coalesce_image != (Image *) NULL)
+        {
+          DestroyImages(image);
+          image=coalesce_image;
+        }
+    }
   if (resources.map_type == (char *) NULL)
     if ((visual_info->class != TrueColor) &&
         (visual_info->class != DirectColor))
@@ -606,20 +623,20 @@ Export void XAnimateBackgroundImage(Display *display,
       (visual_info->class == DirectColor))
     displayed_image->class=DirectClass;
   XMakeStandardColormap(display,visual_info,&resources,displayed_image,map_info,
-    &pixel_info);
+    &pixel);
   /*
     Graphic context superclass.
   */
-  context_values.background=pixel_info.background_color.pixel;
-  context_values.foreground=pixel_info.foreground_color.pixel;
-  pixel_info.annotate_context=XCreateGC(display,window_info.id,GCBackground |
+  context_values.background=pixel.background_color.pixel;
+  context_values.foreground=pixel.foreground_color.pixel;
+  pixel.annotate_context=XCreateGC(display,window_info.id,GCBackground |
     GCForeground,&context_values);
-  if (pixel_info.annotate_context == (GC) NULL)
+  if (pixel.annotate_context == (GC) NULL)
     MagickError(XServerError,"Unable to create graphic context",(char *) NULL);
   /*
     Initialize Image window attributes.
   */
-  XGetWindowInfo(display,visual_info,map_info,&pixel_info,(XFontStruct *) NULL,
+  XGetWindowInfo(display,visual_info,map_info,&pixel,(XFontStruct *) NULL,
     &resources,&window_info);
   /*
     Create the X image.
@@ -739,7 +756,7 @@ Export void XAnimateBackgroundImage(Display *display,
           /*
             Get pixel info for this scene.
           */
-          XGetPixelInfo(display,visual_info,map_info,&resources,images[scene],
+          XGetPixelPacket(display,visual_info,map_info,&resources,images[scene],
             &scene_info);
           window_info.pixel_info=(&scene_info);
         }
@@ -782,7 +799,7 @@ Export void XAnimateBackgroundImage(Display *display,
     else
       XDelay(display,(unsigned long) image->delay*10);
   }
-  window_info.pixel_info=(&pixel_info);
+  window_info.pixel_info=(&pixel);
   /*
     Display pixmap on the window.
   */
@@ -825,9 +842,10 @@ Export void XAnimateBackgroundImage(Display *display,
 %
 %  Method XAnimateImages displays an image via X11.
 %
-%  The format of the XAnimateImages routine is:
+%  The format of the XAnimateImages method is:
 %
-%      XAnimateImages(display,resource_info,argv,argc,image)
+%      Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
+%        char **argv,const int argc,Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -953,9 +971,6 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   CommandType
     command_type;
 
-  ErrorHandler
-    warning_handler;
-
   Image
     *displayed_image,
     **images,
@@ -995,6 +1010,9 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
     number_scenes,
     state;
 
+  WarningHandler
+    warning_handler;
+
   Window
     root_window;
 
@@ -1012,7 +1030,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
 
   XPixelInfo
     *icon_pixel,
-    *pixel_info,
+    *pixel,
     scene_info;
 
   XResourceInfo
@@ -1036,14 +1054,14 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
     *manager_hints;
 
   monitor_handler=(MonitorHandler) NULL;
-  warning_handler=(ErrorHandler) NULL;
+  warning_handler=(WarningHandler) NULL;
   windows=XSetWindows((XWindows *) ~0);
   if (windows != (XWindows *) NULL)
     {
       (void) chdir(working_directory);
       monitor_handler=SetMonitorHandler(XProgressMonitor);
       warning_handler=resource_info->display_warnings ?
-        SetWarningHandler(XWarning) : SetWarningHandler((ErrorHandler) NULL);
+        SetWarningHandler(XWarning) : SetWarningHandler((WarningHandler) NULL);
       (void) signal(SIGINT,XSignalHandler);
       (void) signal(SIGSEGV,XSignalHandler);
     }
@@ -1098,7 +1116,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   icon_map=windows->icon_map;
   visual_info=windows->visual_info;
   icon_visual=windows->icon_visual;
-  pixel_info=windows->pixel_info;
+  pixel=windows->pixel_info;
   icon_pixel=windows->icon_pixel;
   font_info=windows->font_info;
   icon_resources=windows->icon_resources;
@@ -1106,7 +1124,17 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   manager_hints=windows->manager_hints;
   root_window=XRootWindow(display,visual_info->screen);
   if (image->next != (Image *)NULL)
-    CoalesceImages(image);
+    {
+      Image
+        *coalesce_image;
+
+      coalesce_image=CoalesceImages(image);
+      if (coalesce_image != (Image *) NULL)
+        {
+          DestroyImages(image);
+          image=coalesce_image;
+        }
+    }
   if (resource_info->map_type == (char *) NULL)
     if ((visual_info->class != TrueColor) &&
         (visual_info->class != DirectColor))
@@ -1183,12 +1211,12 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
       (void) fprintf(stderr,"%.1024s\n",displayed_image->magick);
     }
   XMakeStandardColormap(display,visual_info,resource_info,displayed_image,
-    map_info,pixel_info);
+    map_info,pixel);
   /*
     Initialize graphic context.
   */
   windows->context.id=(Window) NULL;
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->context);
   class_hints->res_name="superclass";
   class_hints->res_class="Display";
@@ -1199,33 +1227,33 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
     &windows->context);
   if (resource_info->debug)
     (void) fprintf(stderr,"Window id: 0x%lx (context)\n",windows->context.id);
-  context_values.background=pixel_info->background_color.pixel;
+  context_values.background=pixel->background_color.pixel;
   context_values.font=font_info->fid;
-  context_values.foreground=pixel_info->foreground_color.pixel;
+  context_values.foreground=pixel->foreground_color.pixel;
   context_values.graphics_exposures=False;
   context_mask=GCBackground | GCFont | GCForeground | GCGraphicsExposures;
-  if (pixel_info->annotate_context != (GC) NULL)
-    XFreeGC(display,pixel_info->annotate_context);
-  pixel_info->annotate_context=
+  if (pixel->annotate_context != (GC) NULL)
+    XFreeGC(display,pixel->annotate_context);
+  pixel->annotate_context=
     XCreateGC(display,windows->context.id,context_mask,&context_values);
-  if (pixel_info->annotate_context == (GC) NULL)
+  if (pixel->annotate_context == (GC) NULL)
     MagickError(XServerError,"Unable to create graphic context",(char *) NULL);
-  context_values.background=pixel_info->depth_color.pixel;
-  if (pixel_info->widget_context != (GC) NULL)
-    XFreeGC(display,pixel_info->widget_context);
-  pixel_info->widget_context=
+  context_values.background=pixel->depth_color.pixel;
+  if (pixel->widget_context != (GC) NULL)
+    XFreeGC(display,pixel->widget_context);
+  pixel->widget_context=
     XCreateGC(display,windows->context.id,context_mask,&context_values);
-  if (pixel_info->widget_context == (GC) NULL)
+  if (pixel->widget_context == (GC) NULL)
     MagickError(XServerError,"Unable to create graphic context",(char *) NULL);
-  context_values.background=pixel_info->foreground_color.pixel;
-  context_values.foreground=pixel_info->background_color.pixel;
+  context_values.background=pixel->foreground_color.pixel;
+  context_values.foreground=pixel->background_color.pixel;
   context_values.plane_mask=
     context_values.background ^ context_values.foreground;
-  if (pixel_info->highlight_context != (GC) NULL)
-    XFreeGC(display,pixel_info->highlight_context);
-  pixel_info->highlight_context=XCreateGC(display,windows->context.id,
+  if (pixel->highlight_context != (GC) NULL)
+    XFreeGC(display,pixel->highlight_context);
+  pixel->highlight_context=XCreateGC(display,windows->context.id,
     context_mask | GCPlaneMask,&context_values);
-  if (pixel_info->highlight_context == (GC) NULL)
+  if (pixel->highlight_context == (GC) NULL)
     MagickError(XServerError,"Unable to create graphic context",(char *) NULL);
   XDestroyWindow(display,windows->context.id);
   /*
@@ -1263,10 +1291,10 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   */
   if (windows->image.id != (Window) NULL)
     {
-      FreeMemory((char *) windows->image.name);
-      FreeMemory((char *) windows->image.icon_name);
+      FreeMemory(windows->image.name);
+      FreeMemory(windows->image.icon_name);
     }
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->image);
   windows->image.shape=True;  /* non-rectangular shape hint */
   windows->image.shared_memory=resource_info->use_shared_memory;
@@ -1310,7 +1338,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
     ButtonReleaseMask | EnterWindowMask | ExposureMask | KeyPressMask |
     KeyReleaseMask | LeaveWindowMask | OwnerGrabButtonMask |
     PropertyChangeMask | StructureNotifyMask | SubstructureNotifyMask;
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->backdrop);
   if ((resource_info->backdrop) || (windows->backdrop.id != (Window) NULL))
     {
@@ -1390,7 +1418,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   /*
     Initialize Info widget.
   */
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->info);
   windows->info.name="Info";
   windows->info.icon_name="Info";
@@ -1420,7 +1448,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   /*
     Initialize Command widget.
   */
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->command);
   windows->command.data=MagickMenus;
   (void) XCommandWidget(display,windows,CommandMenu,(XEvent *) NULL);
@@ -1449,8 +1477,8 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
     Initialize Widget window.
   */
   if (windows->widget.id != (Window) NULL)
-    FreeMemory((char *) windows->widget.name);
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+    FreeMemory(windows->widget.name);
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->widget);
   FormatString(resource_name,"%.1024s.widget",resource_info->client_name);
   windows->widget.geometry=XGetResourceClass(resource_info->resource_database,
@@ -1484,8 +1512,8 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
     Initialize popup window.
   */
   if (windows->popup.id != (Window) NULL)
-    FreeMemory((char *) windows->popup.name);
-  XGetWindowInfo(display,visual_info,map_info,pixel_info,font_info,
+    FreeMemory(windows->popup.name);
+  XGetWindowInfo(display,visual_info,map_info,pixel,font_info,
     resource_info,&windows->popup);
   windows->popup.name=(char *) AllocateMemory(MaxTextExtent*sizeof(char));
   if (windows->popup.name == NULL)
@@ -1518,9 +1546,9 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   */
   if (monitor_handler == (MonitorHandler) NULL)
     monitor_handler=SetMonitorHandler(XProgressMonitor);
-  if (warning_handler == (ErrorHandler) NULL)
+  if (warning_handler == (WarningHandler) NULL)
     warning_handler=resource_info->display_warnings ?
-      SetWarningHandler(XWarning) : SetWarningHandler((ErrorHandler) NULL);
+      SetWarningHandler(XWarning) : SetWarningHandler((WarningHandler) NULL);
   (void) signal(SIGINT,XSignalHandler);
   (void) signal(SIGSEGV,XSignalHandler);
   /*
@@ -1566,7 +1594,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
           /*
             Get pixel info for this scene.
           */
-          XGetPixelInfo(display,visual_info,map_info,resource_info,
+          XGetPixelPacket(display,visual_info,map_info,resource_info,
             images[scene],&scene_info);
           windows->image.pixel_info=(&scene_info);
         }
@@ -1686,7 +1714,10 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
               FormatString(windows->image.name,
                 "ImageMagick: %.1024s[%u of %u]",p,scene,number_scenes);
               if (resource_info->title != (char *) NULL)
-                (void) strcpy(windows->image.name,image->label);
+                {
+                  LabelImage(image,resource_info->title);
+                  (void) strcpy(windows->image.name,image->label);
+                }
               status=
                 XStringListToTextProperty(&windows->image.name,1,&window_name);
               if (status != 0)
@@ -1698,7 +1729,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
           /*
             Copy X pixmap to Image window.
           */
-          XGetPixelInfo(display,visual_info,map_info,resource_info,
+          XGetPixelPacket(display,visual_info,map_info,resource_info,
           images[scene],&scene_info);
           windows->image.pixel_info=(&scene_info);
           windows->image.ximage->width=image->columns;
@@ -1828,22 +1859,22 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
                 {
                   if (magick_windows[i]->id == windows->icon.id)
                     continue;
-                  context_values.background=pixel_info->background_color.pixel;
-                  context_values.foreground=pixel_info->foreground_color.pixel;
+                  context_values.background=pixel->background_color.pixel;
+                  context_values.foreground=pixel->foreground_color.pixel;
                   XChangeGC(display,magick_windows[i]->annotate_context,
                     context_mask,&context_values);
                   XChangeGC(display,magick_windows[i]->widget_context,
                     context_mask,&context_values);
-                  context_values.background=pixel_info->foreground_color.pixel;
-                  context_values.foreground=pixel_info->background_color.pixel;
+                  context_values.background=pixel->foreground_color.pixel;
+                  context_values.foreground=pixel->background_color.pixel;
                   context_values.plane_mask=
                     context_values.background ^ context_values.foreground;
                   XChangeGC(display,magick_windows[i]->highlight_context,
                     context_mask | GCPlaneMask,&context_values);
                   magick_windows[i]->attributes.background_pixel=
-                    pixel_info->background_color.pixel;
+                    pixel->background_color.pixel;
                   magick_windows[i]->attributes.border_pixel=
-                    pixel_info->border_color.pixel;
+                    pixel->border_color.pixel;
                   magick_windows[i]->attributes.colormap=map_info->colormap;
                   XChangeWindowAttributes(display,magick_windows[i]->id,
                     magick_windows[i]->mask,&magick_windows[i]->attributes);
@@ -2345,9 +2376,9 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
       XFreePixmap(display,windows->image.matte_pixmaps[scene]);
     windows->image.matte_pixmaps[scene]=(Pixmap) NULL;
   }
-  FreeMemory((char *) windows->image.pixmaps);
+  FreeMemory(windows->image.pixmaps);
   windows->image.pixmaps=(Pixmap *) NULL;
-  FreeMemory((char *) windows->image.matte_pixmaps);
+  FreeMemory(windows->image.matte_pixmaps);
   windows->image.matte_pixmaps=(Pixmap *) NULL;
   if (loaded_image == (Image *) NULL)
     {
@@ -2376,7 +2407,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
       */
       XFreeStandardColormap(display,icon_visual,icon_map,icon_pixel);
       if (resource_info->map_type == (char *) NULL)
-        XFreeStandardColormap(display,visual_info,map_info,pixel_info);
+        XFreeStandardColormap(display,visual_info,map_info,pixel);
       /*
         Free X resources.
       */
@@ -2386,9 +2417,9 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
         XFreePixmap(display,windows->widget.highlight_stipple);
       if (windows->widget.highlight_stipple != (Pixmap) NULL)
         XFreePixmap(display,windows->widget.shadow_stipple);
-      XFreeGC(display,pixel_info->widget_context);
-      XFreeGC(display,pixel_info->highlight_context);
-      XFreeGC(display,pixel_info->annotate_context);
+      XFreeGC(display,pixel->widget_context);
+      XFreeGC(display,pixel->highlight_context);
+      XFreeGC(display,pixel->annotate_context);
       XFreeGC(display,icon_pixel->annotate_context);
       XFreeFont(display,font_info);
       XFree((void *) class_hints);
@@ -2397,13 +2428,13 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
       XFree((void *) visual_info);
       XFree((void *) icon_map);
       XFree((void *) map_info);
-      FreeMemory((char *) windows->popup.name);
-      FreeMemory((char *) windows->widget.name);
-      FreeMemory((char *) windows->image.icon_name);
-      FreeMemory((char *) windows->image.name);
-      FreeMemory((char *) windows->icon_resources);
-      FreeMemory((char *) windows->icon_pixel);
-      FreeMemory((char *) windows->pixel_info);
+      FreeMemory(windows->popup.name);
+      FreeMemory(windows->widget.name);
+      FreeMemory(windows->image.icon_name);
+      FreeMemory(windows->image.name);
+      FreeMemory(windows->icon_resources);
+      FreeMemory(windows->icon_pixel);
+      FreeMemory(windows->pixel_info);
       (void) signal(SIGSEGV,SIG_DFL);
       (void) signal(SIGINT,SIG_DFL);
       (void) XSetWindows((XWindows *) NULL);
@@ -2419,6 +2450,7 @@ Export Image *XAnimateImages(Display *display,XResourceInfo *resource_info,
   */
   (void) getcwd(working_directory,MaxTextExtent-1);
   (void) chdir(resource_info->home_directory);
+  DestroyImages(image);
   return(loaded_image);
 }
 #endif
