@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -109,12 +110,12 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
     y;
 
   register int
-    i,
-    j;
+    i;
 
   unsigned int
     height,
     length,
+    matte,
     number_lines,
     width;
 
@@ -127,8 +128,6 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
     return;
   if (*annotate_info->text == '\0')
     return;
-  if (!UncondenseImage(image))
-    return;
   /*
     Translate any embedded format characters (e.g. %f).
   */
@@ -140,6 +139,7 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
       return;
     }
   textlist=StringToList(text);
+  FreeMemory(text);
   if (textlist == (char **) NULL)
     {
       DestroyAnnotateInfo(local_info);
@@ -183,6 +183,7 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
   /*
     Annotate image.
   */
+  matte=image->matte;
   for (i=0; textlist[i] != (char *) NULL; i++)
   {
     if (*textlist[i] == '\0')
@@ -195,24 +196,34 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
     */
     FormatString(label,"%.1024s",textlist[i]);
     FreeMemory(textlist[i]);
-    for (j=strlen(label)-1; ; j--)
-    {
-      (void) strcpy(local_info->image_info->filename,label);
-      annotate_image=ReadLABELImage(local_info->image_info);
-      if (annotate_image == (Image *) NULL)
-        {
-          MagickWarning(ResourceLimitWarning,"Unable to annotate image",
-            (char *) NULL);
-          for ( ; textlist[i] != (char *) NULL; i++)
-            FreeMemory(textlist[i]);
-          FreeMemory((char *) textlist);
-          break;
-        }
-      if ((annotate_image->columns <= width) || (strlen(label) < 4))
-        break;
-      DestroyImage(annotate_image);
-      (void) strcpy(label+j,"...");
-    }
+    (void) strcpy(local_info->image_info->filename,label);
+    annotate_image=ReadLABELImage(local_info->image_info);
+    if (annotate_image == (Image *) NULL)
+      {
+        MagickWarning(ResourceLimitWarning,"Unable to annotate image",
+          (char *) NULL);
+        for (i++ ; textlist[i] != (char *) NULL; i++)
+          FreeMemory(textlist[i]);
+        FreeMemory(textlist);
+        DestroyAnnotateInfo(local_info);
+        return;
+      }
+    if (annotate_info->degrees != 0.0)
+      {
+        Image
+          *rotated_image;
+
+        /*
+          Rotate text.
+        */
+        rotated_image=RotateImage(annotate_image,annotate_info->degrees);
+        if (rotated_image != (Image *) NULL)
+          {
+            DestroyImage(annotate_image);
+            annotate_image=rotated_image;
+          }
+        local_info->bounds.height=annotate_image->rows;
+      }
     /*
       Composite text onto the image.
     */
@@ -226,7 +237,7 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
       }
       case NorthGravity:
       {
-        local_info->bounds.x=(width >> 1)-(annotate_image->columns >> 1)+x;
+        local_info->bounds.x=(width/2)-(annotate_image->columns/2)+x;
         local_info->bounds.y=i*local_info->bounds.height+y;
         break;
       }
@@ -239,9 +250,8 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
       case WestGravity:
       {
         local_info->bounds.x=x;
-        local_info->bounds.y=(height >> 1)-
-          (number_lines*local_info->bounds.height >> 1)+
-          i*local_info->bounds.height+y;
+        local_info->bounds.y=(height/2)-(number_lines*
+          local_info->bounds.height/2)+i*local_info->bounds.height+y;
         break;
       }
       case ForgetGravity:
@@ -249,18 +259,16 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
       case CenterGravity:
       default:
       {
-        local_info->bounds.x=(width >> 1)-(annotate_image->columns >> 1)+x;
-        local_info->bounds.y=(height >> 1)-
-          (number_lines*local_info->bounds.height >> 1)+
-          i*local_info->bounds.height+y;
+        local_info->bounds.x=(width/2)-(annotate_image->columns/2)+x;
+        local_info->bounds.y=(height/2)-(number_lines*
+          local_info->bounds.height/2)+i*local_info->bounds.height+y;
         break;
       }
       case EastGravity:
       {
         local_info->bounds.x=width-annotate_image->columns-x;
-        local_info->bounds.y=(height >> 1)-
-          (number_lines*local_info->bounds.height >> 1)+
-          i*local_info->bounds.height-y;
+        local_info->bounds.y=(height/2)-(number_lines*
+          local_info->bounds.height/2)+i*local_info->bounds.height-y;
         break;
       }
       case SouthWestGravity:
@@ -271,7 +279,7 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
       }
       case SouthGravity:
       {
-        local_info->bounds.x=(width >> 1)-(annotate_image->columns >> 1)-x;
+        local_info->bounds.x=(width/2)-(annotate_image->columns/2)-x;
         local_info->bounds.y=height-(i+1)*local_info->bounds.height-y;
         break;
       }
@@ -299,31 +307,16 @@ Export void AnnotateImage(Image *image,const AnnotateInfo *annotate_info)
             DestroyImage(box_image);
           }
       }
-    if (annotate_info->degrees != 0.0)
-      {
-        Image
-          *rotated_image;
-
-        /*
-          Rotate text.
-        */
-        rotated_image=
-          RotateImage(annotate_image,annotate_info->degrees,False,False);
-        if (rotated_image != (Image *) NULL)
-          {
-            DestroyImage(annotate_image);
-            annotate_image=rotated_image;
-          }
-      }
     CompositeImage(image,AnnotateCompositeOp,annotate_image,
       local_info->bounds.x,local_info->bounds.y);
     DestroyImage(annotate_image);
   }
+  image->matte=matte;
   DestroyAnnotateInfo(local_info);
   FreeMemory(text);
   for ( ; textlist[i] != (char *) NULL; i++)
     FreeMemory(textlist[i]);
-  FreeMemory((char *) textlist);
+  FreeMemory(textlist);
 }
 
 /*
@@ -364,7 +357,7 @@ Export AnnotateInfo *CloneAnnotateInfo(const ImageInfo *image_info,
 
   cloned_info=(AnnotateInfo *) AllocateMemory(sizeof(AnnotateInfo));
   if (cloned_info == (AnnotateInfo *) NULL)
-    MagickError(ResourceLimitWarning,"Unable to clone annotate info",
+    MagickError(ResourceLimitError,"Unable to clone annotate info",
       "Memory allocation failed");
   if (annotate_info == (AnnotateInfo *) NULL)
     {
@@ -384,7 +377,7 @@ Export AnnotateInfo *CloneAnnotateInfo(const ImageInfo *image_info,
     cloned_info->font_name=AllocateString(annotate_info->font_name);
   if (annotate_info->tile != (Image *) NULL)
     cloned_info->tile=CloneImage(annotate_info->tile,
-      annotate_info->tile->columns,annotate_info->tile->rows,True);
+      annotate_info->tile->columns,annotate_info->tile->rows,False);
   return(cloned_info);
 }
 
@@ -417,20 +410,21 @@ Export void DestroyAnnotateInfo(AnnotateInfo *annotate_info)
   assert(annotate_info != (AnnotateInfo *) NULL);
   DestroyImageInfo(annotate_info->image_info);
   if (annotate_info->geometry != (char *) NULL)
-    FreeMemory((char *) annotate_info->geometry);
+    FreeMemory(annotate_info->geometry);
   annotate_info->geometry=(char *) NULL;
   if (annotate_info->text != (char *) NULL)
-    FreeMemory((char *) annotate_info->text);
+    FreeMemory(annotate_info->text);
   annotate_info->text=(char *) NULL;
   if (annotate_info->primitive != (char *) NULL)
-    FreeMemory((char *) annotate_info->primitive);
+    FreeMemory(annotate_info->primitive);
   annotate_info->primitive=(char *) NULL;
   if (annotate_info->font_name != (char *) NULL)
-    FreeMemory((char *) annotate_info->font_name);
+    FreeMemory(annotate_info->font_name);
   annotate_info->font_name=(char *) NULL;
   if (annotate_info->tile != (Image *) NULL)
     DestroyImage(annotate_info->tile);
   annotate_info->tile=(Image *) NULL;
+  FreeMemory(annotate_info);
 }
 
 /*
@@ -474,8 +468,8 @@ Export void GetAnnotateInfo(const ImageInfo *image_info,
   annotate_info->primitive=(char *) NULL;
   annotate_info->font_name=(char *) NULL;
   annotate_info->degrees=0.0;
-  annotate_info->bounds.width=annotate_info->image_info->pointsize;
-  annotate_info->bounds.height=annotate_info->image_info->pointsize;
+  annotate_info->bounds.width=ceil(annotate_info->image_info->pointsize);
+  annotate_info->bounds.height=ceil(annotate_info->image_info->pointsize);
   annotate_info->bounds.x=0;
   annotate_info->bounds.y=0;
   /*
