@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,19 +60,19 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   R e a d C M Y K I m a g e                                                 %
+%   R e a d C Y M K I m a g e                                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadCMYKImage reads an image of raw cyan, magenta, yellow, and
-%  black bytes and returns it.  It allocates the memory necessary for the new
-%  Image structure and returns a pointer to the new image.
+%  Method ReadCMYKImage reads an image of raw cyan, yellow, magenta, and black
+%  bytes and returns it.  It allocates the memory necessary for the new Image
+%  structure and returns a pointer to the new image.
 %
-%  The format of the ReadCMYKImage routine is:
+%  The format of the ReadCMYKImage method is:
 %
-%      image=ReadCMYKImage(image_info)
+%      Image *ReadCMYKImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -96,12 +97,6 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
     i,
     x;
 
-  register RunlengthPacket
-    *q;
-
-  register unsigned char
-    *p;
-
   unsigned char
     *scanline;
 
@@ -109,9 +104,6 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
     packet_size,
     status;
 
-  unsigned short
-    value;
-
   /*
     Allocate image structure.
   */
@@ -119,7 +111,7 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
   if (image == (Image *) NULL)
     return((Image *) NULL);
   if ((image->columns == 0) || (image->rows == 0))
-    ReaderExit(OptionWarning,"must specify image size",image);
+    ReaderExit(OptionWarning,"Must specify image size",image);
   if (image_info->interlace != PartitionInterlace)
     {
       /*
@@ -134,7 +126,7 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
   /*
     Allocate memory for a scanline.
   */
-  packet_size=4*(QuantumDepth >> 3);
+  packet_size=image->depth > 8 ? 8 : 4;
   scanline=(unsigned char *)
     AllocateMemory(packet_size*image->tile_info.width*sizeof(unsigned char));
   if (scanline == (unsigned char *) NULL)
@@ -146,60 +138,42 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
         Skip to next image.
       */
       image->scene++;
-      for (y=0; y < (int) image->rows; i++)
-        (void) ReadBlob(image,packet_size*image->tile_info.width,
-          (char *) scanline);
+      for (y=0; y < (int) image->rows; y++)
+        (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
     }
+  x=packet_size*image->tile_info.x;
   do
   {
     /*
-      Initialize image structure.
+      Convert raster image to pixel packets.
     */
     image->colorspace=CMYKColorspace;
-    image->packets=image->columns*image->rows;
-    image->pixels=(RunlengthPacket *)
-      AllocateMemory(image->packets*sizeof(RunlengthPacket));
-    if (image->pixels == (RunlengthPacket *) NULL)
-      ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
-    SetImage(image);
-    /*
-      Convert raster image to runlength-encoded packets.
-    */
     switch (image_info->interlace)
     {
       case NoInterlace:
       default:
       {
         /*
-          No interlacing:  CMYKCMYKCMYKCMYKCMYK...
+          No interlacing:  CMYKCMYKCMYKCMYKCMYKCMYK...
         */
         for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-        q=image->pixels;
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         for (y=0; y < (int) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->tile_info.width,
-              (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->red,p);
-            ReadQuantum(q->green,p);
-            ReadQuantum(q->blue,p);
-            ReadQuantum(q->index,p);
-            q->length=0;
-            q++;
-          }
+            (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          if (!SetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) ReadPixelCache(image,CMYKQuantum,scanline+x);
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         break;
       }
       case LineInterlace:
@@ -207,63 +181,39 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
         /*
           Line interlacing:  CCC...MMM...YYY...KKK...CCC...MMM...YYY...KKK...
         */
-        packet_size=image->depth >> 3;
+        packet_size=image->depth > 8 ? 2 : 1;
         for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         for (y=0; y < (int) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->tile_info.width,
-              (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          q=image->pixels+y*image->columns;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->red,p);
-            q->length=0;
-            q++;
-          }
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          q=image->pixels+y*image->columns;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->green,p);
-            q++;
-          }
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          q=image->pixels+y*image->columns;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->blue,p);
-            q++;
-          }
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          q=image->pixels+y*image->columns;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->index,p);
-            q++;
-          }
+            (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          if (!SetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) ReadPixelCache(image,CyanQuantum,scanline+x);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          (void) ReadPixelCache(image,YellowQuantum,scanline+x);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          (void) ReadPixelCache(image,MagentaQuantum,scanline+x);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          (void) ReadPixelCache(image,BlackQuantum,scanline+x);
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         break;
       }
       case PlaneInterlace:
       case PartitionInterlace:
       {
+        unsigned int
+          span;
+
         /*
           Plane interlacing:  CCCCCC...MMMMMM...YYYYYY...KKKKKK...
         */
@@ -274,64 +224,28 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
             if (status == False)
               ReaderExit(FileOpenWarning,"Unable to open file",image);
           }
-        packet_size=image->depth >> 3;
+        packet_size=image->depth > 8 ? 2 : 1;
         for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         i=0;
-        q=image->pixels;
+        span=image->rows*(image->matte ? 4 : 3);
         for (y=0; y < (int) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->tile_info.width,
-              (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->red,p);
-            q->length=0;
-            q++;
-          }
+            (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          if (!SetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) ReadPixelCache(image,CyanQuantum,scanline+x);
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->rows << 2))
-              ProgressMonitor(LoadImageText,i,image->rows << 2);
+            if (QuantumTick(i,span))
+              ProgressMonitor(LoadImageText,i,span);
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-        if (image_info->interlace == PartitionInterlace)
-          {
-            CloseBlob(image);
-            AppendImageFormat("M",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType);
-            if (status == False)
-              ReaderExit(FileOpenWarning,"Unable to open file",image);
-          }
-        q=image->pixels;
-        for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-        for (y=0; y < (int) image->rows; y++)
-        {
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->green,p);
-            q++;
-          }
-          if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->rows << 2))
-              ProgressMonitor(LoadImageText,i,image->rows << 2);
-          i++;
-        }
-        count=image->tile_info.height-image->rows-image->tile_info.y;
-        for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
@@ -340,29 +254,50 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
             if (status == False)
               ReaderExit(FileOpenWarning,"Unable to open file",image);
           }
-        q=image->pixels;
         for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         for (y=0; y < (int) image->rows; y++)
         {
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->blue,p);
-            q++;
-          }
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) ReadPixelCache(image,YellowQuantum,scanline+x);
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->rows << 2))
-              ProgressMonitor(LoadImageText,i,image->rows << 2);
+            if (QuantumTick(i,span))
+              ProgressMonitor(LoadImageText,i,span);
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        if (image_info->interlace == PartitionInterlace)
+          {
+            CloseBlob(image);
+            AppendImageFormat("M",image->filename);
+            status=OpenBlob(image_info,image,ReadBinaryType);
+            if (status == False)
+              ReaderExit(FileOpenWarning,"Unable to open file",image);
+          }
+        for (y=0; y < image->tile_info.y; y++)
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        for (y=0; y < (int) image->rows; y++)
+        {
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) ReadPixelCache(image,MagentaQuantum,scanline+x);
+          if (!SyncPixelCache(image))
+            break;
+          if (image->previous == (Image *) NULL)
+            if (QuantumTick(i,span))
+              ProgressMonitor(LoadImageText,i,span);
+          i++;
+        }
+        count=image->tile_info.height-image->rows-image->tile_info.y;
+        for (y=0; y < count; y++)
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
@@ -371,44 +306,37 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
             if (status == False)
               ReaderExit(FileOpenWarning,"Unable to open file",image);
           }
-        q=image->pixels;
         for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         for (y=0; y < (int) image->rows; y++)
         {
           (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
-          p=scanline+packet_size*image->tile_info.x;
-          for (x=0; x < (int) image->columns; x++)
-          {
-            ReadQuantum(q->index,p);
-            q++;
-          }
+            scanline);
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) ReadPixelCache(image,BlackQuantum,scanline+x);
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->rows << 2))
-              ProgressMonitor(LoadImageText,i,image->rows << 2);
+            if (QuantumTick(i,span))
+              ProgressMonitor(LoadImageText,i,span);
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,
-            (char *) scanline);
+          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           (void) strcpy(image->filename,image_info->filename);
         break;
       }
     }
-    CondenseImage(image);
-    if (EOFBlob(image))
-      MagickWarning(CorruptImageWarning,"not enough pixels",image->filename);
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
-        break;
     /*
       Proceed to next image.
     */
-    count=ReadBlob(image,packet_size*image->columns,(char *) scanline);
+    if (image_info->subrange != 0)
+      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+        break;
+    count=ReadBlob(image,packet_size*image->tile_info.width,scanline);
     if (count > 0)
       {
         /*
@@ -421,11 +349,10 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
             return((Image *) NULL);
           }
         image=image->next;
-        ProgressMonitor(LoadImagesText,(unsigned int) TellBlob(image),
-          (unsigned int) image->filesize);
+        ProgressMonitor(LoadImagesText,TellBlob(image),image->filesize);
       }
   } while (count > 0);
-  FreeMemory((char *) scanline);
+  FreeMemory(scanline);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   CloseBlob(image);
@@ -443,12 +370,12 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteCMYKImage writes an image to a file in cyan, magenta, yellow,
+%  Method WriteCMYKImage writes an image to a file in cyan, yellow, magenta,
 %  and black rasterfile format.
 %
-%  The format of the WriteCMYKImage routine is:
+%  The format of the WriteCMYKImage method is:
 %
-%      status=WriteCMYKImage(image_info,image)
+%      unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -465,21 +392,24 @@ Export Image *ReadCMYKImage(const ImageInfo *image_info)
 Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
 {
   int
-    x,
     y;
 
-  register int
-    i,
-    j;
-
-  register RunlengthPacket
-    *p;
+  unsigned char
+    *pixels;
 
   unsigned int
+    packet_size,
     scene,
     status;
 
-  image->depth=QuantumDepth;
+  /*
+    Allocate memory for pixels.
+  */
+  packet_size=image->depth > 8 ? 8 : 4;
+  pixels=(unsigned char *)
+    AllocateMemory(packet_size*image->columns*sizeof(unsigned char));
+  if (pixels == (unsigned char *) NULL)
+    WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
   if (image_info->interlace != PartitionInterlace)
     {
       /*
@@ -502,52 +432,19 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
       case NoInterlace:
       default:
       {
-        register unsigned char
-          *q;
-
-        unsigned char
-          *pixels;
-
-        unsigned short
-          value;
-
-        /*
-          Allocate memory for pixels.
-        */
-        pixels=(unsigned char *)
-          AllocateMemory(image->columns*sizeof(RunlengthPacket));
-        if (pixels == (unsigned char *) NULL)
-          WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
         /*
           No interlacing:  CMYKCMYKCMYKCMYKCMYKCMYK...
         */
-        x=0;
-        y=0;
-        p=image->pixels;
-        q=pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
-          {
-            WriteQuantum(p->red,q);
-            WriteQuantum(p->green,q);
-            WriteQuantum(p->blue,q);
-            WriteQuantum(p->index,q);
-            x++;
-            if (x == (int) image->columns)
-              {
-                (void) WriteBlob(image,q-pixels,(char *) pixels);
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    ProgressMonitor(SaveImageText,y,image->rows);
-                q=pixels;
-                x=0;
-                y++;
-              }
-          }
-          p++;
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,CMYKQuantum,pixels);
+          (void) WriteBlob(image,packet_size*image->columns,pixels);
+          if (image->previous == (Image *) NULL)
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
-        FreeMemory((char *) pixels);
         break;
       }
       case LineInterlace:
@@ -555,34 +452,18 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
         /*
           Line interlacing:  CCC...MMM...YYY...KKK...CCC...MMM...YYY...KKK...
         */
-        if (!UncondenseImage(image))
-          return(False);
         for (y=0; y < (int) image->rows; y++)
         {
-          p=image->pixels+(y*image->columns);
-          for (x=0; x < (int) image->columns; x++)
-          {
-            WriteQuantumFile(p->red);
-            p++;
-          }
-          p=image->pixels+(y*image->columns);
-          for (x=0; x < (int) image->columns; x++)
-          {
-            WriteQuantumFile(p->green);
-            p++;
-          }
-          p=image->pixels+(y*image->columns);
-          for (x=0; x < (int) image->columns; x++)
-          {
-            WriteQuantumFile(p->blue);
-            p++;
-          }
-          p=image->pixels+(y*image->columns);
-          for (x=0; x < (int) image->columns; x++)
-          {
-            WriteQuantumFile(p->index);
-            p++;
-          }
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,CyanQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
+          (void) WritePixelCache(image,YellowQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
+          (void) WritePixelCache(image,MagentaQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
+          (void) WritePixelCache(image,BlackQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
           if (QuantumTick(y,image->rows))
             ProgressMonitor(SaveImageText,y,image->rows);
         }
@@ -601,12 +482,12 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             if (status == False)
               WriterExit(FileOpenWarning,"Unable to open file",image);
           }
-        p=image->pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
-            WriteQuantumFile(p->red);
-          p++;
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,CyanQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -617,12 +498,12 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
               WriterExit(FileOpenWarning,"Unable to open file",image);
           }
         ProgressMonitor(SaveImageText,100,400);
-        p=image->pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
-            WriteQuantumFile(p->green);
-          p++;
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,YellowQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -633,15 +514,14 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
               WriterExit(FileOpenWarning,"Unable to open file",image);
           }
         ProgressMonitor(SaveImageText,200,400);
-        p=image->pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
-            WriteQuantumFile(p->blue);
-          p++;
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,MagentaQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
         }
         ProgressMonitor(SaveImageText,300,400);
-        p=image->pixels;
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
@@ -650,11 +530,12 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             if (status == False)
               WriterExit(FileOpenWarning,"Unable to open file",image);
           }
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
-            WriteQuantumFile(p->index);
-          p++;
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,BlackQuantum,pixels);
+          (void) WriteBlob(image,image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           (void) strcpy(image->filename,image_info->filename);
@@ -664,10 +545,10 @@ Export unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
     }
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
+  FreeMemory(pixels);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
       image=image->previous;
