@@ -17,7 +17,8 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -32,15 +33,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -81,9 +82,9 @@ const char
 %  specified by the given path and directory name.  True is returned if
 %  the delegates file is read and at least one delegate is noted.
 %
-%  The format of the ReadDelegates routine is:
+%  The format of the ReadDelegates method is:
 %
-%     status=ReadDelegates(path,directory)
+%      unsigned int ReadDelegates(const char *path,const char *directory)
 %
 %  A description of each parameter follows:
 %
@@ -197,7 +198,7 @@ static unsigned int ReadDelegates(const char *path,const char *directory)
     Strip(delegate_info.commands);
     (void) SetDelegateInfo(&delegate_info);
     number_delegates++;
-    FreeMemory((char *) delegate_info.commands);
+    FreeMemory(delegate_info.commands);
   }
   return(number_delegates != 0);
 }
@@ -216,14 +217,9 @@ static unsigned int ReadDelegates(const char *path,const char *directory)
 %  Method DestroyDelegateInfo deallocates memory associated with the delegates
 %  list.
 %
-%  The format of the DestroyDelegateInfo routine is:
-%
-%      DestroyDelegateInfo(image_info)
-%
-%  A description of each parameter follows:
-%
-%    o image_info: Specifies a pointer to a Delegates structure.
+%  The format of the DestroyDelegateInfo method is:
 %
+%      DestroyDelegateInfo(void)
 %
 */
 Export void DestroyDelegateInfo(void)
@@ -241,10 +237,10 @@ Export void DestroyDelegateInfo(void)
   for (p=delegates; p != (DelegateInfo *) NULL; )
   {
     if (p->commands != (char *) NULL)
-      FreeMemory((char *) p->commands);
+      FreeMemory(p->commands);
     delegate=p;
     p=p->next;
-    FreeMemory((char *) delegate);
+    FreeMemory(delegate);
   }
 }
 
@@ -262,9 +258,10 @@ Export void DestroyDelegateInfo(void)
 %  Method GetDelegateInfo returns any delegates associated with the specified
 %  tag.  True is returned if a delegate is found, otherwise False.
 %
-%  The format of the GetDelegateInfo routine is:
+%  The format of the GetDelegateInfo method is:
 %
-%     status=GetDelegateInfo(decode_tag,encode_tag,delegate_info)
+%      unsigned int GetDelegateInfo(const char *decode_tag,
+%        const char *encode_tag,DelegateInfo *delegate_info)
 %
 %  A description of each parameter follows:
 %
@@ -305,7 +302,7 @@ Export unsigned int GetDelegateInfo(const char *decode_tag,
       delegate_info.commands=(char *) NULL;
       (void) SetDelegateInfo(&delegate_info);
       (void) ReadDelegates(DelegatePath,(char *) NULL);
-      (void) ReadDelegates((char *) getenv("DELEGATE_PATH"),"/");
+      (void) ReadDelegates((char *) getenv("DELEGATE_PATH"),DirectorySeparator);
       (void) ReadDelegates((char *) getenv("HOME"),"/.magick/");
       (void) ReadDelegates((char *) NULL,(char *) NULL);
       delegates=SetDelegateInfo((DelegateInfo *) NULL);
@@ -358,9 +355,10 @@ Export unsigned int GetDelegateInfo(const char *decode_tag,
 %  Method GetDelegateCommand replaces any embedded formatting characters with
 %  the appropriate image attribute and returns the resulting command.
 %
-%  The format of the GetDelegateCommand routine is:
+%  The format of the GetDelegateCommand method is:
 %
-%      command=GetDelegateCommand(image_info,image,decode_tag,encode_tag)
+%      char *GetDelegateCommand(const ImageInfo *image_info,const Image *image,
+%        const char *decode_tag,const char *encode_tag)
 %
 %  A description of each parameter follows:
 %
@@ -412,8 +410,8 @@ Export char *GetDelegateCommand(const ImageInfo *image_info,const Image *image,
     Free resources.
   */
   for (i=0; commands[i] != (char *) NULL; i++)
-    FreeMemory((char *) commands[i]);
-  FreeMemory((char *) commands);
+    FreeMemory(commands[i]);
+  FreeMemory(commands);
   return(command);
 }
 
@@ -432,9 +430,10 @@ Export char *GetDelegateCommand(const ImageInfo *image_info,const Image *image,
 %  the appropriate image attribute and executes the resulting command.  False
 %  is returned if the commands execute with success otherwise True.
 %
-%  The format of the InvokeDelegate routine is:
+%  The format of the InvokeDelegate method is:
 %
-%      InvokeDelegate(image_info,image,decode_tag,encode_tag)
+%      unsigned int InvokeDelegate(const ImageInfo *image_info,
+%        Image *image,const char *decode_tag,const char *encode_tag)
 %
 %  A description of each parameter follows:
 %
@@ -580,7 +579,7 @@ Export unsigned int InvokeDelegate(const ImageInfo *image_info,
         Latin1Upper(magick);
         (void) strcpy((char *) image_info->magick,magick);
         (void) strcpy(image->magick,magick);
-        FreeMemory((char *) magick);
+        FreeMemory(magick);
         (void) strcpy(filename,image->filename);
         local_info=CloneImageInfo(image_info);
         if (local_info == (ImageInfo *) NULL)
@@ -627,13 +626,13 @@ Export unsigned int InvokeDelegate(const ImageInfo *image_info,
       Execute delegate.
     */
     status=SystemCommand(image_info->verbose,command);
-    FreeMemory((char *) command);
+    FreeMemory(command);
     if (status != False)
       {
         MagickWarning(DelegateWarning,"delegate failed",commands[i]);
         break;
       }
-    FreeMemory((char *) commands[i]);
+    FreeMemory(commands[i]);
   }
   /*
     Free resources.
@@ -641,8 +640,8 @@ Export unsigned int InvokeDelegate(const ImageInfo *image_info,
   (void) remove(image_info->unique);
   (void) remove(image_info->zero);
   for ( ; commands[i] != (char *) NULL; i++)
-    FreeMemory((char *) commands[i]);
-  FreeMemory((char *) commands);
+    FreeMemory(commands[i]);
+  FreeMemory(commands);
   return(status);
 }
 
@@ -659,9 +658,9 @@ Export unsigned int InvokeDelegate(const ImageInfo *image_info,
 %
 %  Method ListDelegateInfo lists the image formats to a file.
 %
-%  The format of the ListDelegateInfo routine is:
+%  The format of the ListDelegateInfo method is:
 %
-%      ListDelegateInfo(file)
+%      void ListDelegateInfo(FILE *file)
 %
 %  A description of each parameter follows.
 %
@@ -750,9 +749,9 @@ Export void ListDelegateInfo(FILE *file)
 %  returns the address of the first delegate.  If the delegate is NULL, just
 %  the address of the first delegate is returned.
 %
-%  The format of the SetDelegateInfo routine is:
+%  The format of the SetDelegateInfo method is:
 %
-%     delegate_info=SetDelegateInfo(delegate_info)
+%      DelegateInfo *SetDelegateInfo(DelegateInfo *delegate_info)
 %
 %  A description of each parameter follows:
 %
@@ -814,9 +813,9 @@ Export DelegateInfo *SetDelegateInfo(DelegateInfo *delegate_info)
         /*
           Delegate overrides an existing one with the same tags.
         */
-        FreeMemory((char *) p->commands);
+        FreeMemory(p->commands);
         p->commands=new_delegate->commands;
-        FreeMemory((char *) new_delegate);
+        FreeMemory(new_delegate);
         return(delegates);
       }
     if (p->next == (DelegateInfo *) NULL)
