@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,6 +60,48 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s F I T S                                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method IsFITS returns True if the image format type, identified by the
+%  magick string, is FITS.
+%
+%  The format of the ReadFITSImage method is:
+%
+%      unsigned int IsFITS(const unsigned char *magick,
+%        const unsigned int length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsFITS returns True if the image format type is FITS.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+Export unsigned int IsFITS(const unsigned char *magick,
+  const unsigned int length)
+{
+  if (length < 6)
+    return(False);
+  if (strncmp((char *) magick,"IT0",3) == 0)
+    return(True);
+  if (strncmp((char *) magick,"SIMPLE",6) == 0)
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e a d F I T S I m a g e                                                 %
 %                                                                             %
 %                                                                             %
@@ -69,9 +112,9 @@
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
-%  The format of the ReadFITSImage routine is:
+%  The format of the ReadFITSImage method is:
 %
-%      image=ReadFITSImage(image_info)
+%      Image *ReadFITSImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -94,9 +137,9 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
       bits_per_pixel;
 
     unsigned int
-      number_of_axis,
       columns,
       rows,
+      number_axes,
       number_scenes;
 
     double
@@ -122,6 +165,9 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
   Image
     *image;
 
+  IndexPacket
+    index;
+
   int
     c,
     j,
@@ -136,7 +182,7 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
     i,
     x;
 
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
   register unsigned char
@@ -237,7 +283,7 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
         if (Latin1Compare(keyword,"BITPIX") == 0)
           fits_header.bits_per_pixel=(unsigned int) atoi(value);
         if (Latin1Compare(keyword,"NAXIS") == 0)
-          fits_header.number_of_axis=(unsigned int) atoi(value);
+          fits_header.number_axes=(unsigned int) atoi(value);
         if (Latin1Compare(keyword,"NAXIS1") == 0)
           fits_header.columns=(unsigned int) atoi(value);
         if (Latin1Compare(keyword,"NAXIS2") == 0)
@@ -266,8 +312,8 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
   /*
     Verify that required image information is defined.
   */
-  if ((!fits_header.simple) || (fits_header.number_of_axis < 1) ||
-      (fits_header.number_of_axis > 4) ||
+  if ((!fits_header.simple) || (fits_header.number_axes < 1) ||
+      (fits_header.number_axes > 4) ||
       (fits_header.columns*fits_header.rows) == 0)
     ReaderExit(CorruptImageWarning,"image type not supported",image);
   for (scene=0; scene < fits_header.number_scenes; scene++)
@@ -285,35 +331,30 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
         CloseBlob(image);
         return(image);
       }
-    image->colormap=(ColorPacket *)
-      AllocateMemory(image->colors*sizeof(ColorPacket));
-    if (image->colormap == (ColorPacket *) NULL)
+    image->colormap=(PixelPacket *)
+      AllocateMemory(image->colors*sizeof(PixelPacket));
+    if (image->colormap == (PixelPacket *) NULL)
       ReaderExit(FileOpenWarning,"Unable to open file",image);
     for (i=0; i < (int) image->colors; i++)
     {
-      image->colormap[i].red=(Quantum) ((long) (MaxRGB*i)/(image->colors-1));
-      image->colormap[i].green=(Quantum) ((long) (MaxRGB*i)/(image->colors-1));
-      image->colormap[i].blue=(Quantum) ((long) (MaxRGB*i)/(image->colors-1));
+      image->colormap[i].red=((unsigned long) (MaxRGB*i)/(image->colors-1));
+      image->colormap[i].green=((unsigned long) (MaxRGB*i)/(image->colors-1));
+      image->colormap[i].blue=((unsigned long) (MaxRGB*i)/(image->colors-1));
     }
     /*
       Initialize image structure.
     */
-    image->packets=image->columns*image->rows;
-    image->pixels=(RunlengthPacket *)
-      AllocateMemory(image->packets*sizeof(RunlengthPacket));
     packet_size=fits_header.bits_per_pixel/8;
     if (packet_size < 0)
       packet_size=(-packet_size);
-    fits_pixels=(unsigned char *)
-      AllocateMemory(image->packets*packet_size*sizeof(unsigned char));
-    if ((image->pixels == (RunlengthPacket *) NULL) ||
-        (fits_pixels == (unsigned char *) NULL))
+    fits_pixels=(unsigned char *) AllocateMemory(
+      packet_size*image->columns*image->rows*sizeof(unsigned char));
+    if (fits_pixels == (unsigned char *) NULL)
       ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
-    SetImage(image);
     /*
-      Convert FITS pixels to runlength-encoded packets.
+      Convert FITS pixels to pixel packets.
     */
-    status=ReadBlob(image,packet_size*image->packets,(char *) fits_pixels);
+    status=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
     if (status == False)
       MagickWarning(CorruptImageWarning,"Insufficient image data in file",
         image->filename);
@@ -340,7 +381,7 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
           pixel=(double) (*((double *) long_quantum));
         fits_header.min_data=pixel*fits_header.scale+fits_header.zero;
         fits_header.max_data=pixel*fits_header.scale+fits_header.zero;
-        for (i=1; i < (int) image->packets; i++)
+        for (i=1; i < (int) (image->columns*image->rows); i++)
         {
           long_quantum[0]=(*p);
           quantum=(*p++);
@@ -365,16 +406,17 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
         }
       }
     /*
-      Convert FITS pixels to runlength-encoded packets.
+      Convert FITS pixels to pixel packets.
     */
     scale=1.0;
-    if (fits_header.min_data != fits_header.max_data)
+    if ((fits_header.max_data-fits_header.min_data) <= 1.0)
       scale=MaxRGB/(fits_header.max_data-fits_header.min_data);
     p=fits_pixels;
-    q=image->pixels;
     for (y=image->rows-1; y >= 0; y--)
     {
-      q=image->pixels+(y*image->columns);
+      q=SetPixelCache(image,0,y,image->columns,1);
+      if (q == (PixelPacket *) NULL)
+        break;
       for (x=0; x < (int) image->columns; x++)
       {
         long_quantum[0]=(*p);
@@ -399,16 +441,16 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
         else
           if (scaled_pixel > MaxRGB)
             scaled_pixel=MaxRGB;
-        q->index=(unsigned short) scaled_pixel;
-        q->length=0;
-        q++;
+        index=(IndexPacket) (scaled_pixel+0.5);
+        image->indexes[x]=index;
+        *q++=image->colormap[index];
       }
+      if (!SyncPixelCache(image))
+        break;
       if (QuantumTick(y,image->rows))
         ProgressMonitor(LoadImageText,y,image->rows);
     }
-    FreeMemory((char *) fits_pixels);
-    SyncImage(image);
-    CondenseImage(image);
+    FreeMemory(fits_pixels);
     /*
       Proceed to next image.
     */
@@ -429,8 +471,7 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
             return((Image *) NULL);
           }
         image=image->next;
-        ProgressMonitor(LoadImagesText,(unsigned int) TellBlob(image),
-          (unsigned int) image->filesize);
+        ProgressMonitor(LoadImagesText,TellBlob(image),image->filesize);
       }
   }
   while (image->previous != (Image *) NULL)
@@ -453,9 +494,9 @@ Export Image *ReadFITSImage(const ImageInfo *image_info)
 %  Method WriteFITSImage writes a Flexible Image Transport System image to a
 %  file as gray scale intensities [0..255].
 %
-%  The format of the WriteFITSImage routine is:
+%  The format of the WriteFITSImage method is:
 %
-%      status=WriteFITSImage(image_info,image)
+%      unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -475,25 +516,22 @@ Export unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     buffer[81],
     *fits_header;
 
-  register int
-    i,
-    x,
+  int
     y;
 
-  register RunlengthPacket
-    *p;
+  register int
+    i;
 
-  register unsigned char
-    *q;
+  register PixelPacket
+    *p;
 
   unsigned char
     *pixels;
 
   unsigned int
+    packet_size,
     status;
 
-  if (!UncondenseImage(image))
-    return(False);
   /*
     Open output image file.
   */
@@ -504,9 +542,10 @@ Export unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   /*
     Allocate image memory.
   */
+  packet_size=image->depth > 8 ? 2 : 1;
   fits_header=(char *) AllocateMemory(2880*sizeof(unsigned char));
   pixels=(unsigned char *)
-    AllocateMemory(image->columns*sizeof(RunlengthPacket));
+    AllocateMemory(packet_size*image->columns*sizeof(unsigned char));
   if ((fits_header == (char *) NULL) || (pixels == (unsigned char *) NULL))
     WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
   /*
@@ -516,7 +555,7 @@ Export unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     fits_header[i]=' ';
   (void) strcpy(buffer,"SIMPLE  =                    T");
   (void) strncpy(fits_header+0,buffer,Extent(buffer));
-  (void) strcpy(buffer,"BITPIX  =                    8");
+  FormatString(buffer,"BITPIX  =                    %d",image->depth);
   (void) strncpy(fits_header+80,buffer,Extent(buffer));
   (void) strcpy(buffer,"NAXIS   =                    2");
   (void) strncpy(fits_header+160,buffer,Extent(buffer));
@@ -524,29 +563,30 @@ Export unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   (void) strncpy(fits_header+240,buffer,Extent(buffer));
   FormatString(buffer,"NAXIS2  =           %10u",image->rows);
   (void) strncpy(fits_header+320,buffer,Extent(buffer));
-  (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
+  FormatString(buffer,"DATAMIN =           %10u",0);
   (void) strncpy(fits_header+400,buffer,Extent(buffer));
-  (void) strcpy(buffer,"END");
+  FormatString(buffer,"DATAMAX =           %10u",MaxRGB);
   (void) strncpy(fits_header+480,buffer,Extent(buffer));
+  (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
+  (void) strncpy(fits_header+560,buffer,Extent(buffer));
+  (void) strcpy(buffer,"END");
+  (void) strncpy(fits_header+640,buffer,Extent(buffer));
   (void) WriteBlob(image,2880,(char *) fits_header);
-  FreeMemory((char *) fits_header);
+  FreeMemory(fits_header);
   /*
     Convert image to fits scale PseudoColor class.
   */
   for (y=image->rows-1; y >= 0; y--)
   {
-    p=image->pixels+(y*image->columns);
-    q=pixels;
-    for (x=0; x < (int) image->columns; x++)
-    {
-      *q++=DownScale(Intensity(*p));
-      p++;
-    }
-    (void) WriteBlob(image,q-pixels,(char *) pixels);
+    p=GetPixelCache(image,0,y,image->columns,1);
+    if (p == (PixelPacket *) NULL)
+      break;
+    (void) WritePixelCache(image,GrayQuantum,pixels);
+    (void) WriteBlob(image,packet_size*image->columns,pixels);
     if (QuantumTick(image->rows-y-1,image->rows))
       ProgressMonitor(SaveImageText,image->rows-y-1,image->rows);
   }
-  FreeMemory((char *) pixels);
+  FreeMemory(pixels);
   CloseBlob(image);
   return(True);
 }
