@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -69,11 +70,11 @@
 %
 %  Method ReadFPXImage reads a FlashPix image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
-%  pointer to the new image.  This routine was contributed by BillR@corbis.com.
+%  pointer to the new image.  This method was contributed by BillR@corbis.com.
 %
-%  The format of the ReadFPXImage routine is:
+%  The format of the ReadFPXImage method is:
 %
-%      image=ReadFPXImage(image_info)
+%      Image *ReadFPXImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -111,22 +112,20 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
   Image
     *image;
 
+  IndexPacket
+    index;
+
   int
-    x,
     y;
 
-  Quantum
-    blue,
-    green,
-    red;
-
   register int
-    i;
+    i,
+    x;
 
   register long
     packets;
 
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
   register unsigned char
@@ -149,9 +148,6 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
     tile_height,
     width;
 
-  unsigned short
-    index;
-
   /*
     Allocate image structure.
   */
@@ -182,7 +178,7 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
       c=ReadByte(image);
       while (c != EOF)
       {
-        (void) putc(c,file);
+        (void) fputc(c,file);
         c=ReadByte(image);
       }
       (void) fclose(file);
@@ -315,13 +311,12 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
       */
       image->class=PseudoClass;
       image->colors=MaxRGB+1;
-      image->colormap=(ColorPacket *)
-        AllocateMemory(image->colors*sizeof(ColorPacket));
-      if (image->colormap == (ColorPacket *) NULL)
+      image->colormap=(PixelPacket *)
+        AllocateMemory(image->colors*sizeof(PixelPacket));
+      if (image->colormap == (PixelPacket *) NULL)
         {
           FPX_ClearSystem();
-          ReaderExit(ResourceLimitWarning,"Memory allocation failed",
-            image);
+          ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
         }
       for (i=0; i < (int) image->colors; i++)
       {
@@ -341,10 +336,7 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
   packets=0;
   scanline=(unsigned char *) AllocateMemory(colorspace.numberOfComponents*
     image->columns*(tile_height+1)*sizeof(unsigned char));
-  image->pixels=(RunlengthPacket *)
-    AllocateMemory(image->columns*image->rows*sizeof(RunlengthPacket));
-  if ((image->pixels == (RunlengthPacket *) NULL) ||
-      (scanline == (unsigned char *) NULL))
+  if (scanline == (unsigned char *) NULL)
     {
       FPX_ClearSystem();
       (void) FPX_CloseImage(flashpix);
@@ -379,14 +371,11 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
   /*
     Initialize image pixels.
   */
-  red=0;
-  green=0;
-  blue=0;
-  index=0;
-  q=image->pixels;
-  SetRunlengthEncoder(q);
   for (y=0; y < (int) image->rows; y++)
   {
+    q=SetPixelCache(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
     if ((y % tile_height) == 0)
       {
         /*
@@ -402,11 +391,10 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
             &fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
-            FreeMemory((char *) scanline);
+            FreeMemory(scanline);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
-            ReaderExit(ResourceLimitWarning,"Memory allocation failed",
-              image);
+            ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
           }
       }
     /*
@@ -420,43 +408,32 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
     {
       if (fpx_info.numberOfComponents > 2)
         {
-          red=UpScale(*r);
-          green=UpScale(*g);
-          blue=UpScale(*b);
+          q->red=UpScale(*r);
+          q->green=UpScale(*g);
+          q->blue=UpScale(*b);
         }
       else
         {
           index=UpScale(*r);
-          red=index;
-          green=index;
-          blue=index;
+          image->indexes[x]=index;
+          q->red=index;
+          q->green=index;
+          q->blue=index;
         }
       if (image->matte)
-        index=UpScale(*a);
-      if ((red == q->red) && (green == q->green) && (blue == q->blue) &&
-          (index == q->index) && ((int) q->length < MaxRunlength))
-        q->length++;
-      else
-        {
-          if (packets != 0)
-            q++;
-          packets++;
-          q->red=red;
-          q->green=green;
-          q->blue=blue;
-          q->index=index;
-          q->length=0;
-        }
+        q->opacity=UpScale(*a);
+      q++;
       r+=red_component->columnStride;
       g+=green_component->columnStride;
       b+=blue_component->columnStride;
       a+=alpha_component->columnStride;
     }
+    if (!SyncPixelCache(image))
+      break;
     if (QuantumTick(y,image->rows))
       ProgressMonitor(LoadImageText,y,image->rows);
   }
-  SetRunlengthPackets(image,packets);
-  FreeMemory((char *) scanline);
+  FreeMemory(scanline);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
   if (image->temporary)
@@ -488,11 +465,11 @@ Export Image *ReadFPXImage(const ImageInfo *image_info)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Method WriteFPXImage writes an image in the FlashPix image format.  This
-%  routine was contributed by BillR@corbis.com.
+%  method was contributed by BillR@corbis.com.
 %
-%  The format of the WriteFPXImage routine is:
+%  The format of the WriteFPXImage method is:
 %
-%      fpx_status=WriteFPXImage(image_info,image)
+%      unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -722,18 +699,10 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     fpx_image;
 
   int
-    x,
     y;
 
   register int
-    i,
-    j;
-
-  register RunlengthPacket
-    *p;
-
-  register unsigned char
-    *q;
+    i;
 
   unsigned char
     *pixels;
@@ -746,9 +715,6 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     tile_height,
     tile_width;
 
-  unsigned short
-    value;
-
   /*
     Open input file.
   */
@@ -774,8 +740,7 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    WriterExit(ResourceLimitWarning,"Unable to initialize FPX library",
-      image);
+    WriterExit(ResourceLimitWarning,"Unable to initialize FPX library",image);
   tile_width=64;
   tile_height=64;
   colorspace.numberOfComponents=3;
@@ -821,8 +786,10 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   /*
     Set image summary info.
   */
+  summary_info.title_valid=False;
   summary_info.subject_valid=False;
   summary_info.author_valid=False;
+  summary_info.comments_valid=False;
   summary_info.keywords_valid=False;
   summary_info.OLEtemplate_valid=False;
   summary_info.last_author_valid=False;
@@ -904,38 +871,22 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   /*
     Write image scanlines.
   */
-  x=0;
-  y=0;
-  p=image->pixels;
-  q=pixels;
-  for (i=0; i < (int) image->packets; i++)
+  for (y=0; y < (int) image->rows; y++)
   {
-    for (j=0; j <= (int) p->length; j++)
-    {
-      if (fpx_info.numberOfComponents == 1)
-        WriteQuantum(Intensity(*p),q)
+    if (!GetPixelCache(image,0,y,image->columns,1))
+      break;
+    if (fpx_info.numberOfComponents == 1)
+      (void) WritePixelCache(image,GrayQuantum,pixels);
+    else
+      if (!image->matte)
+        (void) WritePixelCache(image,RGBQuantum,pixels);
       else
-        {
-          WriteQuantum(p->red,q);
-          WriteQuantum(p->green,q);
-          WriteQuantum(p->blue,q);
-          if (image->matte)
-            WriteQuantum(p->index,q);
-        }
-      x++;
-      if (x == (int) image->columns)
-        {
-          fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
-          if (fpx_status != FPX_OK)
-            break;
-          if (QuantumTick(y,image->rows))
-            ProgressMonitor(SaveImageText,y,image->rows);
-          q=pixels;
-          x=0;
-          y++;
-        }
-    }
-    p++;
+        (void) WritePixelCache(image,RGBAQuantum,pixels);
+    fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
+    if (fpx_status != FPX_OK)
+      break;
+    if (QuantumTick(y,image->rows))
+      ProgressMonitor(SaveImageText,y,image->rows);
   }
   if (image_info->view != (char *) NULL)
     {
@@ -1068,7 +1019,7 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     }
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  FreeMemory((char *) pixels);
+  FreeMemory(pixels);
   if (image->temporary)
     {
       FILE
@@ -1084,7 +1035,7 @@ Export unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       if (file == (FILE *) NULL)
         WriterExit(FileOpenWarning,"Unable to open file",image);
       for (c=fgetc(file); c != EOF; c=fgetc(file))
-        (void) putc(c,fpx_image.file);
+        (void) fputc(c,fpx_image.file);
       (void) fclose(file);
       (void) remove(image->filename);
       image->temporary=False;
