@@ -18,7 +18,8 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -69,9 +70,9 @@
 %  Method Contrast enhances the intensity differences between the lighter
 %  and darker elements of the image.
 %
-%  The format of the Contrast routine is:
+%  The format of the Contrast method is:
 %
-%      Contrast(sign,red,green,blue)
+%      void Contrast(const int sign,Quantum *red,Quantum *green,Quantum *blue)
 %
 %  A description of each parameter follows:
 %
@@ -87,6 +88,7 @@ Export void Contrast(const int sign,Quantum *red,Quantum *green,Quantum *blue)
     brightness,
     hue,
     saturation,
+    scale,
     theta;
 
   /*
@@ -97,7 +99,8 @@ Export void Contrast(const int sign,Quantum *red,Quantum *green,Quantum *blue)
   assert(blue != (Quantum *) NULL);
   TransformHSL(*red,*green,*blue,&hue,&saturation,&brightness);
   theta=(brightness-0.5)*M_PI;
-  brightness+=(((((sin(theta)+1.0))*0.5)-brightness)*sign)*0.5;
+  scale=0.5000000000000001;
+  brightness+=scale*(((scale*((sin(theta)+1.0)))-brightness)*sign);
   if (brightness > 1.0)
     brightness=1.0;
   else
@@ -119,9 +122,9 @@ Export void Contrast(const int sign,Quantum *red,Quantum *green,Quantum *blue)
 %
 %  Method GenerateNoise adds noise to a pixel.
 %
-%  The format of the GenerateNoise routine is:
+%  The format of the GenerateNoise method is:
 %
-%      GenerateNoise(pixel,noise_type)
+%      Quantum GenerateNoise(const Quantum pixel,const NoiseType noise_type)
 %
 %  A description of each parameter follows:
 %
@@ -247,9 +250,10 @@ Export Quantum GenerateNoise(const Quantum pixel,const NoiseType noise_type)
 %  Method HSLTransform converts a (hue, saturation, luminosity) to a
 %  (red, green, blue) triple.
 %
-%  The format of the HSLTransformImage routine is:
+%  The format of the HSLTransformImage method is:
 %
-%      HSLTransform(hue,saturation,luminosity,red,green,blue)
+%      void HSLTransform(const double hue,const double saturation,
+%        const double luminosity,Quantum *red,Quantum *green,Quantum *blue)
 %
 %  A description of each parameter follows:
 %
@@ -322,9 +326,10 @@ Export void HSLTransform(const double hue,const double saturation,
 %  one of its eight of its surrounding pixels using a polarity and negative
 %  hull function.
 %
-%  The format of the Hull routine is:
+%  The format of the Hull method is:
 %
-%      Hull(x_offset,y_offset,polarity,columns,rows,f,g)
+%      void Hull(const int x_offset,const int y_offset,const int polarity,
+%        const unsigned int columns,const unsigned int rows,Quantum *f,Quantum *g)
 %
 %  A description of each parameter follows:
 %
@@ -448,9 +453,10 @@ Export void Hull(const int x_offset,const int y_offset,const int polarity,
 %  Method InterpolateColor applies bi-linear interpolation between a pixel and
 %  it's neighbors.
 %
-%  The format of the InterpolateColor routine is:
+%  The format of the InterpolateColor method is:
 %
-%      InterpolateColor(image,x_offset,y_offset)
+%      PixelPacket InterpolateColor(Image *image,const double x_offset,
+%        const double y_offset)
 %
 %  A description of each parameter follows:
 %
@@ -461,84 +467,84 @@ Export void Hull(const int x_offset,const int y_offset,const int polarity,
 %
 %
 */
-Export ColorPacket InterpolateColor(Image *image,const double x_offset,
+Export PixelPacket InterpolateColor(Image *image,const double x_offset,
   const double y_offset)
 {
-  ColorPacket
-    interpolated_pixel;
-
   double
     alpha,
-    beta,
+    beta;
+
+  PixelPacket
+    p,
+    q,
+    r,
+    s;
+
+  register double
     x,
     y;
 
-  register RunlengthPacket
-    *p,
-    *q,
-    *r,
-    *s;
-
-  RunlengthPacket
-    background_pixel;
-
   assert(image != (Image *) NULL);
-  if (image->packets != (image->columns*image->rows))
-    if (!UncondenseImage(image))
-      return(image->background_color);
-  if ((x_offset < -1) || (x_offset >= image->columns) ||
-      (y_offset < -1) || (y_offset >= image->rows))
-    return(image->background_color);
-  background_pixel.red=image->background_color.red;
-  background_pixel.green=image->background_color.green;
-  background_pixel.blue=image->background_color.blue;
-  background_pixel.index=image->background_color.index;
   x=x_offset;
   y=y_offset;
-  if ((x >= 0) && (y >= 0))
+  if ((x < -1) || (x >= image->columns) || (y < -1) || (y >= image->rows))
+    return(image->background_color);
+  if ((x >= 0) && (y >= 0) && (x < (image->columns-1)) && (y < (image->rows-1)))
     {
-      p=image->pixels+((int) y)*image->columns+(int) x;
-      q=p+1;
-      if ((x+1) >= image->columns)
-        q=(&background_pixel);
-      r=p+image->columns;
-      if ((y+1) >= image->rows)
-        r=(&background_pixel);
-      s=p+image->columns+1;
-      if (((x+1) >= image->columns) || ((y+1) >= image->rows))
-        s=(&background_pixel);
+      register PixelPacket
+        *t;
+
+      t=GetPixelCache(image,(int) x,(int) y,2,2);
+      if (t == (PixelPacket *) NULL)
+        return(image->background_color);
+      p=(*t++);
+      q=(*t++);
+      r=(*t++);
+      s=(*t++);
     }
   else
     {
-      p=(&background_pixel);
-      q=(&background_pixel);
-      r=image->pixels+(int) x;
-      s=r+1;
-      if ((x >= -1) && (x < 0))
+      register PixelPacket
+        *t;
+
+      p=image->background_color;
+      if ((x >= 0) && (y >= 0))
         {
-          q=image->pixels+(int) y*image->columns;
-          r=(&background_pixel);
-          s=q+(int) image->columns;
-          if ((y >= -1) && (y < 0))
-            {
-              q=(&background_pixel);
-              s=image->pixels;
-            }
+          t=GetPixelCache(image,(int) x,(int) y,1,1);
+          if (t != (PixelPacket *) NULL)
+            p=(*t);
+        }
+      q=image->background_color;
+      if (((x+1) < image->columns) && (y >= 0))
+        {
+          t=GetPixelCache(image,(int) x+1,(int) y,1,1);
+          if (t != (PixelPacket *) NULL)
+            q=(*t);
+        }
+      r=image->background_color;
+      if ((x >= 0) && ((y+1) < image->rows))
+        {
+          t=GetPixelCache(image,(int) x,(int) y+1,1,1);
+          if (t != (PixelPacket *) NULL)
+            r=(*t);
+        }
+      s=image->background_color;
+      if (((x+1) < image->columns) && ((y+1) < image->rows))
+        {
+          t=GetPixelCache(image,(int) x+1,(int) y+1,1,1);
+          if (t != (PixelPacket *) NULL)
+            s=(*t);
         }
     }
   x-=floor(x);
   y-=floor(y);
   alpha=1.0-x;
   beta=1.0-y;
-  interpolated_pixel.red=(Quantum)
-    (beta*(alpha*p->red+x*q->red)+y*(alpha*r->red+x*s->red));
-  interpolated_pixel.green=(Quantum)
-    (beta*(alpha*p->green+x*q->green)+y*(alpha*r->green+x*s->green));
-  interpolated_pixel.blue=(Quantum)
-    (beta*(alpha*p->blue+x*q->blue)+y*(alpha*r->blue+x*s->blue));
-  interpolated_pixel.index=(unsigned short)
-    (beta*(alpha*p->index+x*q->index)+y*(alpha*r->index+x*s->index));
-  return(interpolated_pixel);
+  p.red=beta*(alpha*p.red+x*q.red)+y*(alpha*r.red+x*s.red);
+  p.green=beta*(alpha*p.green+x*q.green)+y*(alpha*r.green+x*s.green);
+  p.blue=beta*(alpha*p.blue+x*q.blue)+y*(alpha*r.blue+x*s.blue);
+  p.opacity=beta*(alpha*p.opacity+x*q.opacity)+y*(alpha*r.opacity+x*s.opacity);
+  return(p);
 }
 
 /*
@@ -555,9 +561,10 @@ Export ColorPacket InterpolateColor(Image *image,const double x_offset,
 %  Method Modulate modulates the hue, saturation, and brightness of an
 %  image.
 %
-%  The format of the ModulateImage routine is:
+%  The format of the ModulateImage method is:
 %
-%      Modulate(percent_hue,percent_saturation,percent_brightness,red,green,
+%      void Modulate(double percent_hue,double percent_saturation,
+%        double percent_brightness,Quantum *red,Quantum *green,Quantum *blue)
 %        blue)
 %
 %  A description of each parameter follows:
@@ -575,7 +582,8 @@ Export void Modulate(double percent_hue,double percent_saturation,
   double
     brightness,
     hue,
-    saturation;
+    saturation,
+    scale;
 
   /*
     Increase or decrease color brightness, saturation, or hue.
@@ -584,13 +592,14 @@ Export void Modulate(double percent_hue,double percent_saturation,
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
   TransformHSL(*red,*green,*blue,&hue,&saturation,&brightness);
-  brightness+=percent_brightness/100.0;
+  scale=0.0100000000000001;
+  brightness+=scale*percent_brightness;
   if (brightness < 0.0)
     brightness=0.0;
   else
     if (brightness > 1.0)
       brightness=1.0;
-  saturation+=percent_saturation/100.0;
+  saturation+=scale*percent_saturation;
   if (saturation < 0.0)
     saturation=0.0;
   else
@@ -598,7 +607,7 @@ Export void Modulate(double percent_hue,double percent_saturation,
       saturation=1.0;
   if (hue != -1.0)
     {
-      hue+=percent_hue/100.0;
+      hue+=scale*percent_hue;
       if (hue < 0.0)
         hue+=1.0;
       else
@@ -622,9 +631,10 @@ Export void Modulate(double percent_hue,double percent_saturation,
 %  Method TransformHSL converts a (red, green, blue) to a (hue, saturation,
 %  luminosity) triple.
 %
-%  The format of the TransformHSL routine is:
+%  The format of the TransformHSL method is:
 %
-%      TransformHSL(red,green,blue,hue,saturation,luminosity)
+%      void TransformHSL(const Quantum red,const Quantum green,
+%        const Quantum blue,double *hue,double *saturation,double *luminosity)
 %
 %  A description of each parameter follows:
 %
@@ -688,9 +698,10 @@ Export void TransformHSL(const Quantum red,const Quantum green,
 %
 %  Method Upsample doubles the size of the image.
 %
-%  The format of the UpSample routine is:
+%  The format of the UpSample method is:
 %
-%      Upsample(width,height,scaled_width,pixels)
+%      void Upsample(const unsigned int width,const unsigned int height,
+%        const unsigned int scaled_width,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
@@ -752,9 +763,5 @@ Export void Upsample(const unsigned int width,const unsigned int height,
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
-  for (x=0; x < (int) width; x++)
-  {
-    *q++=(*p++);
-    *q++=(*p++);
-  }
+  (void) memcpy(q,p,2*width*sizeof(unsigned char));
 }
