@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -57,8 +58,12 @@
 #include "xwindows.h"
 #endif
 #if defined(HasTTF)
+#if defined(HAVE_FREETYPE_FREETYPE_H)
+#include "freetype/freetype.h"
+#else
 #include "freetype.h"
 #endif
+#endif
 
 /*
   Font declaration.
@@ -81,9 +86,9 @@ Export const char
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
-%  The format of the ReadLABELImage routine is:
+%  The format of the ReadLABELImage method is:
 %
-%      image=ReadLABELImage(image_info)
+%      Image *ReadLABELImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -177,7 +182,7 @@ static void GetFontInfo(TT_Face face,TT_Face_Properties *face_properties,
 }
 
 static void RenderGlyph(TT_Raster_Map *canvas,TT_Raster_Map *character,
-  TT_Glyph glyph,int x_off,int y_off,TT_Glyph_Metrics *glyph_metrics)
+  TT_Glyph glyph,int x_offset,int y_offset,TT_Glyph_Metrics *glyph_metrics)
 {
   int
     y;
@@ -204,14 +209,14 @@ static void RenderGlyph(TT_Raster_Map *canvas,TT_Raster_Map *character,
   /*
     Composite character on canvas.
   */
-  x_off=((glyph_metrics->bbox.xMin & -64)/64)+x_off;
-  y_off=(-(glyph_metrics->bbox.yMin & -64)/64)-y_off;
-  bounds.x1=x_off < 0 ? -x_off : 0;
-  bounds.y1=y_off < 0 ? -y_off : 0;
-  bounds.x2=(int) canvas->cols-x_off;
+  x_offset+=(glyph_metrics->bbox.xMin & -64)/64;
+  y_offset=(-(glyph_metrics->bbox.yMin & -64)/64)-y_offset;
+  bounds.x1=x_offset < 0 ? -x_offset : 0;
+  bounds.y1=y_offset < 0 ? -y_offset : 0;
+  bounds.x2=canvas->cols-x_offset;
   if (bounds.x2 > character->cols)
     bounds.x2=character->cols;
-  bounds.y2=(int) canvas->rows-y_off;
+  bounds.y2=canvas->rows-y_offset;
   if (bounds.y2 > character->rows)
     bounds.y2=character->rows;
   if (bounds.x1 >= bounds.x2)
@@ -219,8 +224,8 @@ static void RenderGlyph(TT_Raster_Map *canvas,TT_Raster_Map *character,
   for (y=(int) bounds.y1; y < (int) bounds.y2; y++)
   {
     p=((unsigned char *) character->bitmap)+y*character->cols+(int) bounds.x1;
-    q=((unsigned char *) canvas->bitmap)+(y+y_off)*canvas->cols+
-      (int) bounds.x1+x_off;
+    q=((unsigned char *) canvas->bitmap)+(y+y_offset)*canvas->cols+
+      (int) bounds.x1+x_offset;
     for (x=(int) bounds.x1; x < bounds.x2; x++)
       *q++|=(*p++);
   }
@@ -237,7 +242,7 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
     text[MaxTextExtent],
     page[MaxTextExtent];
 
-  ColorPacket
+  PixelPacket
     pen_color;
 
   FILE
@@ -250,20 +255,19 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
     *local_info;
 
   int
-    x,
     y;
 
   RectangleInfo
     crop_info;
 
   register int
-    i,
-    runlength;
+    x;
 
-  register RunlengthPacket
+  register PixelPacket
+    *p,
     *q;
 
-  RunlengthPacket
+  PixelPacket
     corner;
 
   /*
@@ -334,10 +338,6 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
       TT_UShort
         code;
 
-      unsigned int
-        height,
-        width;
-
       unsigned short
         encoding,
         platform,
@@ -435,7 +435,8 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
         }
       error|=TT_Set_Instance_Resolutions(instance,(unsigned short)
         image->x_resolution,(unsigned short) image->y_resolution);
-      error|=TT_Set_Instance_CharSize(instance,local_info->pointsize*64);
+      error|=
+        TT_Set_Instance_CharSize(instance,(int) (64.0*local_info->pointsize));
       if (error)
         ReaderExit(DelegateWarning,"Cannot initialize TTF instance",image);
       for (code=0; (int) code < (int) face_properties.num_CharMaps; code++)
@@ -484,20 +485,24 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
       }
       TT_Get_Face_Properties(face,&face_properties);
       TT_Get_Instance_Metrics(instance,&instance_metrics);
-      width=local_info->pointsize >> 1;
-      height=((int) (face_properties.horizontal->Ascender*
-        instance_metrics.y_ppem)/(int) face_properties.header->Units_Per_EM)-
-        ((int) (face_properties.horizontal->Descender*instance_metrics.y_ppem)/
-        (int) face_properties.header->Units_Per_EM)+1;
+      canvas.width=4;
       for (i=0; i < length; i++)
       {
         if (glyphs[unicode[i]].z == (TT_Glyph *) NULL)
           continue;
         TT_Get_Glyph_Metrics(glyphs[unicode[i]],&glyph_metrics);
-        width+=glyph_metrics.advance/64;
+        if (i == (length-1))
+          {
+            canvas.width+=
+              (glyph_metrics.bbox.xMin+glyph_metrics.bbox.xMax) >> 6;
+            continue;
+          }
+        canvas.width+=(glyph_metrics.advance >> 6)+1;
       }
-      canvas.rows=height;
-      canvas.width=(width+3) & -4;
+      canvas.width=(canvas.width+3) & -4;
+      canvas.rows=instance_metrics.y_ppem*(face_properties.horizontal->Ascender-
+        face_properties.horizontal->Descender)/
+        face_properties.header->Units_Per_EM;
       canvas.flow=TT_Flow_Down;
       canvas.cols=canvas.width;
       canvas.size=canvas.rows*canvas.width;
@@ -507,7 +512,7 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
       p=(unsigned char *) canvas.bitmap;
       for (i=0; i < canvas.size; i++)
         *p++=0;
-      character.rows=height;
+      character.rows=canvas.rows;
       character.width=(instance_metrics.x_ppem+32+3) & -4;
       character.flow=TT_Flow_Down;
       character.cols=character.width;
@@ -516,8 +521,8 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
       if (!character.bitmap)
         ReaderExit(DelegateWarning,"Memory allocation failed",image);
       x=0;
-      y=((int) -(face_properties.horizontal->Descender*instance_metrics.y_ppem)/
-        (int) face_properties.header->Units_Per_EM);
+      y=(-instance_metrics.y_ppem*face_properties.horizontal->Descender)/
+        face_properties.header->Units_Per_EM+1;
       for (i=0; i < length; i++)
       {
         if (glyphs[unicode[i]].z == (TT_Glyph *) NULL)
@@ -532,49 +537,53 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
       image->matte=True;
       image->columns=canvas.width;
       image->rows=canvas.rows;
-      image->packets=image->columns*image->rows;
-      image->pixels=(RunlengthPacket *)
-        AllocateMemory(image->packets*sizeof(RunlengthPacket));
-      if (image->pixels == (RunlengthPacket *) NULL)
-        ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
+      crop_info.width=0;
+      crop_info.height=image->rows;
       p=(unsigned char *) canvas.bitmap;
-      q=image->pixels;
-      x=0;
-      for (i=0; i < (int) image->packets; i++)
+      for (y=0; y < (int) image->rows; y++)
       {
-        q->red=XDownScale(pen_color.red);
-        q->green=XDownScale(pen_color.green);
-        q->blue=XDownScale(pen_color.blue);
-        if (local_info->antialias)
-          q->index=(int) (Opaque*Min(*p,4))/4;
-        else
-          q->index=(*p) > 1 ? Opaque : Transparent;
-        if (q->index == Transparent)
-          {
-            q->red=(~q->red);
-            q->green=(~q->green);
-            q->blue=(~q->blue);
-          }
-        q->length=0;
-        x++;
-        if (x == (int) image->columns)
-          {
-            if ((image->columns % 2) != 0)
-              p++;
-            x=0;
-          }
-        p++;
-        q++;
+        q=SetPixelCache(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (int) image->columns; x++)
+        {
+          q->red=pen_color.red;
+          q->green=pen_color.green;
+          q->blue=pen_color.blue;
+          if (local_info->antialias)
+            q->opacity=(int) (Opaque*Min(*p,4))/4;
+          else
+            q->opacity=(*p) > 1 ? Opaque : Transparent;
+          if (q->opacity != Transparent)
+            {
+              if (x > (int) crop_info.width)
+                crop_info.width=x;
+            }
+          else
+            {
+              q->red=(~q->red);
+              q->green=(~q->green);
+              q->blue=(~q->blue);
+            }
+          p++;
+          q++;
+        }
+        if (!SyncPixelCache(image))
+          break;
+        if ((image->columns % 2) != 0)
+          p++;
       }
+      (void) sprintf(geometry,"%ux%u+0+0",crop_info.width+1,crop_info.height);
+      TransformImage(&image,geometry,(char *) NULL);
       /*
         Free TrueType resources.
       */
-      FreeMemory((char *) canvas.bitmap);
-      FreeMemory((char *) character.bitmap);
+      FreeMemory(canvas.bitmap);
+      FreeMemory(character.bitmap);
       for (i=0; i < MaxGlyphs; i++)
         TT_Done_Glyph(glyphs[i]);
       FreeMemory(glyphs);
-      FreeMemory((char *) unicode);
+      FreeMemory(unicode);
       TT_Done_Instance(instance);
       TT_Close_Face(face);
       TT_Done_FreeType(engine);
@@ -604,7 +613,7 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
         *font_info;
 
       static XPixelInfo
-        pixel_info;
+        pixel;
 
       static XResourceInfo
         resource_info;
@@ -655,16 +664,16 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
               if (visual_info == (XVisualInfo *) NULL)
                 ReaderExit(XServerWarning,"Unable to get visual",image);
               map_info->colormap=(Colormap) NULL;
-              pixel_info.pixels=(unsigned long *) NULL;
-              pixel_info.gamma_map=(XColor *) NULL;
+              pixel.pixels=(unsigned long *) NULL;
+              pixel.gamma_map=(XColor *) NULL;
               /*
                 Initialize Standard Colormap info.
               */
               XGetMapInfo(visual_info,XDefaultColormap(display,
                 visual_info->screen),map_info);
-              XGetPixelInfo(display,visual_info,map_info,&resource_info,
-                (Image *) NULL,&pixel_info);
-              pixel_info.annotate_context=
+              XGetPixelPacket(display,visual_info,map_info,&resource_info,
+                (Image *) NULL,&pixel);
+              pixel.annotate_context=
                 XDefaultGC(display,visual_info->screen);
               /*
                 Initialize font info.
@@ -676,7 +685,7 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
                   (visual_info == (XVisualInfo *) NULL) ||
                   (font_info == (XFontStruct *) NULL))
                 {
-                  XFreeResources(display,visual_info,map_info,&pixel_info,
+                  XFreeResources(display,visual_info,map_info,&pixel,
                     font_info,&resource_info,(XWindowInfo *) NULL);
                   display=(Display *) NULL;
                 }
@@ -722,32 +731,34 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
       /*
         Render label with a X11 server font.
       */
+      image->matte=True;
       image->columns=annotate_info.width;
       image->rows=annotate_info.height;
-      image->packets=image->columns*image->rows;
-      image->pixels=(RunlengthPacket *)
-        AllocateMemory(image->packets*sizeof(RunlengthPacket));
-      if (image->pixels == (RunlengthPacket *) NULL)
-        ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
       image->background_color=image->border_color;
-      status=XAnnotateImage(display,&pixel_info,&annotate_info,image);
+      status=XAnnotateImage(display,&pixel,&annotate_info,image);
       if (status == 0)
         ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
-      image->matte=True;
-      q=image->pixels;
-      for (i=0; i < (int) image->packets; i++)
+      for (y=0; y < (int) image->rows; y++)
       {
-        q->index=Intensity(*q);
-        q->red=XDownScale(pen_color.red);
-        q->green=XDownScale(pen_color.green);
-        q->blue=XDownScale(pen_color.blue);
-        if (q->index == Transparent)
-          {
-            q->red=(~q->red);
-            q->green=(~q->green);
-            q->blue=(~q->blue);
-          }
-        q++;
+        q=GetPixelCache(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (int) image->columns; x++)
+        {
+          q->opacity=Intensity(*q);
+          q->red=pen_color.red;
+          q->green=pen_color.green;
+          q->blue=pen_color.blue;
+          if (q->opacity == Transparent)
+            {
+              q->red=(~q->red);
+              q->green=(~q->green);
+              q->blue=(~q->blue);
+            }
+          q++;
+        }
+        if (!SyncPixelCache(image))
+          break;
       }
       DestroyImageInfo(local_info);
       return(image);
@@ -760,8 +771,9 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
     Render label with a Postscript font.
   */
   local_info->density=(char *) NULL;
-  (void) sprintf(page,"%ux%u+0+0!",local_info->pointsize*Extent(text),
-    local_info->pointsize << 1);
+  (void) sprintf(page,"%ux%u+0+0!",
+    (unsigned int) ceil(local_info->pointsize*Extent(text)),
+    (unsigned int) ceil(2*local_info->pointsize));
   (void) CloneString(&local_info->page,page);
   TemporaryFilename(filename);
   file=fopen(filename,WriteBinaryType);
@@ -777,13 +789,14 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
     "  /Encoding ISOLatin1Encoding def currentdict end definefont pop\n");
   (void) fprintf(file,"} bind def\n");
   (void) fprintf(file,
-    "/%.1024s-ISO dup /%.1024s ReencodeFont findfont %u scalefont setfont\n",
+    "/%.1024s-ISO dup /%.1024s ReencodeFont findfont %f scalefont setfont\n",
     local_info->font,local_info->font,local_info->pointsize);
   (void) fprintf(file,"0.0 0.0 0.0 setrgbcolor\n");
   (void) fprintf(file,"0 0 %u %u rectfill\n",
-    local_info->pointsize*Extent(text),local_info->pointsize << 1);
+    (unsigned int) ceil(local_info->pointsize*Extent(text)),
+    (unsigned int) ceil(2*local_info->pointsize));
   (void) fprintf(file,"1.0 1.0 1.0 setrgbcolor\n");
-  (void) fprintf(file,"0 %u moveto (%.1024s) show\n",local_info->pointsize,
+  (void) fprintf(file,"0 %f moveto (%.1024s) show\n",local_info->pointsize,
     EscapeParenthesis(text));
   (void) fprintf(file,"showpage\n");
   (void) fclose(file);
@@ -795,52 +808,53 @@ Export Image *ReadLABELImage(const ImageInfo *image_info)
     Set bounding box to the image dimensions.
   */
   crop_info.width=0;
-  crop_info.height=local_info->pointsize;
+  crop_info.height=ceil(local_info->pointsize);
   crop_info.x=0;
-  crop_info.y=local_info->pointsize >> 2;
+  crop_info.y=local_info->pointsize/4;
   DestroyImageInfo(local_info);
   if (image == (Image *) NULL)
     return(image);
-  q=image->pixels;
-  runlength=q->length+1;
   corner.red=0;
   corner.green=0;
   corner.blue=0;
   for (y=0; y < (int) image->rows; y++)
   {
+    p=GetPixelCache(image,0,y,image->columns,1);
+    if (p == (PixelPacket *) NULL)
+      break;
     for (x=0; x < (int) image->columns; x++)
     {
-      if (runlength != 0)
-        runlength--;
-      else
-        {
-          q++;
-          runlength=q->length;
-        }
-      if (!ColorMatch(*q,corner,0))
+      if (!ColorMatch(*p,corner,0))
         if (x > (int) crop_info.width)
           crop_info.width=x;
+      p++;
     }
   }
-  crop_info.width++;
-  (void) sprintf(geometry,"%ux%u%+d%+d",crop_info.width,crop_info.height,
+  (void) sprintf(geometry,"%ux%u%+d%+d",crop_info.width+1,crop_info.height,
     crop_info.x,crop_info.y);
   TransformImage(&image,geometry,(char *) NULL);
   image->matte=True;
-  q=image->pixels;
-  for (i=0; i < (int) image->packets; i++)
+  for (y=0; y < (int) image->rows; y++)
   {
-    q->index=Intensity(*q);
-    q->red=XDownScale(pen_color.red);
-    q->green=XDownScale(pen_color.green);
-    q->blue=XDownScale(pen_color.blue);
-    if (q->index == Transparent)
-      {
-        q->red=(~q->red);
-        q->green=(~q->green);
-        q->blue=(~q->blue);
-      }
-    q++;
+    q=GetPixelCache(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    for (x=0; x < (int) image->columns; x++)
+    {
+      q->opacity=Intensity(*q);
+      q->red=pen_color.red;
+      q->green=pen_color.green;
+      q->blue=pen_color.blue;
+      if (q->opacity == Transparent)
+        {
+          q->red=(~q->red);
+          q->green=(~q->green);
+          q->blue=(~q->blue);
+        }
+      q++;
+    }
+    if (!SyncPixelCache(image))
+      break;
   }
   return(image);
 }
