@@ -18,7 +18,8 @@
 %                                September 1996                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,22 +34,23 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  The directory routines are strongly based on similar routines written
+%  The directory methods are strongly based on similar methods written
 %  by Steve Summit, scs@eskimo.com.  The Ghostscript launch code is strongly
 %  based on Dave Schooley's Mac Gnuplot and provided by
-%  schindall@wave14i.nrl.navy.mil.
+%  schindall@wave14i.nrl.navy.mil.  Mac-centric improvements provided by
+%  leonardr@digapp.com.
 %
 %
 */
@@ -115,9 +117,9 @@ static pascal void
 %
 %  Method BottleneckTest intercepts any compressed images.
 %
-%  The format of the BottleneckTest routine is:
+%  The format of the BottleneckTest method is:
 %
-%      BottleneckTest(picture,codec,depth,colormap_id)
+%      int ImageFormatConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
@@ -285,7 +287,7 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
 %  Method closedir closes the named directory stream and frees the DIR
 %  structure.
 %
-%  The format of the closedir routine is:
+%  The format of the closedir method is:
 %
 %      closedir(entry)
 %
@@ -295,7 +297,7 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
 %
 %
 */
-void closedir(DIR *entry)
+Export void closedir(DIR *entry)
 {
   assert(entry != (DIR *) NULL);
   FreeMemory((void *) entry);
@@ -314,7 +316,7 @@ void closedir(DIR *entry)
 %
 %  Method Exit exits the process.
 %
-%  The format of the Exit routine is:
+%  The format of the Exit method is:
 %
 %      Exit(status)
 %
@@ -325,7 +327,7 @@ void closedir(DIR *entry)
 %
 %
 */
-int Exit(int status)
+Export int Exit(int status)
 {
   (void) fprintf(stderr,"Select File->Quit to exit.\n");
   exit(status);
@@ -345,7 +347,7 @@ int Exit(int status)
 %
 %  Method FilenameToFSSpec sets the file type of an image.
 %
-%  The format of the FilenameToFSSpec routine is:
+%  The format of the FilenameToFSSpec method is:
 %
 %      FilenameToFSSpec(filename,fsspec)
 %
@@ -357,7 +359,7 @@ int Exit(int status)
 %
 %
 */
-void pascal FilenameToFSSpec(const char *filename,FSSpec *fsspec)
+Export void pascal FilenameToFSSpec(const char *filename,FSSpec *fsspec)
 {
   Str255
     name;
@@ -384,7 +386,7 @@ void pascal FilenameToFSSpec(const char *filename,FSSpec *fsspec)
 %
 %  Contributed by Mark Gavin of Digital Applications, Inc.
 %
-%  The format of the ImageFormatConflict routine is:
+%  The format of the ImageFormatConflict method is:
 %
 %      status=ImageFormatConflict(magick)
 %
@@ -405,13 +407,13 @@ Export int ImageFormatConflict(const char *magick)
   OSErr
     status,
     volume;
-    
+
   short
     index;
-  
+
   Str255
     volume_name;
-  
+
   StringPtr
     p;
 
@@ -442,9 +444,10 @@ Export int ImageFormatConflict(const char *magick)
 %  Method MACErrorHandler displays an error message and then terminates
 %  the program.
 %
-%  The format of the MACErrorHandler routine is:
+%  The format of the MACErrorHandler method is:
 %
-%      MACErrorHandler(error,message,qualifier)
+%      void MACErrorHandler(const ErrorType error,const char *message,
+%        const char *qualifier)
 %
 %  A description of each parameter follows:
 %
@@ -457,7 +460,7 @@ Export int ImageFormatConflict(const char *magick)
 %
 %
 */
-Export void MACErrorHandler(const unsigned int error,const char *message,
+Export void MACErrorHandler(const ErrorType error,const char *message,
   const char *qualifier)
 {
   char
@@ -742,9 +745,9 @@ static OSErr MacGSSetWorkingFolder(char *directory)
 %   Method MACSystemCommand executes the specified command and waits until it
 %   terminates.  The returned value is the exit status of the command.
 %
-%  The format of the MACSystemCommand routine is:
+%  The format of the MACSystemCommand method is:
 %
-%      MACSystemCommand(command)
+%      int MACSystemCommand(const char * command)
 %
 %  A description of each parameter follows:
 %
@@ -752,7 +755,7 @@ static OSErr MacGSSetWorkingFolder(char *directory)
 %
 %
 */
-int MACSystemCommand(const char * command)
+Export int MACSystemCommand(const char * command)
 {
   /*
     We only know how to launch Ghostscript.
@@ -775,9 +778,10 @@ int MACSystemCommand(const char * command)
 %
 %  Method MACWarningHandler displays a warning message.
 %
-%  The format of the MACWarningHandler routine is:
+%  The format of the MACWarningHandler method is:
 %
-+      MACWarningHandler(warning,message,qualifier)
++      void MACWarningHandler(const WarningType warning,const char *message,
+%        const char *qualifier)
 %
 %  A description of each parameter follows:
 %
@@ -790,11 +794,11 @@ int MACSystemCommand(const char * command)
 %
 %
 */
-Export void MACWarningHandler(const unsigned int warning,const char *message,
+Export void MACWarningHandler(const WarningType warning,const char *message,
   const char *qualifier)
 {
   char
-    buffer[2048];
+    buffer[1664];
 
   if (message == (char *) NULL)
     return;
@@ -821,9 +825,9 @@ Export void MACWarningHandler(const unsigned int warning,const char *message,
 %  Method opendir opens the directory named by filename and associates a
 %  directory stream with it.
 %
-%  The format of the opendir routine is:
+%  The format of the opendir method is:
 %
-%      opendir(entry)
+%      Export DIR *opendir(char *path)
 %
 %  A description of each parameter follows:
 %
@@ -831,10 +835,10 @@ Export void MACWarningHandler(const unsigned int warning,const char *message,
 %
 %
 */
-DIR *opendir(char *path)
+Export DIR *opendir(char *path)
 {
   char
-    pathname[2048];
+    pathname[1664];
 
   CInfoPBRec
     search_info;
@@ -882,7 +886,7 @@ DIR *opendir(char *path)
 %  Method ProcessPendingEvents processes any pending events.  This prevents
 %  ImageMagick from monopolizing the processor.
 %
-%  The format of the ProcessPendingEvents routine is:
+%  The format of the ProcessPendingEvents method is:
 %
 %      ProcessPendingEvents(text)
 %
@@ -894,7 +898,7 @@ DIR *opendir(char *path)
 */
 Export void ProcessPendingEvents(const char *text)
 {
-  static char
+  static const char
     *mark = (char *) NULL;
 
   EventRecord
@@ -926,7 +930,7 @@ Export void ProcessPendingEvents(const char *text)
 %
 %  The format of the readdir
 %
-%      readdir(entry)
+%      struct dirent *readdir(DIR *entry)
 %
 %  A description of each parameter follows:
 %
@@ -934,7 +938,7 @@ Export void ProcessPendingEvents(const char *text)
 %
 %
 */
-struct dirent *readdir(DIR *entry)
+Export struct dirent *readdir(DIR *entry)
 {
   CInfoPBRec
     search_info;
@@ -946,7 +950,7 @@ struct dirent *readdir(DIR *entry)
     dir_entry;
 
   static unsigned char
-    pathname[2048];
+    pathname[1664];
 
   if (entry == (DIR *) NULL)
     return((struct dirent *) NULL);
@@ -979,16 +983,16 @@ struct dirent *readdir(DIR *entry)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Method ReadPICTImage reads an Apple Macintosh QuickDraw/PICT image
-%  file using MacOS QuickDraw routines and returns it.  It allocates the
+%  file using MacOS QuickDraw methods and returns it.  It allocates the
 %  memory necessary for the new Image structure and returns a pointer to
 %  the new image.
 %
-%  This routine was written and contributed by spd@daphne.cps.unizar.es
+%  This method was written and contributed by spd@daphne.cps.unizar.es
 %  (feel free to copy and use it as you want. No warranty).
 %
-%  The format of the ReadPICTImage routine is:
+%  The format of the ReadPICTImage method is:
 %
-%      image=ReadPICTImage(image_info)
+%      void ProcessPendingEvents(const char *text)
 %
 %  A description of each parameter follows:
 %
@@ -1000,7 +1004,7 @@ struct dirent *readdir(DIR *entry)
 %
 %
 */
-Image *ReadPICTImage(const ImageInfo *image_info)
+Export Image *ReadPICTImage(const ImageInfo *image_info)
 {
 #define LoadImageText  "  Loading image...  "
 #define PICTHeaderSize    512
@@ -1023,7 +1027,7 @@ Image *ReadPICTImage(const ImageInfo *image_info)
 
   Image
     *image;
-    
+
   int
     depth,
     status,
@@ -1035,10 +1039,8 @@ Image *ReadPICTImage(const ImageInfo *image_info)
   PictInfo
     picture_info;
 
-  Quantum
-    blue,
-    green,
-    red;
+  QDErr
+    theErr = noErr;
 
   Rect
     rectangle;
@@ -1046,7 +1048,7 @@ Image *ReadPICTImage(const ImageInfo *image_info)
   register int
     x;
 
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
   RGBColor
@@ -1055,12 +1057,6 @@ Image *ReadPICTImage(const ImageInfo *image_info)
   short
     colormap_id;
 
-  unsigned int
-    packets;
-
-  unsigned short
-    index;
-  
   /*
     Allocate image structure.
   */
@@ -1081,22 +1077,23 @@ Image *ReadPICTImage(const ImageInfo *image_info)
   (void) ReadBlob(image,PICTHeaderSize,*(char **) picture_handle);
   status=
     ReadBlob(image,image->filesize-PICTHeaderSize,*(char **) picture_handle);
-  HUnlock((Handle) picture_handle);
   if (status == False)
     {
       DisposeHandle((Handle) picture_handle);
       PrematureExit(CorruptImageWarning,"Unable to read image data",image);
     }
   GetGWorld(&port,&device);
-  status=NewGWorld(&graphic_world,0,&(**picture_handle).picFrame,nil,nil,0);
-  if (status != noErr)
+  theErr=NewGWorld(&graphic_world,0,&(**picture_handle).picFrame,nil,nil,
+    useTempMem|keepLocal);
+  if (theErr != noErr && graphic_world == nil)
     {
       DisposeHandle((Handle) picture_handle);
       PrematureExit(ResourceLimitWarning,"Unable to allocate memory",image);
     }
+  HUnlock((Handle) picture_handle);
   SetGWorld(graphic_world,nil);
-  status=GetPictInfo(picture_handle,&picture_info,0,1,systemMethod,0);
-  if (status != noErr)
+  theErr=GetPictInfo(picture_handle,&picture_info,0,1,systemMethod,0);
+  if (theErr != noErr)
     {
       DisposeGWorld(graphic_world);
       DisposeHandle((Handle) picture_handle);
@@ -1132,17 +1129,6 @@ Image *ReadPICTImage(const ImageInfo *image_info)
   image->units=PixelsPerInchResolution;
   image->columns=picture_info.sourceRect.right-picture_info.sourceRect.left;
   image->rows=picture_info.sourceRect.bottom-picture_info.sourceRect.top;
-  image->packets=0;
-  packets=Max((image->columns*image->rows+2) >> 2,1);
-  image->pixels=(RunlengthPacket *)
-    AllocateMemory(packets*sizeof(RunlengthPacket));
-  if (image->pixels == (RunlengthPacket *) NULL)
-    {
-      if (picture_info.theColorTable != nil)
-        DisposeHandle((Handle) picture_info.theColorTable); 
-      DisposeGWorld(graphic_world);
-      PrematureExit(ResourceLimitWarning,"Memory allocation failed",image);
-    }
   if ((depth <= 8) && ((*(picture_info.theColorTable))->ctSize != 0))
     {
       /*
@@ -1150,12 +1136,12 @@ Image *ReadPICTImage(const ImageInfo *image_info)
       */
       image->class=PseudoClass;
       image->colors=(*(picture_info.theColorTable))->ctSize;
-      image->colormap=(ColorPacket *)
-        AllocateMemory(image->colors*sizeof(ColorPacket));
-      if (image->colormap == (ColorPacket *) NULL)
+      image->colormap=(PixelPacket *)
+        AllocateMemory(image->colors*sizeof(PixelPacket));
+      if (image->colormap == (PixelPacket *) NULL)
         {
           if (picture_info.theColorTable != nil)
-            DisposeHandle((Handle) picture_info.theColorTable); 
+            DisposeHandle((Handle) picture_info.theColorTable);
           DisposeGWorld(graphic_world);
           DisposeHandle((Handle) picture_handle);
           PrematureExit(ResourceLimitWarning,"Unable to allocate memory",image);
@@ -1183,56 +1169,25 @@ Image *ReadPICTImage(const ImageInfo *image_info)
     }
   DisposeHandle((Handle) picture_handle);
   /*
-    Convert PICT pixels to runlength-encoded packets.
+    Convert PICT pixels to pixel packets.
   */
-  red=0;
-  green=0;
-  blue=0;
-  index=0;
-  q=image->pixels;
-  SetRunlengthEncoder(q);
   for (y=0; y < image->rows; y++)
   {
+    q=SetPixelCache(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
     for (x=0; x < image->columns; x++)
     {
       GetCPixel(x,y,&Pixel);
-      red=UpScale(Pixel.red & 0xff);
-      green=UpScale(Pixel.green & 0xff);
-      blue=UpScale(Pixel.blue & 0xff);
+      q->red=UpScale(Pixel.red & 0xff);
+      q->green=UpScale(Pixel.green & 0xff);
+      q->blue=UpScale(Pixel.blue & 0xff);
       if (image->class == PseudoClass)
-        index=(unsigned short) Color2Index(&Pixel);
-      if ((red == q->red) && (green == q->green) && (blue == q->blue) &&
-          (index == q->index) && ((int) q->length < MaxRunlength))
-        q->length++;
-      else
-        {
-          if (image->packets != 0)
-            q++;
-          image->packets++;
-          if (image->packets == packets)
-            {
-              packets<<=1;
-              image->pixels=(RunlengthPacket *) ReallocateMemory((char *)
-                image->pixels,packets*sizeof(RunlengthPacket));
-              if (image->pixels == (RunlengthPacket *) NULL)
-                {
-                  UnlockPixels(graphic_world->portPixMap);
-                  SetGWorld(port,device);
-                  if (picture_info.theColorTable != nil)
-                    DisposeHandle((Handle) picture_info.theColorTable); 
-                  DisposeGWorld(graphic_world);
-                  PrematureExit(ResourceLimitWarning,
-                    "Memory allocation failed",image);
-                }
-              q=image->pixels+image->packets-1;
-            }
-          q->red=red;
-          q->green=green;
-          q->blue=blue;
-          q->index=index;
-          q->length=0;
-        }
+        image->indexes[x]=(unsigned short) Color2Index(&Pixel);
+      q++;
     }
+    if (!SyncPixelCache(image))
+      break;
     if (QuantumTick(y,image->rows))
       ProgressMonitor(LoadImageText,y,image->rows);
   }
@@ -1347,9 +1302,9 @@ static Boolean SearchForFile(OSType creator_type,OSType file_type,FSSpec *file,
 %  Method seekdir sets the position of the next readdir() operation
 %  on the directory stream.
 %
-%  The format of the seekdir routine is:
+%  The format of the seekdir method is:
 %
-%      seekdir(entry,position)
+%      void seekdir(DIR *entry,long position)
 %
 %  A description of each parameter follows:
 %
@@ -1361,7 +1316,7 @@ static Boolean SearchForFile(OSType creator_type,OSType file_type,FSSpec *file,
 %
 %
 */
-void seekdir(DIR *entry,long position)
+Export void seekdir(DIR *entry,long position)
 {
   assert(entry != (DIR *) NULL);
   entry->d_index=position;
@@ -1380,9 +1335,10 @@ void seekdir(DIR *entry,long position)
 %
 %  Method SetApplicationType sets the file type of an image.
 %
-%  The format of the SetApplicationType routine is:
+%  The format of the SetApplicationType method is:
 %
-%      SetApplicationType(filename,magick,application)
+%      void SetApplicationType(const char *filename,const char *magick,
+%        OSType application)
 %
 %  A description of each parameter follows:
 %
@@ -1394,7 +1350,7 @@ void seekdir(DIR *entry,long position)
 %
 %
 */
-void SetApplicationType(const char *filename,const char *magick,
+Export void SetApplicationType(const char *filename,const char *magick,
   OSType application)
 {
   OSType
@@ -1425,9 +1381,9 @@ void SetApplicationType(const char *filename,const char *magick,
 %   Method telldir returns the current location associated  with  the
 %   named directory stream.
 %
-%  The format of the telldir routine is:
+%  The format of the telldir method is:
 %
-%      telldir(entry)
+%      telldir(DIR *entry)
 %
 %  A description of each parameter follows:
 %
@@ -1435,7 +1391,7 @@ void SetApplicationType(const char *filename,const char *magick,
 %
 %
 */
-long telldir(DIR *entry)
+Export long telldir(DIR *entry)
 {
   return(entry->d_index);
 }
