@@ -4,70 +4,84 @@
 #ifndef _MAGICK_H
 #define _MAGICK_H
 
-#if defined(sun)
 #define __EXTENSIONS__  1
-#endif
-
-#if defined(__hpux)
-#define _HPUX_SOURCE  1
-#endif
-
-#if defined(vms)
-#define _POSIX_C_SOURCE  1
-#endif
+#define _POSIX_C_SOURCE  199506L
+#define _XOPEN_SOURCE  500
+#define _XOPEN_SOURCE_EXTENDED  1
 
 /*
   System include declarations.
 */
 #if defined(__cplusplus) || defined(c_plusplus)
-#include <cstdio>
-#include <cstdlib>
-#include <cstdarg>
-#include <cstring>
+#  include <cstdio>
+#  include <cstdlib>
+#  include <cstdarg>
+#  include <cstring>
 #else
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
+#  include <stdio.h>
+#  include <stdlib.h>
+#  include <stdarg.h>
+#  include <string.h>
 #endif
+
+#if defined _FILE_OFFSET_BITS && _FILE_OFFSET_BITS == 64
+#define fseek  fseeko
+#define ftell  ftello
+#endif
+
 #if defined(_VISUALC_)
-#include <direct.h>
+#  include <direct.h>
 #else
-#include <unistd.h>
+#  include <unistd.h>
 #endif
+
 #include <ctype.h>
 #include <signal.h>
 #include <locale.h>
 #include <errno.h>
 #include <math.h>
-#include <assert.h>
-#if !defined(__OPENNT)
+#include <limits.h>
 #include <time.h>
-#else
-#include <sys/time.h>
+#include <assert.h>
+
+#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+#  include <sys/types.h>
+#  include <sys/stat.h>
+#  include <sys/time.h>
+#  include <sys/times.h>
+#if defined(HAVE_CONFIG_H)
+#  include "magick/config.h"
 #endif
-#if !defined(__MWERKS__)
-#include <sys/types.h>
-#include <sys/stat.h>
-#else
-#include <SIOUX.h>
-#include <console.h>
-#include <unix.h>
-#include <types.h>
-#include <stat.h>
+#if defined(HAVE_MMAP)
+#  include <sys/mman.h>
 #endif
+#if defined(HasPTHREADS)
+#  include <pthread.h>
+#endif
+#if defined(sysv)
+#  include <sys/poll.h>
+#endif
+#  include "magick/api.h"
+#else
+#if defined(WIN32)
+#  include <sys/types.h>
+#  include <sys/stat.h>
+#else
+#  include <types.h>
+#  include <stat.h>
+#endif
+#if defined(macintosh)
+#  include <SIOUX.h>
+#  include <console.h>
+#  include <unix.h>
+#endif
+#  include "api.h"
+#endif
+#include <fcntl.h>
 
 /*
   ImageMagick API headers
 */
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
-#include <magick/api.h>
-#else
-#include "api.h"
-#endif
-
-#undef index
-
 #if defined(macintosh)
 #define HasJPEG
 #define HasLZW
@@ -77,17 +91,6 @@
 #define HasZLIB
 #endif
 
-#if defined(WIN32)
-#define HasJBIG
-#define HasJPEG
-#define HasLZW
-#define HasPNG
-#define HasTIFF
-#define HasTTF
-#define HasX11
-#define HasZLIB
-#endif
-
 #if defined(VMS)
 #define HasJPEG
 #define HasLZW
@@ -98,4 +101,20 @@
 #define HasZLIB
 #endif
 
+#if defined(WIN32)
+#define HasJBIG
+#define HasJPEG
+#define HasLZW
+#define HasPNG
+#define HasTIFF
+#define HasTTF
+#define HasX11
+#define HasZLIB
+#define HasBZLIB
+#define HasHDF
+#define HAVE_MMAP
+#endif
+
+#undef index
+
 #endif
