@@ -18,7 +18,8 @@
 %                               December 1995                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -72,12 +73,13 @@ static MonitorHandler
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ProgressMonitor calls the monitor handler routines with a text
+%  Method ProgressMonitor calls the monitor handler methods with a text
 %  string that describes the task and a measure of completion.
 %
-%  The format of the ProgressMonitor routine is:
+%  The format of the ProgressMonitor method is:
 %
-%      ProgressMonitor(text,quantum,span)
+%      void ProgressMonitor(const char *text,const off_t quantum,
+%        const off_t span)
 %
 %  A description of each parameter follows:
 %
@@ -88,8 +90,8 @@ static MonitorHandler
 %
 %
 */
-Export void ProgressMonitor(const char *text,const unsigned int quantum,
-  const unsigned int span)
+Export void ProgressMonitor(const char *text,const off_t quantum,
+  const off_t span)
 {
   ProcessPendingEvents(text);
   if (monitor_handler != (MonitorHandler) NULL)
@@ -107,16 +109,16 @@ Export void ProgressMonitor(const char *text,const unsigned int quantum,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method SetMonitorHandler sets the monitor handler to the specified routine
+%  Method SetMonitorHandler sets the monitor handler to the specified method
 %  and returns the previous monitor handler.
 %
-%  The format of the SetMonitorHandler routine is:
+%  The format of the SetMonitorHandler method is:
 %
-%      previous_handler=SetMonitorHandler(handler)
+%      MonitorHandler SetMonitorHandler(MonitorHandler handler)
 %
 %  A description of each parameter follows:
 %
-%    o handler: Specifies a pointer to a routine to handle monitors.
+%    o handler: Specifies a pointer to a method to handle monitors.
 %
 %
 */
