@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -83,34 +84,25 @@ Export void DestroyMontageInfo(MontageInfo *montage_info)
 {
   assert(montage_info != (MontageInfo *) NULL);
   if (montage_info->geometry != (char *) NULL)
-    FreeMemory((char *) montage_info->geometry);
+    FreeMemory(montage_info->geometry);
   montage_info->geometry=(char *) NULL;
   if (montage_info->tile != (char *) NULL)
-    FreeMemory((char *) montage_info->tile);
+    FreeMemory(montage_info->tile);
   montage_info->tile=(char *) NULL;
-  if (montage_info->background_color != (char *) NULL)
-    FreeMemory((char *) montage_info->background_color);
-  montage_info->background_color=(char *) NULL;
-  if (montage_info->border_color != (char *) NULL)
-    FreeMemory((char *) montage_info->border_color);
-  montage_info->border_color=(char *) NULL;
-  if (montage_info->matte_color != (char *) NULL)
-    FreeMemory((char *) montage_info->matte_color);
-  montage_info->matte_color=(char *) NULL;
   if (montage_info->title != (char *) NULL)
-    FreeMemory((char *) montage_info->title);
+    FreeMemory(montage_info->title);
   montage_info->title=(char *) NULL;
   if (montage_info->frame != (char *) NULL)
-    FreeMemory((char *) montage_info->frame);
+    FreeMemory(montage_info->frame);
   montage_info->frame=(char *) NULL;
   if (montage_info->texture != (char *) NULL)
-    FreeMemory((char *) montage_info->texture);
+    FreeMemory(montage_info->texture);
   montage_info->texture=(char *) NULL;
   if (montage_info->pen != (char *) NULL)
-    FreeMemory((char *) montage_info->pen);
+    FreeMemory(montage_info->pen);
   montage_info->pen=(char *) NULL;
   if (montage_info->font != (char *) NULL)
-    FreeMemory((char *) montage_info->font);
+    FreeMemory(montage_info->font);
   montage_info->font=(char *) NULL;
 }
 
@@ -143,15 +135,15 @@ Export void GetMontageInfo(MontageInfo *montage_info)
   *montage_info->filename='\0';
   montage_info->geometry=AllocateString(DefaultTileGeometry);
   montage_info->tile=AllocateString("6x4");
-  montage_info->background_color=AllocateString("#c0c0c0");
-  montage_info->border_color=(char *) NULL;
-  montage_info->matte_color=AllocateString("#bdbdbd");
+  (void) QueryColorDatabase("#c0c0c0",&montage_info->background_color);
+  (void) QueryColorDatabase(BorderColor,&montage_info->border_color);
+  (void) QueryColorDatabase("#bdbdbd",&montage_info->matte_color);
   montage_info->title=(char *) NULL;
   montage_info->frame=(char *) NULL;
   montage_info->texture=(char *) NULL;
   montage_info->pen=(char *) NULL;
   montage_info->font=(char *) NULL;
-  montage_info->pointsize=atoi(DefaultPointSize);
+  montage_info->pointsize=atof(DefaultPointSize);
   montage_info->border_width=0;
   montage_info->gravity=CenterGravity;
   montage_info->shadow=False;
@@ -249,7 +241,7 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
 #define TileImageText  "  Creating image tiles...  "
 
   AnnotateInfo
-    annotate_info;
+    *annotate_info;
 
   char
     geometry[MaxTextExtent];
@@ -280,8 +272,8 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
   register int
     i;
 
-  register RunlengthPacket
-    *p;
+  register PixelPacket
+    *q;
 
   RectangleInfo
     bounding_box,
@@ -330,11 +322,9 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
     height=image_list[tile]->rows;
     x=0;
     y=0;
-    image_list[tile]->orphan=True;
     (void) ParseImageGeometry(montage_info->geometry,&x,&y,&width,&height);
     image_list[tile]->orphan=True;
     tiled_image=ZoomImage(image_list[tile],width,height);
-    image_list[tile]->orphan=False;
     if (tiled_image == (Image *) NULL)
       {
         for (i=0; i < (int) tile; i++)
@@ -443,11 +433,10 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
   (void) CloneString(&local_info->pen,montage_info->pen);
   (void) CloneString(&local_info->font,montage_info->font);
   local_info->pointsize=montage_info->pointsize;
-  (void) CloneString(&local_info->background_color,
-    montage_info->background_color);
-  (void) CloneString(&local_info->border_color,montage_info->border_color);
-  GetAnnotateInfo(local_info,&annotate_info);
-  annotate_info.gravity=NorthGravity;
+  local_info->background_color=montage_info->background_color;
+  local_info->border_color=montage_info->border_color;
+  annotate_info=CloneAnnotateInfo(local_info,(AnnotateInfo *) NULL);
+  annotate_info->gravity=NorthGravity;
   texture=(Image *) NULL;
   if (montage_info->texture != (char *) NULL)
     {
@@ -457,7 +446,7 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
   /*
     Initialize font info.
   */
-  font_height=annotate_info.bounds.height;
+  font_height=annotate_info->bounds.height;
   FormatLabel(local_info,montage_info->title,((tile_info.width+
     (border_width << 1))*Min(number_images,tiles_per_column)) >> 1,
     &font_height);
@@ -525,16 +514,7 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
     (void) strcpy(montage_image->filename,montage_info->filename);
     montage_image->columns=bounding_box.width;
     montage_image->rows=bounding_box.height;
-    montage_image->packets=montage_image->columns*montage_image->rows;
-    montage_image->pixels=(RunlengthPacket *) AllocateMemory((unsigned int)
-      montage_image->packets*sizeof(RunlengthPacket));
-    if (montage_image->pixels == (RunlengthPacket *) NULL)
-      {
-        MagickWarning(ResourceLimitWarning,"Unable to montage image_list",
-          "Memory allocation failed");
-        DestroyImages(montage_image);
-        return((Image *) NULL);
-      }
+    SetImage(montage_image);
     /*
       Set montage geometry.
     */
@@ -563,10 +543,6 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
       (void) strcat(montage_image->directory,image_list[tile]->filename);
       (void) strcat(montage_image->directory,"\n");
     }
-    /*
-      Initialize montage image to background color.
-    */
-    SetImage(montage_image);
     handler=SetMonitorHandler((MonitorHandler) NULL);
     if (texture != (Image *) NULL)
       TextureImage(montage_image,texture);
@@ -577,13 +553,13 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
         */
         FormatString(geometry,"%ux%u%+d%+d",montage_image->columns,
           font_height << 1,0,tile_info.y+4);
-        (void) CloneString(&annotate_info.geometry,geometry);
-        (void) CloneString(&annotate_info.text,montage_info->title);
-        AnnotateImage(montage_image,&annotate_info);
+        (void) CloneString(&annotate_info->geometry,geometry);
+        (void) CloneString(&annotate_info->text,montage_info->title);
+        AnnotateImage(montage_image,annotate_info);
       }
     (void) SetMonitorHandler(handler);
     /*
-      Copy tile image_list to the composite image.
+      Copy tile image to the composite image.
     */
     x_offset=tile_info.x;
     y_offset=title_offset+tile_info.y;
@@ -619,7 +595,6 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
             }
           image->orphan=True;
           bordered_image=BorderImage(image,&border_info);
-          image->orphan=False;
           if (bordered_image != (Image *) NULL)
             {
               DestroyImage(image);
@@ -701,12 +676,11 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
             Put an ornamental border around this tile.
           */
           tile_info=frame_info;
-          tile_info.width=width+(frame_info.width << 1);
-          tile_info.height=height+(frame_info.height << 1)+(font_height+4)*
-            MultilineCensus(image->label);
+          tile_info.width=width+2*frame_info.width;
+          tile_info.height=height+2*frame_info.height+
+            (font_height+4)*MultilineCensus(image->label);
           image->orphan=True;
           framed_image=FrameImage(image,&tile_info);
-          image->orphan=False;
           if (framed_image != (Image *) NULL)
             {
               DestroyImage(image);
@@ -738,27 +712,31 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
               */
               for (rows=0; rows < ((int) image->rows-4); rows++)
               {
-                p=montage_image->pixels+montage_image->columns*
-                  (y_offset+y+rows+4)+x_offset+x+image->columns;
+                q=GetPixelCache(montage_image,x+x_offset+image->columns,
+                  y_offset+y+rows+4,Min(tile_info.x,4),1);
+                if (q == (PixelPacket *) NULL)
+                  break;
                 for (columns=0; columns < Min(tile_info.x,4); columns++)
                 {
-                  if (p >= (montage_image->pixels+montage_image->packets))
-                    continue;
-                  Modulate(0.0,0.0,-25.0+4*columns,&p->red,&p->green,&p->blue);
-                  p++;
+                  Modulate(0.0,0.0,-25.0+4*columns,&q->red,&q->green,&q->blue);
+                  q++;
                 }
+                if (!SyncPixelCache(montage_image))
+                  break;
               }
               for (rows=0; rows < Min(tile_info.y,4); rows++)
               {
-                p=montage_image->pixels+montage_image->columns*
-                  (y_offset+y+image->rows+rows)+x_offset+x+4;
+                q=GetPixelCache(montage_image,x+x_offset+4,y_offset+y+
+                  image->rows+rows,image->columns,1);
+                if (q == (PixelPacket *) NULL)
+                  break;
                 for (columns=0; columns < (int) image->columns; columns++)
                 {
-                  if (p >= (montage_image->pixels+montage_image->packets))
-                    continue;
-                  Modulate(0.0,0.0,-25.0+4*rows,&p->red,&p->green,&p->blue);
-                  p++;
+                  Modulate(0.0,0.0,-25.0+4*rows,&q->red,&q->green,&q->blue);
+                  q++;
                 }
+                if (!SyncPixelCache(montage_image))
+                  break;
               }
             }
           if (image->label != (char *) NULL)
@@ -772,9 +750,9 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
                 (int) (montage_info->frame ? y_offset+height+
                 (border_width << 1)-bevel_width-2 : y_offset+tile_info.height+
                 (border_width << 1)+(montage_info->shadow ? 4 : 0)+2));
-              (void) CloneString(&annotate_info.geometry,geometry);
-              (void) CloneString(&annotate_info.text,image->label);
-              AnnotateImage(montage_image,&annotate_info);
+              (void) CloneString(&annotate_info->geometry,geometry);
+              (void) CloneString(&annotate_info->text,image->label);
+              AnnotateImage(montage_image,annotate_info);
             }
         }
       x_offset+=width+(tile_info.x+border_width)*2;
@@ -790,13 +768,12 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
       ProgressMonitor(MontageImageText,tiles,total_tiles);
       tiles++;
     }
-    CondenseImage(montage_image);
     if ((i+1) < (int) images_per_page)
       {
         /*
           Allocate next image structure.
         */
-        AllocateNextImage((ImageInfo *) NULL,montage_image);
+        AllocateNextImage(local_info,montage_image);
         if (montage_image->next == (Image *) NULL)
           {
             DestroyImages(montage_image);
@@ -808,8 +785,9 @@ Export Image *MontageImages(const Image *images,const MontageInfo *montage_info)
       }
   }
   if (texture != (Image *) NULL)
-    FreeMemory((char *) texture);
-  FreeMemory((char *) master_list);
+    FreeMemory(texture);
+  FreeMemory(master_list);
+  DestroyAnnotateInfo(annotate_info);
   DestroyImageInfo(local_info);
   while (montage_image->previous != (Image *) NULL)
     montage_image=montage_image->previous;
