@@ -18,7 +18,8 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -81,9 +82,9 @@ extern char
 %  Method closedir closes the named directory stream and frees the DIR
 %  structure.
 %
-%  The format of the closedir routine is:
+%  The format of the closedir method is:
 %
-%      closedir(entry)
+%      void closedir(DIR *entry)
 %
 %  A description of each parameter follows:
 %
@@ -95,7 +96,7 @@ Export void closedir(DIR *entry)
 {
   assert(entry != (DIR *) NULL);
   FindClose(entry->hSearch);
-  FreeMemory((char *) entry);
+  FreeMemory(entry);
 }
 
 /*
@@ -111,9 +112,9 @@ Export void closedir(DIR *entry)
 %
 %  Method Exit calls TerminateProcess for Win95.
 %
-%  The format of the Exit routine is:
+%  The format of the Exit method is:
 %
-%      Exit(status)
+%      int Exit(int status)
 %
 %  A description of each parameter follows:
 %
@@ -144,9 +145,9 @@ Export int Exit(int status)
 %  Method ImageFormatConflict returns true if the image format conflicts with
 %  a logical drive (.e.g. X:).
 %
-%  The format of the ImageFormatConflict routine is:
+%  The format of the ImageFormatConflict method is:
 %
-%      status=ImageFormatConflict(magick)
+%      int ImageFormatConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
@@ -178,9 +179,9 @@ Export int ImageFormatConflict(const char *magick)
 %
 %  Method IsWindows95 returns true if the system is Windows 95.
 %
-%  The format of the IsWindows95 routine is:
+%  The format of the IsWindows95 method is:
 %
-%      IsWindows95()
+%      int IsWindows95()
 %
 %  A description of each parameter follows:
 %
@@ -206,6 +207,150 @@ Export int IsWindows95()
 %                                                                             %
 %                                                                             %
 %                                                                             %
++  m m a p                                                                    %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method mmap emulates the Unix method of the same name.
+%
+%  The format of the mmap method is:
+%
+%    Export void *mmap(char *address,size_t length,int protection,int access,
+%      int file,off_t offset)
+%
+%
+*/
+Export void *mmap(char *address,size_t length,int protection,int access,
+  int file,off_t offset)
+{
+  void
+    *map;
+
+  HANDLE
+    handle;
+
+  map=(void *) NULL;
+  handle=INVALID_HANDLE_VALUE;
+  switch (protection)
+  {
+    case PROT_READ:
+    default:
+    {
+      handle=CreateFileMapping((HANDLE) _get_osfhandle(file),0,
+        PAGE_READONLY,0,length,0);
+      if (!handle)
+        break;
+      map=(void *) MapViewOfFile(handle,FILE_MAP_READ,0,0,length);
+      CloseHandle(handle);
+      break;
+    }
+    case PROT_WRITE:
+    {
+      handle=CreateFileMapping((HANDLE) _get_osfhandle(file),0,
+        PAGE_READWRITE,0,length,0);
+      if (!handle)
+        break;
+      map=(void *) MapViewOfFile(handle,FILE_MAP_WRITE,0,0,length);
+      CloseHandle(handle);
+      break;
+    }
+    case PROT_READWRITE:
+    {
+      handle=CreateFileMapping((HANDLE) _get_osfhandle(file),0,
+        PAGE_READWRITE,0,length,0);
+      if (!handle)
+        break;
+      map=(void *) MapViewOfFile(handle,FILE_MAP_ALL_ACCESS,0,0,length);
+      CloseHandle(handle);
+      break;
+    }
+  }
+  if (map == (void *) NULL)
+    return((void *) MAP_FAILED);
+  return((void *) ((char *) map+offset));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++  m u n m a p                                                                %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method munmap emulates the Unix method with the same name.
+%
+%  The format of the munmap method is:
+%
+%      int munmap(void *map,size_t length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method munmap returns 0 on success; otherwise, it
+%      returns -1 and sets errno to indicate the error.
+%
+%    o map: The address of the binary large object.
+%
+%    o length: The length of the binary large object.
+%
+%
+*/
+Export int munmap(void *map,size_t length)
+{
+  if (!UnmapViewOfFile(map))
+    return(-1);
+  return(0);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   N T E l a p s e d T i m e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method NTElapsedTime returns the elapsed time (in seconds) since the last
+%  call to StartTimer().
+%
+%  The format of the ElapsedTime method is:
+%
+%      double NTElapsedTime(void)
+%
+%
+*/
+Export double NTElapsedTime(void)
+{
+  union
+  {
+    FILETIME
+      filetime;
+
+    __int64
+      filetime64;
+  } elapsed_time;
+
+  SYSTEMTIME
+    system_time;
+
+  GetSystemTime(&system_time);
+  SystemTimeToFileTime(&system_time,&elapsed_time.filetime);
+  return((double) 1.0e-7*elapsed_time.filetime64);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   N T E r r o r H a n d l e r                                               %
 %                                                                             %
 %                                                                             %
@@ -215,9 +360,10 @@ Export int IsWindows95()
 %  Method NTErrorHandler displays an error message and then terminates
 %  the program.
 %
-%  The format of the NTErrorHandler routine is:
+%  The format of the NTErrorHandler method is:
 %
-%      NTErrorHandler(error,message,qualifier)
+%      void NTErrorHandler(const ErrorType error,const char *message,
+%        const char *qualifier)
 %
 %  A description of each parameter follows:
 %
@@ -230,7 +376,7 @@ Export int IsWindows95()
 %
 %
 */
-Export void NTErrorHandler(const unsigned int error,const char *message,
+Export void NTErrorHandler(const ErrorType error,const char *message,
   const char *qualifier)
 {
   char
@@ -271,9 +417,9 @@ Export void NTErrorHandler(const unsigned int error,const char *message,
 %   Method NTSystemCommand executes the specified command and waits until it
 %   terminates.  The returned value is the exit status of the command.
 %
-%  The format of the NTSystemCommand routine is:
+%  The format of the NTSystemCommand method is:
 %
-%      NTSystemCommand(command)
+%      int NTSystemCommand(const char *command)
 %
 %  A description of each parameter follows:
 %
@@ -344,12 +490,12 @@ Export int NTSystemCommand(const char *command)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%   Method NTTemporaryFilename creates a name for a temporary file.  It
-%   returns zero if an error occurs.
+%  Method NTTemporaryFilename creates a name for a temporary file.  It
+%  returns zero if an error occurs.
 %
-%  The format of the TemporaryFilename routine is:
+%  The format of the TemporaryFilename method is:
 %
-%      NTTemporaryFilename(filename)
+%      int NTTemporaryFilename(char *filename)
 %
 %  A description of each parameter follows:
 %
@@ -371,7 +517,7 @@ Export int NTTemporaryFilename(char *filename)
     status=GetTempFileName(path,"magick",0,filename);
   (void) remove(filename);
   if (strlen(filename) > 4)
-    if (strcmp(filename+strlen(filename)-4,".tmp") == 0)
+    if (Latin1Compare(filename+strlen(filename)-4,".tmp") == 0)
       filename[strlen(filename)-4]='\0';
   return(status);
 }
@@ -381,6 +527,67 @@ Export int NTTemporaryFilename(char *filename)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   N T U s e r T i m e                                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method NTUserTime returns the total time the process has been scheduled (i
+%  seconds) since the last call to StartTimer().
+%
+%  The format of the UserTime method is:
+%
+%      double NTUserTime(void)
+%
+%
+*/
+Export double NTUserTime(void)
+{
+  DWORD
+    status;
+
+  FILETIME
+    create_time,
+    exit_time;
+
+  OSVERSIONINFO
+    OsVersionInfo;
+
+  union
+  {
+    FILETIME
+      filetime;
+
+    __int64
+      filetime64;
+  } kernel_time;
+
+  union
+  {
+    FILETIME
+      filetime;
+
+    __int64
+      filetime64;
+  } user_time;
+
+  OsVersionInfo.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
+  GetVersionEx(&OsVersionInfo);
+  if (OsVersionInfo.dwPlatformId != VER_PLATFORM_WIN32_NT)
+    return(NTElapsedTime());
+  status=GetProcessTimes(GetCurrentProcess(),&create_time,&exit_time,
+    &kernel_time.filetime,&user_time.filetime);
+  if (status != TRUE)
+    return(0.0);
+  return((double) 1.0e-7*(kernel_time.filetime64+user_time.filetime64));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   N T W a r n i n g H a n d l e r                                           %
 %                                                                             %
 %                                                                             %
@@ -389,9 +596,10 @@ Export int NTTemporaryFilename(char *filename)
 %
 %  Method NTWarningHandler displays a warning message.
 %
-%  The format of the NTWarningHandler routine is:
+%  The format of the NTWarningHandler method is:
 %
-%      NTWarningHandler(warning,message,qualifier)
+%      void NTWarningHandler(const ErrorType warning,const char *message,
+%        const char *qualifier)
 %
 %  A description of each parameter follows:
 %
@@ -404,7 +612,7 @@ Export int NTTemporaryFilename(char *filename)
 %
 %
 */
-Export void NTWarningHandler(const unsigned int warning,const char *message,
+Export void NTWarningHandler(const ErrorType warning,const char *message,
   const char *qualifier)
 {
   char
@@ -436,9 +644,9 @@ Export void NTWarningHandler(const unsigned int warning,const char *message,
 %  Method opendir opens the directory named by filename and associates
 %  a directory stream with it.
 %
-%  The format of the opendir routine is:
+%  The format of the opendir method is:
 %
-%      opendir(entry)
+%      DIR *opendir(char *path)
 %
 %  A description of each parameter follows:
 %
@@ -519,9 +727,9 @@ Export struct dirent *readdir(DIR *entry)
 %   Method seekdir sets the position of the next readdir() operation
 %   on the directory stream.
 %
-%  The format of the seekdir routine is:
+%  The format of the seekdir method is:
 %
-%      seekdir(entry,position)
+%      void seekdir(DIR *entry,long position)
 %
 %  A description of each parameter follows:
 %
@@ -552,9 +760,9 @@ Export void seekdir(DIR *entry,long position)
 %   Method telldir returns the current location associated  with  the
 %   named directory stream.
 %
-%  The format of the telldir routine is:
+%  The format of the telldir method is:
 %
-%      telldir(entry)
+%      long telldir(DIR *entry)
 %
 %  A description of each parameter follows:
 %
