@@ -1,5 +1,5 @@
 /*
-  Windows NT specific include declarations.
+  Windows NT Utility Methods for ImageMagick.
 */
 #ifndef _NT_H
 #define _NT_H
@@ -8,11 +8,23 @@
 extern "C" {
 #endif
 
+#define WIN32_LEAN_AND_MEAN
+#define VC_EXTRALEAN
 #include <windows.h>
 #include <winuser.h>
 #include <wingdi.h>
+#include <io.h>
 #include <errno.h>
 
+/*
+  Define declarations.
+*/
+#define PROT_READ  1
+#define PROT_WRITE  2
+#define PROT_READWRITE  3
+#define MAP_SHARED  1
+#define MAP_PRIVATE  2
+#define MAP_FAILED  ((void*) 0xffffffff)
 /*
   Typedef declarations.
 */
@@ -21,7 +33,7 @@ typedef struct _DIR
 {
   HANDLE
     hSearch;
-
+ 
   WIN32_FIND_DATA
     Win32FindData;
 } DIR;
@@ -44,15 +56,20 @@ extern __declspec(dllexport) char
 
 extern __declspec(dllexport) int
   Exit(int),
-  ImageFormatConflict(char *),
+  ImageFormatConflict(const char *),
   IsWindows95(),
-  NTSystemCommand(char *);
+  NTSystemCommand(const char *);
 
 #if !defined(XS_VERSION)
 extern __declspec(dllexport) DIR
   *opendir(char *);
  
+extern __declspec(dllexport) double
+  NTElapsedTime(void),
+  NTUserTime(void);
+
 extern __declspec(dllexport) int
+  munmap(void *,size_t),
   NTTemporaryFilename(char *);
 
 extern __declspec(dllexport) long
@@ -63,8 +80,9 @@ extern __declspec(dllexport) struct dirent
  
 extern __declspec(dllexport) void
   closedir(DIR *),
-  NTErrorHandler(const unsigned int,const char *,const char *),
-  NTWarningHandler(const unsigned int,const char *,const char *),
+  *mmap(char *,size_t,int,int,int,off_t),
+  NTErrorHandler(const ErrorType,const char *,const char *),
+  NTWarningHandler(const WarningType,const char *,const char *),
   seekdir(DIR *,long);
 #endif
 
