@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -64,6 +65,45 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s P D F                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method IsPDF returns True if the image format type, identified by the
+%  magick string, is PDF.
+%
+%  The format of the ReadPDFImage method is:
+%
+%      unsigned int IsPDF(const unsigned char *magick,
+%        const unsigned int length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsPDF returns True if the image format type is PDF.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+Export unsigned int IsPDF(const unsigned char *magick,const unsigned int length)
+{
+  if (length < 5)
+    return(False);
+  if (strncmp((char *) magick,"%PDF-",5) == 0)
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e a d P D F I m a g e                                                   %
 %                                                                             %
 %                                                                             %
@@ -274,7 +314,7 @@ Export Image *ReadPDFImage(const ImageInfo *image_info)
       return((Image *) NULL);
     }
   local_info=CloneImageInfo(image_info);
-  GetBlobInfo(&local_info->blob);
+  GetBlobInfo(&local_info->blob_info);
   image=ReadPNMImage(local_info);
   DestroyImageInfo(local_info);
   (void) remove(postscript_filename);
@@ -300,8 +340,7 @@ Export Image *ReadPDFImage(const ImageInfo *image_info)
           Rotate image.
         */
         image->orphan=True;
-        rotated_image=RotateImage(image,90,False,True);
-        image->orphan=False;
+        rotated_image=RotateImage(image,90);
         if (rotated_image != (Image *) NULL)
           {
             DestroyImage(image);
@@ -349,6 +388,7 @@ Export Image *ReadPDFImage(const ImageInfo *image_info)
 */
 Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
 {
+#define CFormat  "/Filter [ /ASCII85Decode /%.1024s ]\n"
 #define ObjectsPerImage  12
 
   char
@@ -382,15 +422,14 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   RectangleInfo
     media_info;
 
-  register RunlengthPacket
+  register PixelPacket
     *p;
 
   register unsigned char
     *q;
 
   register int
-    i,
-    j;
+    i;
 
   time_t
     timer;
@@ -431,16 +470,9 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       if (status == False)
         WriterExit(FileOpenWarning,"Unable to open file",image);
     }
-  compression=image_info->compression;
-#if defined(HasZLIB)
-  if (compression == UndefinedCompression)
-    compression=ZipCompression;
-#else
-#if defined(HasLZW)
-  if (compression == UndefinedCompression)
-    compression=LZWCompression;
-#endif
-#endif
+  compression=image->compression;
+  if (image_info->compression != UndefinedCompression)
+    compression=image_info->compression;
   /*
     Allocate X ref memory.
   */
@@ -451,7 +483,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     Write Info object.
   */
   object=0;
-  (void) strcpy(buffer,"%PDF-1.1 \n");
+  (void) strcpy(buffer,"%PDF-1.2 \n");
   (void) WriteBlob(image,strlen(buffer),buffer);
   xref[object++]=TellBlob(image);
   info_id=object;
@@ -547,8 +579,9 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     if (image_info->page != (char *) NULL)
       (void) strcpy(geometry,image_info->page);
     else
-      if (image->page != (char *) NULL)
-        (void) strcpy(geometry,image->page);
+      if ((image->page_info.width != 0) && (image->page_info.height != 0))
+        (void) FormatString(geometry,"%ux%u%+d%+d",image->page_info.width,
+	  image->page_info.height,image->page_info.x,image->page_info.y);
       else
         if (Latin1Compare(image_info->magick,"PDF") == 0)
           (void) strcpy(geometry,PSPageGeometry);
@@ -632,10 +665,10 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
         {
           (void) strcpy(buffer,"BT\n");
           (void) WriteBlob(image,strlen(buffer),buffer);
-          (void) sprintf(buffer,"/F%u %u Tf\n",image->scene,
+          (void) sprintf(buffer,"/F%u %f Tf\n",image->scene,
             image_info->pointsize);
           (void) WriteBlob(image,strlen(buffer),buffer);
-          (void) sprintf(buffer,"%d %u Td\n",x,y+height+
+          (void) sprintf(buffer,"%d %f Td\n",x,y+height+
             i*image_info->pointsize+12);
           (void) WriteBlob(image,strlen(buffer),buffer);
           (void) sprintf(buffer,"(%.1024s) Tj\n",labels[i]);
@@ -644,7 +677,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlob(image,strlen(buffer),buffer);
           FreeMemory(labels[i]);
         }
-        FreeMemory((char *) labels);
+        FreeMemory(labels);
       }
     (void) sprintf(buffer,"%g 0 0 %g %d %d cm\n",x_scale,y_scale,x,y);
     (void) WriteBlob(image,strlen(buffer),buffer);
@@ -726,9 +759,13 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     else
       if (!IsFaxImage(image))
         {
-          (void) sprintf(buffer,"/Filter [ /ASCII85Decode /%.1024s ]\n",
-            compression == ZipCompression ? "FlateDecode" :
-            compression == LZWCompression ? "LZWDecode" : "RunLengthDecode");
+          switch (compression)
+          {
+            case JPEGCompression: sprintf(buffer,CFormat,"DCTDecode"); break;
+            case LZWCompression: sprintf(buffer,CFormat,"LZWDecode"); break;
+            case ZipCompression: sprintf(buffer,CFormat,"FlateDecode"); break;
+            default: sprintf(buffer,CFormat,"RunLengthDecode"); break;
+          }
           (void) WriteBlob(image,strlen(buffer),buffer);
         }
       else
@@ -757,10 +794,46 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) strcpy(buffer,"stream\n");
     (void) WriteBlob(image,strlen(buffer),buffer);
     length=TellBlob(image);
-    p=image->pixels;
     if (!IsPseudoClass(image) && !IsGrayImage(image))
       switch (compression)
       {
+        case JPEGCompression:
+        {
+          char
+            filename[MaxTextExtent];
+
+          FILE
+            *file;
+
+          Image
+            *jpeg_image;
+
+          int
+            c;
+
+          /*
+            Write image to temporary file in JPEG format.
+          */
+          TemporaryFilename(filename);
+          jpeg_image=CloneImage(image,image->columns,image->rows,True);
+          if (jpeg_image == (Image *) NULL)
+            WriterExit(DelegateWarning,"Unable to clone image",image);
+          (void) strcpy(jpeg_image->filename,filename);
+          status=WriteJPEGImage(image_info,jpeg_image);
+          DestroyImage(jpeg_image);
+          if (status == False)
+            WriterExit(DelegateWarning,"Unable to write image",image);
+          file=fopen(filename,ReadBinaryType);
+          if (file == (FILE *) NULL)
+            WriterExit(FileOpenWarning,"Unable to open file",image);
+          Ascii85Initialize();
+          for (c=fgetc(file); c != EOF; c=fgetc(file))
+            Ascii85Encode(image,c);
+          Ascii85Flush(image);
+          (void) fclose(file);
+          (void) remove(filename);
+          break;
+        }
         case RunlengthEncodedCompression:
         default:
         {
@@ -772,17 +845,19 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           pixels=(unsigned char *)
             AllocateMemory(number_packets*sizeof(unsigned char));
           if (pixels == (unsigned char *) NULL)
-            WriterExit(ResourceLimitWarning,"Memory allocation failed",
-              image);
+            WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
           /*
             Dump runlength encoded pixels.
           */
           q=pixels;
-          for (i=0; i < (int) image->packets; i++)
+          for (y=0; y < (int) image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            p=GetPixelCache(image,0,y,image->columns,1);
+            if (p == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (int) image->columns; x++)
             {
-              if (image->matte && (p->index == Transparent))
+              if (image->matte && (p->opacity == Transparent))
                 {
                   *q++=DownScale(MaxRGB);
                   *q++=DownScale(MaxRGB);
@@ -800,13 +875,13 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                     *q++=DownScale(p->red);
                     *q++=DownScale(p->green);
                     *q++=DownScale(p->blue);
-                    *q++=DownScale(p->index);
+                    *q++=DownScale(p->opacity);
                   }
+              p++;
             }
-            p++;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(i,image->packets))
-                ProgressMonitor(SaveImageText,i,image->packets);
+              if (QuantumTick(y,image->rows))
+                ProgressMonitor(SaveImageText,y,image->rows);
           }
           if (compression == ZipCompression)
             status=
@@ -816,7 +891,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
               status=LZWEncodeImage(image,number_packets,pixels);
             else
               status=PackbitsEncodeImage(image,number_packets,pixels);
-          FreeMemory((char *) pixels);
+          FreeMemory(pixels);
           if (!status)
             {
               CloseBlob(image);
@@ -830,11 +905,14 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
             Dump uncompressed DirectColor packets.
           */
           Ascii85Initialize();
-          for (i=0; i < (int) image->packets; i++)
+          for (y=0; y < (int) image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            p=GetPixelCache(image,0,y,image->columns,1);
+            if (p == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (int) image->columns; x++)
             {
-              if (image->matte && (p->index == Transparent))
+              if (image->matte && (p->opacity == Transparent))
                 {
                   Ascii85Encode(image,DownScale(MaxRGB));
                   Ascii85Encode(image,DownScale(MaxRGB));
@@ -852,13 +930,13 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                     Ascii85Encode(image,DownScale(p->red));
                     Ascii85Encode(image,DownScale(p->green));
                     Ascii85Encode(image,DownScale(p->blue));
-                    Ascii85Encode(image,DownScale(p->index));
+                    Ascii85Encode(image,DownScale(p->opacity));
                   }
+              p++;
             }
-            p++;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(i,image->packets))
-                ProgressMonitor(SaveImageText,i,image->packets);
+              if (QuantumTick(y,image->rows))
+                ProgressMonitor(SaveImageText,y,image->rows);
           }
           Ascii85Flush(image);
           break;
@@ -876,9 +954,6 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           if (image->colors == 2)
             polarity=
               Intensity(image->colormap[0]) < Intensity(image->colormap[1]);
-          bit=0;
-          byte=0;
-          x=0;
           switch (compression)
           {
             case RunlengthEncodedCompression:
@@ -896,12 +971,16 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 Dump uncompressed PseudoColor packets.
               */
               Ascii85Initialize();
-              for (i=0; i < (int) image->packets; i++)
+              for (y=0; y < (int) image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
+                if (!GetPixelCache(image,0,y,image->columns,1))
+                  break;
+                bit=0;
+                byte=0;
+                for (x=0; x < (int) image->columns; x++)
                 {
                   byte<<=1;
-                  if (p->index == polarity)
+                  if (image->indexes[x] == polarity)
                     byte|=0x01;
                   bit++;
                   if (bit == 8)
@@ -910,24 +989,13 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                       bit=0;
                       byte=0;
                     }
-                  x++;
-                  if (x == (int) image->columns)
-                    {
-                      /*
-                        Advance to the next scanline.
-                      */
-                      if (bit != 0)
-                        Ascii85Encode(image,byte << (8-bit));
-                      if (image->previous == (Image *) NULL)
-                        if (QuantumTick(y,image->rows))
-                          ProgressMonitor(SaveImageText,y,image->rows);
-                      bit=0;
-                      byte=0;
-                      x=0;
-                      y++;
-                   }
+                  p++;
                 }
-                p++;
+                if (bit != 0)
+                  Ascii85Encode(image,byte << (8-bit));
+                if (image->previous == (Image *) NULL)
+                  if (QuantumTick(y,image->rows))
+                    ProgressMonitor(SaveImageText,y,image->rows);
               }
               Ascii85Flush(image);
               break;
@@ -957,14 +1025,18 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 Dump Runlength encoded pixels.
               */
               q=pixels;
-              for (i=0; i < (int) image->packets; i++)
+              for (y=0; y < (int) image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
-                  *q++=(unsigned char) p->index;
-                p++;
+                if (!GetPixelCache(image,0,y,image->columns,1))
+                  break;
+                for (x=0; x < (int) image->columns; x++)
+                {
+                  *q++=(unsigned char) image->indexes[x];
+                  p++;
+                }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(i,image->packets))
-                    ProgressMonitor(SaveImageText,i,image->packets);
+                  if (QuantumTick(y,image->rows))
+                    ProgressMonitor(SaveImageText,y,image->rows);
               }
               if (compression == ZipCompression)
                 status=ZLIBEncodeImage(image,number_packets,image_info->quality,
@@ -974,7 +1046,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   status=LZWEncodeImage(image,number_packets,pixels);
                 else
                   status=PackbitsEncodeImage(image,number_packets,pixels);
-              FreeMemory((char *) pixels);
+              FreeMemory(pixels);
               if (!status)
                 {
                   CloseBlob(image);
@@ -988,14 +1060,18 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 Dump uncompressed PseudoColor packets.
               */
               Ascii85Initialize();
-              for (i=0; i < (int) image->packets; i++)
+              for (y=0; y < (int) image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
-                  Ascii85Encode(image,(unsigned char) p->index);
-                p++;
+                if (!GetPixelCache(image,0,y,image->columns,1))
+                  break;
+                for (x=0; x < (int) image->columns; x++)
+                {
+                  Ascii85Encode(image,(unsigned char) image->indexes[x]);
+                  p++;
+                }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(i,image->packets))
-                    ProgressMonitor(SaveImageText,i,image->packets);
+                  if (QuantumTick(y,image->rows))
+                    ProgressMonitor(SaveImageText,y,image->rows);
               }
               Ascii85Flush(image);
               break;
@@ -1044,13 +1120,12 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     height=image->rows;
     x=0;
     y=0;
-    image->orphan=True;
     (void) ParseImageGeometry("106x106+0+0>",&x,&y,&width,&height);
+    image->orphan=True;
     if (image->class == PseudoClass)
       tile_image=SampleImage(image,width,height);
     else
       tile_image=ZoomImage(image,width,height);
-    image->orphan=False;
     if (tile_image == (Image *) NULL)
       WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
     xref[object++]=TellBlob(image);
@@ -1094,7 +1169,6 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) strcpy(buffer,"stream\n");
     (void) WriteBlob(image,strlen(buffer),buffer);
     length=TellBlob(image);
-    p=tile_image->pixels;
     if (!IsPseudoClass(tile_image) && !IsGrayImage(tile_image))
       switch (compression)
       {
@@ -1111,18 +1185,20 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           if (pixels == (unsigned char *) NULL)
             {
               DestroyImage(tile_image);
-              WriterExit(ResourceLimitWarning,"Memory allocation failed",
-                image);
+              WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
             }
           /*
             Dump runlength encoded pixels.
           */
           q=pixels;
-          for (i=0; i < (int) tile_image->packets; i++)
+          for (y=0; y < (int) tile_image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            p=GetPixelCache(tile_image,0,y,tile_image->columns,1);
+            if (p == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (int) tile_image->columns; x++)
             {
-              if (tile_image->matte && (p->index == Transparent))
+              if (tile_image->matte && (p->opacity == Transparent))
                 {
                   *q++=DownScale(MaxRGB);
                   *q++=DownScale(MaxRGB);
@@ -1134,10 +1210,10 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   *q++=DownScale(p->green);
                   *q++=DownScale(p->blue);
                   if (image->colorspace == CMYKColorspace)
-                    *q++=DownScale(p->index);
+                    *q++=DownScale(p->opacity);
                 }
+              p++;
             }
-            p++;
           }
           if (compression == ZipCompression)
             status=
@@ -1147,7 +1223,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
               status=LZWEncodeImage(image,number_packets,pixels);
             else
               status=PackbitsEncodeImage(image,number_packets,pixels);
-          FreeMemory((char *) pixels);
+          FreeMemory(pixels);
           if (!status)
             {
               CloseBlob(image);
@@ -1161,11 +1237,14 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
             Dump uncompressed DirectColor packets.
           */
           Ascii85Initialize();
-          for (i=0; i < (int) tile_image->packets; i++)
+          for (y=0; y < (int) tile_image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            p=GetPixelCache(tile_image,0,y,tile_image->columns,1);
+            if (p == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (int) tile_image->columns; x++)
             {
-              if (tile_image->matte && (p->index == Transparent))
+              if (tile_image->matte && (p->opacity == Transparent))
                 {
                   Ascii85Encode(image,DownScale(MaxRGB));
                   Ascii85Encode(image,DownScale(MaxRGB));
@@ -1177,10 +1256,10 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   Ascii85Encode(image,DownScale(p->green));
                   Ascii85Encode(image,DownScale(p->blue));
                   if (image->colorspace == CMYKColorspace)
-                    Ascii85Encode(image,DownScale(p->index));
+                    Ascii85Encode(image,DownScale(p->opacity));
                 }
+              p++;
             }
-            p++;
           }
           Ascii85Flush(image);
           break;
@@ -1198,9 +1277,6 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           if (image->colors == 2)
             polarity=Intensity(tile_image->colormap[0]) <
               Intensity(tile_image->colormap[1]);
-          bit=0;
-          byte=0;
-          x=0;
           switch (compression)
           {
             case RunlengthEncodedCompression:
@@ -1222,12 +1298,16 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 Dump Runlength encoded pixels.
               */
               q=pixels;
-              for (i=0; i < (int) tile_image->packets; i++)
+              for (y=0; y < (int) tile_image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
+                if (!GetPixelCache(tile_image,0,y,tile_image->columns,1))
+                  break;
+                bit=0;
+                byte=0;
+                for (x=0; x < (int) tile_image->columns; x++)
                 {
                   byte<<=1;
-                  if (p->index == polarity)
+                  if (tile_image->indexes[x] == polarity)
                     byte|=0x01;
                   bit++;
                   if (bit == 8)
@@ -1236,24 +1316,13 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                       bit=0;
                       byte=0;
                     }
-                  x++;
-                  if (x == (int) tile_image->columns)
-                    {
-                      /*
-                        Advance to the next scanline.
-                      */
-                      if (bit != 0)
-                        *q++=byte << (8-bit);
-                      if (image->previous == (Image *) NULL)
-                        if (QuantumTick(y,tile_image->rows))
-                          ProgressMonitor(SaveImageText,y,tile_image->rows);
-                      bit=0;
-                      byte=0;
-                      x=0;
-                      y++;
-                   }
+                  p++;
                 }
-                p++;
+                if (bit != 0)
+                  *q++=byte << (8-bit);
+                if (image->previous == (Image *) NULL)
+                  if (QuantumTick(y,tile_image->rows))
+                    ProgressMonitor(SaveImageText,y,tile_image->rows);
               }
               if (compression == ZipCompression)
                 status=ZLIBEncodeImage(image,number_packets,image_info->quality,
@@ -1263,7 +1332,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   status=LZWEncodeImage(image,number_packets,pixels);
                 else
                   status=PackbitsEncodeImage(image,number_packets,pixels);
-              FreeMemory((char *) pixels);
+              FreeMemory(pixels);
               if (!status)
                 {
                   CloseBlob(image);
@@ -1277,12 +1346,16 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 Dump uncompressed PseudoColor packets.
               */
               Ascii85Initialize();
-              for (i=0; i < (int) tile_image->packets; i++)
+              for (y=0; y < (int) tile_image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
+                if (!GetPixelCache(tile_image,0,y,tile_image->columns,1))
+                  break;
+                bit=0;
+                byte=0;
+                for (x=0; x < (int) tile_image->columns; x++)
                 {
                   byte<<=1;
-                  if (p->index == polarity)
+                  if (tile_image->indexes[x] == polarity)
                     byte|=0x01;
                   bit++;
                   if (bit == 8)
@@ -1291,24 +1364,13 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                       bit=0;
                       byte=0;
                     }
-                  x++;
-                  if (x == (int) tile_image->columns)
-                    {
-                      /*
-                        Advance to the next scanline.
-                      */
-                      if (bit != 0)
-                        Ascii85Encode(image,byte << (8-bit));
-                      if (image->previous == (Image *) NULL)
-                        if (QuantumTick(y,tile_image->rows))
-                          ProgressMonitor(SaveImageText,y,tile_image->rows);
-                      bit=0;
-                      byte=0;
-                      x=0;
-                      y++;
-                   }
+                  p++;
                 }
-                p++;
+                if (bit != 0)
+                  Ascii85Encode(image,byte << (8-bit));
+                if (image->previous == (Image *) NULL)
+                  if (QuantumTick(y,tile_image->rows))
+                    ProgressMonitor(SaveImageText,y,tile_image->rows);
               }
               Ascii85Flush(image);
               break;
@@ -1334,18 +1396,22 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
               if (pixels == (unsigned char *) NULL)
                 {
                   DestroyImage(tile_image);
-                  WriterExit(ResourceLimitWarning,
-                    "Memory allocation failed",image);
+                  WriterExit(ResourceLimitWarning,"Memory allocation failed",
+                    image);
                 }
               /*
                 Dump Runlength encoded pixels.
               */
               q=pixels;
-              for (i=0; i < (int) tile_image->packets; i++)
+              for (y=0; y < (int) tile_image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
-                  *q++=(unsigned char) p->index;
-                p++;
+                if (!GetPixelCache(tile_image,0,y,tile_image->columns,1))
+                  break;
+                for (x=0; x < (int) tile_image->columns; x++)
+                {
+                  *q++=(unsigned char) tile_image->indexes[x];
+                  p++;
+                }
               }
               if (compression == ZipCompression)
                 status=ZLIBEncodeImage(image,number_packets,image_info->quality,
@@ -1355,7 +1421,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   status=LZWEncodeImage(image,number_packets,pixels);
                 else
                   status=PackbitsEncodeImage(image,number_packets,pixels);
-              FreeMemory((char *) pixels);
+              FreeMemory(pixels);
               if (!status)
                 {
                   CloseBlob(image);
@@ -1369,11 +1435,15 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 Dump uncompressed PseudoColor packets.
               */
               Ascii85Initialize();
-              for (i=0; i < (int) tile_image->packets; i++)
+              for (y=0; y < (int) tile_image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
-                  Ascii85Encode(image,(unsigned char) p->index);
-                p++;
+                if (!GetPixelCache(tile_image,0,y,tile_image->columns,1))
+                  break;
+                for (x=0; x < (int) tile_image->columns; x++)
+                {
+                  Ascii85Encode(image,tile_image->indexes[x]);
+                  p++;
+                }
               }
               Ascii85Flush(image);
               break;
@@ -1446,8 +1516,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       }
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
   if (image_info->adjoin)
@@ -1486,7 +1555,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlob(image,strlen(buffer),buffer);
   (void) strcpy(buffer,"%%EOF\n");
   (void) WriteBlob(image,strlen(buffer),buffer);
-  FreeMemory((char *) xref);
+  FreeMemory(xref);
   CloseBlob(image);
   if (image->temporary)
     {
@@ -1503,7 +1572,7 @@ Export unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       if (file == (FILE *) NULL)
         WriterExit(FileOpenWarning,"Unable to open file",image);
       for (c=fgetc(file); c != EOF; c=fgetc(file))
-        (void) putc(c,encode_image.file);
+        (void) fputc(c,encode_image.file);
       (void) fclose(file);
       (void) remove(image->filename);
       image->temporary=False;
