@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,6 +60,45 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s P N M                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method IsPNM returns True if the image format type, identified by the
+%  magick string, is PNM.
+%
+%  The format of the ReadPNMImage method is:
+%
+%      unsigned int IsPNM(const unsigned char *magick,
+%        const unsigned int length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsPNM returns True if the image format type is PNM.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+Export unsigned int IsPNM(const unsigned char *magick,const unsigned int length)
+{
+  if (length < 2)
+    return(False);
+  if ((*magick == 'P') && isdigit((int) magick[1]))
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e a d P N M I m a g e                                                   %
 %                                                                             %
 %                                                                             %
@@ -69,9 +109,9 @@
 %  It allocates the memory necessary for the new Image structure and returns
 %  a pointer to the new image.
 %
-%  The format of the ReadPNMImage routine is:
+%  The format of the ReadPNMImage method is:
 %
-%      image=ReadPNMImage(image_info)
+%      Image *ReadPNMImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -109,7 +149,8 @@ static unsigned int PNMInteger(Image *image,const unsigned int base)
           *q;
 
         unsigned int
-          length;
+          length,
+          offset;
 
         /*
           Read comment.
@@ -122,19 +163,20 @@ static unsigned int PNMInteger(Image *image,const unsigned int base)
         else
           {
             length=MaxTextExtent;
-            image->comments=(char *) AllocateMemory(length*sizeof(char));
+            image->comments=(char *)
+              AllocateMemory((length+strlen(P7Comment)+1)*sizeof(char));
             p=image->comments;
           }
-        q=p;
+        offset=p-image->comments;
         if (image->comments != (char *) NULL)
           for ( ; (c != EOF) && (c != '\n'); p++)
           {
-            if ((p-image->comments+sizeof(P7Comment)) >= length)
+            if ((p-image->comments) >= length)
               {
                 length<<=1;
                 length+=MaxTextExtent;
                 image->comments=(char *) ReallocateMemory((char *)
-                  image->comments,length*sizeof(char));
+                  image->comments,(length+strlen(P7Comment)+1)*sizeof(char));
                 if (image->comments == (char *) NULL)
                   break;
                 p=image->comments+Extent(image->comments);
@@ -149,6 +191,7 @@ static unsigned int PNMInteger(Image *image,const unsigned int base)
               (char *) NULL);
             return(0);
           }
+        q=image->comments+offset;
         if (Latin1Compare(q,P7Comment) == 0)
           *q='\0';
         continue;
@@ -174,15 +217,19 @@ static unsigned int PNMInteger(Image *image,const unsigned int base)
 
 Export Image *ReadPNMImage(const ImageInfo *image_info)
 {
-#define MaxRawValue  255
-
   char
     format;
 
   Image
     *image;
 
+  IndexPacket
+    index;
+
   int
+    blue,
+    green,
+    red,
     y;
 
   MonitorHandler
@@ -195,22 +242,20 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
     i,
     x;
 
-  register long
-    packets;
-
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
+  register unsigned char
+    *p;
+
+  unsigned char
+    *pixels;
+
   unsigned int
     max_value,
+    packets,
     status;
 
-  unsigned short
-    blue,
-    green,
-    index,
-    red;
-
   /*
     Allocate image structure.
   */
@@ -246,10 +291,11 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
       max_value=1;  /* bitmap */
     else
       max_value=PNMInteger(image,10);
+    image->depth=max_value < 256 ? 8 : QuantumDepth;
     if ((format != '3') && (format != '6'))
       {
         image->class=PseudoClass;
-        image->colors=Min(max_value,MaxRGB)+1;
+        image->colors=max_value+1;
       }
     if (image_info->ping)
       {
@@ -265,19 +311,19 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
         /*
           Create colormap.
         */
-        image->colormap=(ColorPacket *)
-          AllocateMemory(image->colors*sizeof(ColorPacket));
-        if (image->colormap == (ColorPacket *) NULL)
+        image->colormap=(PixelPacket *)
+          AllocateMemory(image->colors*sizeof(PixelPacket));
+        if (image->colormap == (PixelPacket *) NULL)
           ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
         if (format != '7')
           for (i=0; i < (int) image->colors; i++)
           {
-            image->colormap[i].red=(Quantum)
-              ((long) (MaxRGB*i)/(image->colors-1));
-            image->colormap[i].green=(Quantum)
-              ((long) (MaxRGB*i)/(image->colors-1));
-            image->colormap[i].blue=(Quantum)
-              ((long) (MaxRGB*i)/(image->colors-1));
+            image->colormap[i].red=
+              ((unsigned long) (MaxRGB*i)/(image->colors-1));
+            image->colormap[i].green=
+              ((unsigned long) (MaxRGB*i)/(image->colors-1));
+            image->colormap[i].blue=
+              ((unsigned long) (MaxRGB*i)/(image->colors-1));
           }
         else
           {
@@ -289,9 +335,10 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
               for (green=0; green < 8; green++)
                 for (blue=0; blue < 4; blue++)
                 {
-                  image->colormap[i].red=(Quantum) ((long) (red*MaxRGB)/7);
-                  image->colormap[i].green=(Quantum) ((long) (green*MaxRGB)/7);
-                  image->colormap[i].blue=(Quantum) ((long) (blue*MaxRGB)/3);
+                  image->colormap[i].red=((unsigned long) (MaxRGB*red)/0x07);
+                  image->colormap[i].green=
+                    ((unsigned long) (green*MaxRGB)/0x07);
+                  image->colormap[i].blue=((unsigned long) (MaxRGB*blue)/0x03);
                   i++;
                 }
           }
@@ -308,39 +355,29 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
           for (i=0; i <= (int) max_value; i++)
             scale[i]=(Quantum) ((i*MaxRGB+(max_value >> 1))/max_value);
         }
-    packets=0;
-    image->pixels=(RunlengthPacket *)
-      AllocateMemory(image->columns*image->rows*sizeof(RunlengthPacket));
-    if (image->pixels == (RunlengthPacket *) NULL)
-      ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
     /*
       Convert PNM pixels to runlength-encoded MIFF packets.
     */
-    q=image->pixels;
-    SetRunlengthEncoder(q);
     switch (format)
     {
       case '1':
       {
         /*
-          Convert PBM image to runlength-encoded packets.
+          Convert PBM image to pixel packets.
         */
         for (y=0; y < (int) image->rows; y++)
         {
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
             index=!PNMInteger(image,2);
-            if ((index == q->index) && ((int) q->length < MaxRunlength))
-              q->length++;
-            else
-              {
-                if (packets != 0)
-                  q++;
-                packets++;
-                q->index=index;
-                q->length=0;
-              }
+            image->indexes[x]=index;
+            *q++=image->colormap[index];
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
@@ -350,24 +387,21 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
       case '2':
       {
         /*
-          Convert PGM image to runlength-encoded packets.
+          Convert PGM image to pixel packets.
         */
         for (y=0; y < (int) image->rows; y++)
         {
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
             index=PNMInteger(image,10);
-            if ((index == q->index) && ((int) q->length < MaxRunlength))
-              q->length++;
-            else
-              {
-                if (packets != 0)
-                  q++;
-                packets++;
-                q->index=index;
-                q->length=0;
-              }
+            image->indexes[x]=index;
+            *q++=image->colormap[index];
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
@@ -377,10 +411,13 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
       case '3':
       {
         /*
-          Convert PNM image to runlength-encoded packets.
+          Convert PNM image to pixel packets.
         */
         for (y=0; y < (int) image->rows; y++)
         {
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
             red=PNMInteger(image,10);
@@ -392,21 +429,13 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
                 green=scale[green];
                 blue=scale[blue];
               }
-            if ((red == q->red) && (green == q->green) && (blue == q->blue) &&
-                ((int) q->length < MaxRunlength))
-              q->length++;
-            else
-              {
-                if (packets != 0)
-                  q++;
-                packets++;
-                q->red=red;
-                q->green=green;
-                q->blue=blue;
-                q->index=0;
-                q->length=0;
-            }
+            q->red=red;
+            q->green=green;
+            q->blue=blue;
+            q++;
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
@@ -420,10 +449,13 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
           byte;
 
         /*
-          Convert PBM raw image to runlength-encoded packets.
+          Convert PBM raw image to pixel packets.
         */
         for (y=0; y < (int) image->rows; y++)
         {
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           bit=0;
           byte=0;
           for (x=0; x < (int) image->columns; x++)
@@ -431,21 +463,15 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
             if (bit == 0)
               byte=ReadByte(image);
             index=(byte & 0x80) ? 0 : 1;
-            if ((index == q->index) && ((int) q->length < MaxRunlength))
-              q->length++;
-            else
-              {
-                if (packets != 0)
-                  q++;
-                packets++;
-                q->index=index;
-                q->length=0;
-              }
+            image->indexes[x]=index;
+            *q++=image->colormap[index];
             bit++;
             if (bit == 8)
               bit=0;
             byte<<=1;
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
@@ -456,55 +482,78 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
       case '7':
       {
         /*
-          Convert PGM raw image to runlength-encoded packets.
+          Convert PGM raw image to pixel packets.
         */
+        packets=image->depth <= 8 ? 1 : 2;
+        pixels=(unsigned char *) AllocateMemory(packets*image->columns);
+        if (pixels == (unsigned char *) NULL)
+          ReaderExit(CorruptImageWarning,"Unable to allocate memory",image);
         for (y=0; y < (int) image->rows; y++)
         {
+          status=ReadBlob(image,packets*image->columns,pixels);
+          if (status == False)
+            ReaderExit(CorruptImageWarning,"Unable to read image data",image);
+          p=pixels;
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
-            if (max_value <= MaxRawValue)
-              index=ReadByte(image);
-            else
-              index=LSBFirstReadShort(image);
-            if (index > max_value)
-              index=max_value;
-            if ((index == q->index) && ((int) q->length < MaxRunlength))
-              q->length++;
+            if (image->depth <= 8)
+              index=(*p++);
             else
               {
-                if (packets != 0)
-                  q++;
-                packets++;
-                q->index=index;
-                q->length=0;
+                index=(*p++) << 8;
+                index|=(*p++);
               }
+            if (index >= image->colors)
+              ReaderExit(CorruptImageWarning,"invalid colormap index",image);
+            image->indexes[x]=index;
+            *q++=image->colormap[index];
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
         }
+        FreeMemory(pixels);
         break;
       }
       case '6':
       {
         /*
-          Convert PNM raster image to runlength-encoded packets.
+          Convert PNM raster image to pixel packets.
         */
+        packets=image->depth <= 8 ? 3 : 6;
+        pixels=(unsigned char *) AllocateMemory(packets*image->columns);
+        if (pixels == (unsigned char *) NULL)
+          ReaderExit(CorruptImageWarning,"Unable to allocate memory",image);
         for (y=0; y < (int) image->rows; y++)
         {
+          status=ReadBlob(image,packets*image->columns,pixels);
+          if (status == False)
+            ReaderExit(CorruptImageWarning,"Unable to read image data",image);
+          p=pixels;
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
-            if (max_value <= MaxRawValue)
+            if (image->depth <= 8)
               {
-                red=ReadByte(image);
-                green=ReadByte(image);
-                blue=ReadByte(image);
+                red=(*p++);
+                green=(*p++);
+                blue=(*p++);
               }
             else
               {
-                red=LSBFirstReadShort(image);
-                green=LSBFirstReadShort(image);
-                blue=LSBFirstReadShort(image);
+                red=(*p++) << 8;
+                red|=(*p++);
+                green=(*p++) << 8;
+                green|=(*p++);
+                blue=(*p++) << 8;
+                blue|=(*p++);
               }
             if (scale != (Quantum *) NULL)
               {
@@ -512,25 +561,18 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
                 green=scale[green];
                 blue=scale[blue];
               }
-            if ((red == q->red) && (green == q->green) && (blue == q->blue) &&
-                ((int) q->length < MaxRunlength))
-              q->length++;
-            else
-              {
-                if (packets != 0)
-                  q++;
-                packets++;
-                q->red=red;
-                q->green=green;
-                q->blue=blue;
-                q->index=0;
-                q->length=0;
-              }
+            q->red=red;
+            q->green=green;
+            q->blue=blue;
+            q++;
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
         }
+        FreeMemory(pixels);
         handler=SetMonitorHandler((MonitorHandler) NULL);
         (void) SetMonitorHandler(handler);
         break;
@@ -539,12 +581,9 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
         ReaderExit(CorruptImageWarning,"Not a PNM image file",image);
     }
     if (scale != (Quantum *) NULL)
-      FreeMemory((char *) scale);
+      FreeMemory(scale);
     if (EOFBlob(image))
       MagickWarning(CorruptImageWarning,"not enough pixels",image->filename);
-    SetRunlengthPackets(image,packets);
-    if (image->class == PseudoClass)
-      SyncImage(image);
     /*
       Proceed to next image.
     */
@@ -574,8 +613,7 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
             return((Image *) NULL);
           }
         image=image->next;
-        ProgressMonitor(LoadImagesText,(unsigned int) TellBlob(image),
-          (unsigned int) image->filesize);
+        ProgressMonitor(LoadImagesText,TellBlob(image),image->filesize);
       }
   } while ((status == True) && (format == 'P'));
   while (image->previous != (Image *) NULL)
@@ -598,9 +636,9 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
 %  Procedure WritePNMImage writes an image to a file in the PNM rasterfile
 %  format.
 %
-%  The format of the WritePNMImage routine is:
+%  The format of the WritePNMImage method is:
 %
-%      status=WritePNMImage(image_info,image)
+%      unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -616,21 +654,19 @@ Export Image *ReadPNMImage(const ImageInfo *image_info)
 */
 Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
 {
-#define MaxRawValue  255
-
   char
     buffer[MaxTextExtent],
     *magick;
 
   int
-    x,
     y;
 
   register int
     i,
-    j;
+    j,
+    x;
 
-  register RunlengthPacket
+  register PixelPacket
     *p;
 
   unsigned char
@@ -673,7 +709,6 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         quantize_info.dither=image_info->dither;
         quantize_info.colorspace=GRAYColorspace;
         (void) QuantizeImage(&quantize_info,image);
-        SyncImage(image);
       }
     /*
       Write PNM file header.
@@ -684,8 +719,7 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           Full color PNM image.
         */
         format='6';
-        if ((image_info->compression == NoCompression) ||
-            (MaxRGB > MaxRawValue))
+        if ((image_info->compression == NoCompression) || (image->depth > 8))
           format='3';
       }
     else
@@ -694,8 +728,7 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           Colormapped PNM image.
         */
         format='6';
-        if ((image_info->compression == NoCompression) ||
-            (MaxRGB > MaxRawValue))
+        if ((image_info->compression == NoCompression) || (image->depth > 8))
           format='3';
         if ((Latin1Compare(magick,"PPM") != 0) && IsGrayImage(image))
           {
@@ -704,7 +737,7 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             */
             format='5';
             if ((image_info->compression == NoCompression) ||
-                (MaxRGB > MaxRawValue))
+                (image->depth > 8))
               format='2';
             if (Latin1Compare(magick,"PGM") != 0)
               if (image->colors == 2)
@@ -748,9 +781,6 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
     /*
       Convert runlength encoded to PNM raster pixels.
     */
-    x=0;
-    y=0;
-    p=image->pixels;
     switch (format)
     {
       case '1':
@@ -765,23 +795,25 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         if (image->colors == 2)
           polarity=
             Intensity(image->colormap[0]) > Intensity(image->colormap[1]);
-        for (i=0; i < (int) image->packets; i++)
+        i=0;
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          for (x=0; x < (int) image->columns; x++)
           {
-            (void) sprintf(buffer,"%d ",(int) (p->index == polarity));
+            (void) sprintf(buffer,"%d ",(int) (image->indexes[x] == polarity));
             (void) WriteBlob(image,strlen(buffer),buffer);
-            x++;
-            if (x == 36)
+            i++;
+            if (i == 36)
               {
                 (void) WriteByte(image,'\n');
-                x=0;
+                i=0;
               }
           }
-          p++;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->packets))
-              ProgressMonitor(SaveImageText,i,image->packets);
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         break;
       }
@@ -790,26 +822,30 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PGM image.
         */
-        (void) sprintf(buffer,"%d\n",MaxRGB);
+        (void) sprintf(buffer,"%ld\n",MaxRGB);
         (void) WriteBlob(image,strlen(buffer),buffer);
-        for (i=0; i < (int) image->packets; i++)
+        i=0;
+        for (y=0; y < (int) image->rows; y++)
         {
-          index=Intensity(*p);
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          for (x=0; x < (int) image->columns; x++)
           {
+            index=Intensity(*p);
             (void) sprintf(buffer,"%d ",index);
             (void) WriteBlob(image,strlen(buffer),buffer);
-            x++;
-            if (x == 12)
+            i++;
+            if (i == 12)
               {
                 (void) WriteByte(image,'\n');
-                x=0;
+                i=0;
               }
+            p++;
           }
-          p++;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->packets))
-              ProgressMonitor(SaveImageText,i,image->packets);
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         break;
       }
@@ -818,25 +854,29 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PNM image.
         */
-        (void) sprintf(buffer,"%d\n",MaxRGB);
+        (void) sprintf(buffer,"%ld\n",MaxRGB);
         (void) WriteBlob(image,strlen(buffer),buffer);
-        for (i=0; i < (int) image->packets; i++)
+        i=0;
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          for (x=0; x < (int) image->columns; x++)
           {
             (void) sprintf(buffer,"%d %d %d ",p->red,p->green,p->blue);
             (void) WriteBlob(image,strlen(buffer),buffer);
-            x++;
-            if (x == 4)
+            i++;
+            if (i == 4)
               {
                 (void) WriteByte(image,'\n');
-                x=0;
+                i=0;
               }
+            p++;
           }
-          p++;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->packets))
-              ProgressMonitor(SaveImageText,i,image->packets);
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         break;
       }
@@ -854,14 +894,16 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         if (image->colors == 2)
           polarity=
             Intensity(image->colormap[0]) > Intensity(image->colormap[1]);
-        bit=0;
-        byte=0;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          bit=0;
+          byte=0;
+          for (x=0; x < (int) image->columns; x++)
           {
             byte<<=1;
-            if (p->index == polarity)
+            if (image->indexes[x] == polarity)
               byte|=0x01;
             bit++;
             if (bit == 8)
@@ -870,24 +912,13 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
                 bit=0;
                 byte=0;
               }
-            x++;
-            if (x == (int) image->columns)
-              {
-                /*
-                  Advance to the next scanline.
-                */
-                if (bit != 0)
-                  (void) WriteByte(image,byte << (8-bit));
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    ProgressMonitor(SaveImageText,y,image->rows);
-                bit=0;
-                byte=0;
-                x=0;
-                y++;
-             }
+            p++;
           }
-          p++;
+          if (bit != 0)
+            (void) WriteByte(image,byte << (8-bit));
+          if (image->previous == (Image *) NULL)
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         break;
       }
@@ -898,15 +929,20 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         */
         (void) sprintf(buffer,"%lu\n",DownScale(MaxRGB));
         (void) WriteBlob(image,strlen(buffer),buffer);
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          index=DownScale(Intensity(*p));
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          for (x=0; x < (int) image->columns; x++)
+          {
+            index=DownScale(Intensity(*p));
             (void) WriteByte(image,index);
-          p++;
+            p++;
+          }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->packets))
-              ProgressMonitor(SaveImageText,i,image->packets);
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         break;
       }
@@ -922,7 +958,7 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           Allocate memory for pixels.
         */
         pixels=(unsigned char *)
-          AllocateMemory(image->columns*sizeof(RunlengthPacket));
+          AllocateMemory(image->columns*sizeof(PixelPacket));
         if (pixels == (unsigned char *) NULL)
           WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
         /*
@@ -930,29 +966,25 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         */
         (void) sprintf(buffer,"%lu\n",DownScale(MaxRGB));
         (void) WriteBlob(image,strlen(buffer),buffer);
-        q=pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          q=pixels;
+          for (x=0; x < (int) image->columns; x++)
           {
             *q++=DownScale(p->red);
             *q++=DownScale(p->green);
             *q++=DownScale(p->blue);
-            x++;
-            if (x == (int) image->columns)
-              {
-                (void) WriteBlob(image,q-pixels,(char *) pixels);
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    ProgressMonitor(SaveImageText,y,image->rows);
-                q=pixels;
-                x=0;
-                y++;
-              }
+            p++;
           }
-          p++;
+          (void) WriteBlob(image,q-pixels,(char *) pixels);
+          if (image->previous == (Image *) NULL)
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
-        FreeMemory((char *) pixels);
+        FreeMemory(pixels);
         break;
       }
       case '7':
@@ -978,18 +1010,13 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           value;
 
         Quantum
-          blue,
-          green,
-          pixel,
-          red;
+          pixel;
 
         unsigned short
           *blue_map[2][16],
           *green_map[2][16],
           *red_map[2][16];
 
-        if (!UncondenseImage(image))
-          break;
         /*
           Allocate and initialize dither maps.
         */
@@ -1005,8 +1032,7 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             if ((red_map[i][j] == (unsigned short *) NULL) ||
                 (green_map[i][j] == (unsigned short *) NULL) ||
                 (blue_map[i][j] == (unsigned short *) NULL))
-              WriterExit(ResourceLimitWarning,"Memory allocation failed",
-                image);
+              WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
           }
         /*
           Initialize dither tables.
@@ -1019,20 +1045,18 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
               if (x < 48)
                 value=x/2+8;
               value+=dither_red[i][j];
-              red_map[i][j][x]=(unsigned short)
-                ((value < 0) ? 0 : (value > 255) ? 255 : value);
+              red_map[i][j][x]=((value < 0) ? 0 : (value > 255) ? 255 : value);
               value=x-16;
               if (x < 48)
                 value=x/2+8;
               value+=dither_green[i][j];
-              green_map[i][j][x]=(unsigned short)
+              green_map[i][j][x]=
                 ((value < 0) ? 0 : (value > 255) ? 255 : value);
               value=x-32;
               if (x < 112)
                 value=x/2+24;
-              value+=(dither_blue[i][j] << 1);
-              blue_map[i][j][x]=(unsigned short)
-                ((value < 0) ? 0 : (value > 255) ? 255 : value);
+              value+=2*dither_blue[i][j];
+              blue_map[i][j][x]=((value < 0) ? 0 : (value > 255) ? 255 : value);
             }
         /*
           Convert image to a P7 image.
@@ -1043,17 +1067,21 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlob(image,strlen(buffer),buffer);
         i=0;
         j=0;
-        p=image->pixels;
         for (y=0; y < (int) image->rows; y++)
         {
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
-            red=DownScale(p->red);
-            green=DownScale(p->green);
-            blue=DownScale(p->blue);
-            pixel=(Quantum) ((red_map[i][j][red] & 0xe0) |
-              ((unsigned int) (green_map[i][j][green] & 0xe0) >> 3) |
-              ((unsigned int) (blue_map[i][j][blue] & 0xc0) >> 6));
+            if (!image_info->dither)
+              pixel=((DownScale(p->red) & 0xe0) |
+                ((DownScale(p->green) & 0xe0) >> 3) |
+                ((DownScale(p->blue) & 0xc0) >> 6));
+            else
+              pixel=((red_map[i][j][DownScale(p->red)] & 0xe0) |
+                ((green_map[i][j][DownScale(p->green)] & 0xe0) >> 3) |
+                ((blue_map[i][j][DownScale(p->blue)] & 0xc0) >> 6));
             (void) WriteByte(image,pixel);
             p++;
             j++;
@@ -1072,17 +1100,16 @@ Export unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         for (i=0; i < 2; i++)
           for (j=0; j < 16; j++)
           {
-            FreeMemory((char *) green_map[i][j]);
-            FreeMemory((char *) blue_map[i][j]);
-            FreeMemory((char *) red_map[i][j]);
+            FreeMemory(green_map[i][j]);
+            FreeMemory(blue_map[i][j]);
+            FreeMemory(red_map[i][j]);
           }
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
   if (image_info->adjoin)
