@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -146,9 +147,9 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
   (void) ParseImageGeometry(DefaultPreviewGeometry,&x,&y,&width,&height);
   image->orphan=True;
   preview_image=ZoomImage(image,width,height);
-  image->orphan=False;
   if (preview_image == (Image *) NULL)
     WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
+  preview_image->exempt=True;
   LabelImage(preview_image,DefaultTileLabel);
   /*
     Apply enhancement at varying strengths.
@@ -167,8 +168,8 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
   commands[0]=SetClientName((char *) NULL);
   for (i=0; i < NumberTiles; i++)
   {
-    images[i]=
-      CloneImage(preview_image,preview_image->columns,preview_image->rows,True);
+    images[i]=CloneImage(preview_image,preview_image->columns,
+      preview_image->rows,True);
     if (images[i] == (Image *) NULL)
       {
         for (x=0;  x < i; x++)
@@ -179,7 +180,7 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
     if (i == (NumberTiles >> 1))
       {
         commands[argc++]="-mattecolor";
-        commands[argc++]="#dfdfdf";;
+        commands[argc++]="#dfdfdf";
         MogrifyImage(local_info,argc,commands,&images[i]);
         continue;
       }
@@ -192,8 +193,6 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
         FormatString(label,"rotate %.1024s",factor);
         commands[argc++]="-rotate";
         commands[argc++]=factor;
-        commands[argc++]="-crop";
-        commands[argc++]="0x0";
         break;
       }
       case ShearPreview:
@@ -203,8 +202,6 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
         FormatString(label,"shear %.1024s",factor);
         commands[argc++]="-shear";
         commands[argc++]=factor;
-        commands[argc++]="-crop";
-        commands[argc++]="0x0";
         break;
       }
       case RollPreview:
@@ -366,8 +363,10 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
       {
         if (i == 0)
           {
-            (void) strcpy(label,"+shade");
+            FormatString(factor,"30.0x30.0");
+            FormatString(label,"+shade %.1024s",factor);
             commands[argc++]="+shade";
+            commands[argc++]=factor;
             break;
           }
         degrees+=10.0;
@@ -379,7 +378,7 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
       }
       case RaisePreview:
       {
-        FormatString(factor,"%d",i+1);
+        FormatString(factor,"%d",2*i+2);
         FormatString(label,"raise %.1024s",factor);
         commands[argc++]="-raise";
         commands[argc++]=factor;
@@ -413,9 +412,9 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
       case WavePreview:
       {
         degrees+=5.0;
-        FormatString(factor,"%.1fx%.1f",degrees,degrees);
+        FormatString(factor,"%.1fx%.1f",0.5*degrees,2.0*degrees);
         FormatString(label,"wave %.1024s",factor);
-        commands[argc++]="-implode";
+        commands[argc++]="-wave";
         commands[argc++]=factor;
         break;
       }
@@ -429,7 +428,7 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
       }
       case CharcoalDrawingPreview:
       {
-        FormatString(factor,"%.1f",percentage);
+        FormatString(factor,"%.1f",percentage+5.0);
         FormatString(label,"charcoal %.1024s",factor);
         commands[argc++]="-charcoal";
         commands[argc++]=factor;
@@ -486,16 +485,15 @@ Export unsigned int WritePREVIEWImage(const ImageInfo *image_info,Image *image)
   }
   GetMontageInfo(&montage_info);
   (void) strcpy(montage_info.filename,image->filename);
-  (void) CloneString(&montage_info.geometry,DefaultPreviewGeometry);
-  (void) CloneString(&montage_info.tile,"3x3");
-  (void) CloneString(&montage_info.font,image_info->font);
   montage_info.pointsize=image_info->pointsize;
-  (void) CloneString(&montage_info.frame,DefaultTileFrame);
   montage_info.shadow=True;
+  (void) CloneString(&montage_info.font,image_info->font);
+  (void) CloneString(&montage_info.tile,"3x3");
+  (void) CloneString(&montage_info.geometry,DefaultPreviewGeometry);
+  (void) CloneString(&montage_info.frame,DefaultTileFrame);
   montage_image=MontageImages(*images,&montage_info);
   DestroyMontageInfo(&montage_info);
-  for (i=0;  i < i; i++)
-    DestroyImage(images[i]);
+  DestroyImages(*images);
   if (montage_image == (Image *) NULL)
     WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
   if (montage_image->montage != (char *) NULL)
