@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -75,9 +76,9 @@
 %  centered on the Postscript page.  Otherwise, the image is positioned as
 %  specified by the geometry.
 %
-%  The format of the WritePS2Image routine is:
+%  The format of the WritePS2Image method is:
 %
-%      status=WritePS2Image(image_info,image)
+%      unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -248,6 +249,7 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     buffer[MaxTextExtent],
     date[MaxTextExtent],
     density[MaxTextExtent],
+    geometry[MaxTextExtent],
     **labels;
 
   const char
@@ -270,12 +272,11 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     x,
     y;
 
-  register RunlengthPacket
+  register PixelPacket
     *p;
 
   register int
-    i,
-    j;
+    i;
 
   SegmentInfo
     bounding_box;
@@ -302,11 +303,9 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryType);
   if (status == False)
     WriterExit(FileOpenWarning,"Unable to open file",image);
-  compression=image_info->compression;
-#if defined(HasLZW)
-  if (compression == UndefinedCompression)
-    compression=LZWCompression;
-#endif
+  compression=image->compression;
+  if (image_info->compression != UndefinedCompression)
+    compression=image_info->compression;
   page=1;
   scene=0;
   do
@@ -321,14 +320,17 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     height=image->rows;
     x=0;
     y=text_size;
+    FormatString(geometry,"%ux%u",image->columns,image->rows);
     if (image_info->page != (char *) NULL)
-      (void) ParseImageGeometry(image_info->page,&x,&y,&width,&height);
+      (void) strcpy(geometry,image_info->page);
     else
-      if (image->page != (char *) NULL)
-        (void) ParseImageGeometry(image->page,&x,&y,&width,&height);
+      if ((image->page_info.width != 0) && (image->page_info.height != 0))
+        (void) FormatString(geometry,"%ux%u%+d%+d",image->page_info.width,
+	  image->page_info.height,image->page_info.x,image->page_info.y);
       else
         if (Latin1Compare(image_info->magick,"PS2") == 0)
-          (void) ParseImageGeometry(PSPageGeometry,&x,&y,&width,&height);
+          (void) strcpy(geometry,PSPageGeometry);
+    (void) ParseImageGeometry(geometry,&x,&y,&width,&height);
     /*
       Scale relative to dots-per-inch.
     */
@@ -417,9 +419,13 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
         */
         for (q=PostscriptProlog; *q; q++)
         {
-          (void) sprintf(buffer,*q,
-            compression == ZipCompression ? "FlateDecode" :
-            compression == LZWCompression ? "LZWDecode" : "RunLengthDecode");
+          switch (compression)
+          {
+            case JPEGCompression: sprintf(buffer,*q,"DCTDecode"); break;
+            case LZWCompression: sprintf(buffer,*q,"LZWDecode"); break;
+            case ZipCompression: sprintf(buffer,*q,"FlateDecode"); break;
+            default: sprintf(buffer,*q,"RunLengthDecode"); break;
+          }
           (void) WriteBlob(image,strlen(buffer),buffer);
           (void) WriteByte(image,'\n');
         }
@@ -429,7 +435,7 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           (void) WriteBlob(image,strlen(buffer),buffer);
           (void) strcpy(buffer,"  currentfile label readline pop\n");
           (void) WriteBlob(image,strlen(buffer),buffer);
-          (void) sprintf(buffer,"  0 y %d add moveto label show pop\n",
+          (void) sprintf(buffer,"  0 y %f add moveto label show pop\n",
             i*image_info->pointsize+12);
           (void) WriteBlob(image,strlen(buffer),buffer);
         }
@@ -479,7 +485,7 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       Output image data.
     */
     labels=StringToList(image->label);
-    (void) sprintf(buffer,"%d %d\n%g %g\n%u\n",x,y,x_scale,y_scale,
+    (void) sprintf(buffer,"%d %d\n%g %g\n%f\n",x,y,x_scale,y_scale,
       image_info->pointsize);
     (void) WriteBlob(image,strlen(buffer),buffer);
     if (labels != (char **) NULL)
@@ -496,10 +502,46 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       IsPseudoClass(image),(int) (image->colorspace == CMYKColorspace),
       (int) (compression == NoCompression));
     (void) WriteBlob(image,strlen(buffer),buffer);
-    p=image->pixels;
     if (!IsPseudoClass(image) && !IsGrayImage(image))
       switch (compression)
       {
+        case JPEGCompression:
+        {
+          char
+            filename[MaxTextExtent];
+
+          FILE
+            *file;
+
+          Image
+            *jpeg_image;
+
+          int
+            c;
+
+          /*
+            Write image to temporary file in JPEG format.
+          */
+          TemporaryFilename(filename);
+          jpeg_image=CloneImage(image,image->columns,image->rows,True);
+          if (jpeg_image == (Image *) NULL)
+            WriterExit(DelegateWarning,"Unable to clone image",image);
+          (void) strcpy(jpeg_image->filename,filename);
+          status=WriteJPEGImage(image_info,jpeg_image);
+          DestroyImage(jpeg_image);
+          if (status == False)
+            WriterExit(DelegateWarning,"Unable to write image",image);
+          file=fopen(filename,ReadBinaryType);
+          if (file == (FILE *) NULL)
+            WriterExit(FileOpenWarning,"Unable to open file",image);
+          Ascii85Initialize();
+          for (c=fgetc(file); c != EOF; c=fgetc(file))
+            Ascii85Encode(image,c);
+          Ascii85Flush(image);
+          (void) fclose(file);
+          (void) remove(filename);
+          break;
+        }
         case RunlengthEncodedCompression:
         default:
         {
@@ -514,17 +556,19 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           pixels=(unsigned char *)
             AllocateMemory(number_packets*sizeof(unsigned char));
           if (pixels == (unsigned char *) NULL)
-            WriterExit(ResourceLimitWarning,"Memory allocation failed",
-              image);
+            WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
           /*
             Dump Packbit encoded pixels.
           */
           q=pixels;
-          for (i=0; i < (int) image->packets; i++)
+          for (y=0; y < (int) image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            p=GetPixelCache(image,0,y,image->columns,1);
+            if (p == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (int) image->columns; x++)
             {
-              if (image->matte && (p->index == Transparent))
+              if (image->matte && (p->opacity == Transparent))
                 {
                   *q++=DownScale(MaxRGB);
                   *q++=DownScale(MaxRGB);
@@ -542,13 +586,13 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                     *q++=DownScale(p->red);
                     *q++=DownScale(p->green);
                     *q++=DownScale(p->blue);
-                    *q++=DownScale(p->index);
+                    *q++=DownScale(p->opacity);
                   }
+              p++;
             }
-            p++;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(i,image->packets))
-                ProgressMonitor(SaveImageText,i,image->packets);
+              if (QuantumTick(y,image->rows))
+                ProgressMonitor(SaveImageText,y,image->rows);
           }
           if (compression == ZipCompression)
             status=
@@ -563,7 +607,7 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               CloseBlob(image);
               return(False);
             }
-          FreeMemory((char *) pixels);
+          FreeMemory(pixels);
           break;
         }
         case NoCompression:
@@ -572,11 +616,14 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
             Dump uncompressed DirectColor packets.
           */
           Ascii85Initialize();
-          for (i=0; i < (int) image->packets; i++)
+          for (y=0; y < (int) image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            p=GetPixelCache(image,0,y,image->columns,1);
+            if (p == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (int) image->columns; x++)
             {
-              if (image->matte && (p->index == Transparent))
+              if (image->matte && (p->opacity == Transparent))
                 {
                   Ascii85Encode(image,DownScale(MaxRGB));
                   Ascii85Encode(image,DownScale(MaxRGB));
@@ -594,13 +641,13 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                     Ascii85Encode(image,DownScale(p->red));
                     Ascii85Encode(image,DownScale(p->green));
                     Ascii85Encode(image,DownScale(p->blue));
-                    Ascii85Encode(image,DownScale(p->index));
+                    Ascii85Encode(image,DownScale(p->opacity));
                   }
+              p++;
             }
-            p++;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(i,image->packets))
-                ProgressMonitor(SaveImageText,i,image->packets);
+              if (QuantumTick(y,image->rows))
+                ProgressMonitor(SaveImageText,y,image->rows);
           }
           Ascii85Flush(image);
           break;
@@ -618,9 +665,6 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           if (image->colors == 2)
             polarity=
               Intensity(image->colormap[0]) < Intensity(image->colormap[1]);
-          bit=0;
-          byte=0;
-          x=0;
           (void) WriteByte(image,'\0');
           (void) WriteByte(image,'\n');
           switch (compression)
@@ -640,12 +684,16 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 Dump uncompressed PseudoColor packets.
               */
               Ascii85Initialize();
-              for (i=0; i < (int) image->packets; i++)
+              for (y=0; y < (int) image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
+                if (!GetPixelCache(image,0,y,image->columns,1))
+                  break;
+                bit=0;
+                byte=0;
+                for (x=0; x < (int) image->columns; x++)
                 {
                   byte<<=1;
-                  if (p->index == polarity)
+                  if (image->indexes[x] == polarity)
                     byte|=0x01;
                   bit++;
                   if (bit == 8)
@@ -654,23 +702,13 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                       bit=0;
                       byte=0;
                     }
-                  x++;
-                  if (x == (int) image->columns)
-                    {
-                      /*
-                        Advance to the next scanline.
-                      */
-                      if (bit != 0)
-                        Ascii85Encode(image,byte << (8-bit));
-                      if (image->previous == (Image *) NULL)
-                        if (QuantumTick(y,image->rows))
-                          ProgressMonitor(SaveImageText,y,image->rows);
-                      bit=0;
-                      byte=0;
-                      x=0;
-                   }
+                  p++;
                 }
-                p++;
+                if (bit != 0)
+                  Ascii85Encode(image,byte << (8-bit));
+                if (image->previous == (Image *) NULL)
+                  if (QuantumTick(y,image->rows))
+                    ProgressMonitor(SaveImageText,y,image->rows);
               }
               Ascii85Flush(image);
               break;
@@ -713,14 +751,18 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 Dump Runlength encoded pixels.
               */
               q=pixels;
-              for (i=0; i < (int) image->packets; i++)
+              for (y=0; y < (int) image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
-                  *q++=(unsigned char) p->index;
-                p++;
+                if (!GetPixelCache(image,0,y,image->columns,1))
+                  break;
+                for (x=0; x < (int) image->columns; x++)
+                {
+                  *q++=image->indexes[x];
+                  p++;
+                }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(i,image->packets))
-                    ProgressMonitor(SaveImageText,i,image->packets);
+                  if (QuantumTick(y,image->rows))
+                    ProgressMonitor(SaveImageText,y,image->rows);
               }
               if (compression == ZipCompression)
                 status=ZLIBEncodeImage(image,number_packets,image_info->quality,
@@ -730,7 +772,7 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   status=LZWEncodeImage(image,number_packets,pixels);
                 else
                   status=PackbitsEncodeImage(image,number_packets,pixels);
-              FreeMemory((char *) pixels);
+              FreeMemory(pixels);
               if (!status)
                 {
                   CloseBlob(image);
@@ -744,14 +786,15 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 Dump uncompressed PseudoColor packets.
               */
               Ascii85Initialize();
-              for (i=0; i < (int) image->packets; i++)
+              for (y=0; y < (int) image->rows; y++)
               {
-                for (j=0; j <= ((int) p->length); j++)
-                  Ascii85Encode(image,(unsigned char) p->index);
-                p++;
+                if (!GetPixelCache(image,0,y,image->columns,1))
+                  break;
+                for (x=0; x < (int) image->columns; x++)
+                  Ascii85Encode(image,image->indexes[x]);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(i,image->packets))
-                    ProgressMonitor(SaveImageText,i,image->packets);
+                  if (QuantumTick(y,image->rows))
+                    ProgressMonitor(SaveImageText,y,image->rows);
               }
               Ascii85Flush(image);
               break;
@@ -770,8 +813,7 @@ Export unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlob(image,strlen(buffer),buffer);
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
   if (image_info->adjoin)
