@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -93,10 +94,13 @@
 */
 Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
 {
+#define CFormat  "/%s filter "
+
   char
     buffer[MaxTextExtent],
     date[MaxTextExtent],
-    density[MaxTextExtent];
+    density[MaxTextExtent],
+    geometry[MaxTextExtent];
 
   CompressionType
     compression;
@@ -115,13 +119,9 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     x,
     y;
 
-  register RunlengthPacket
+  register PixelPacket
     *p;
 
-  register int
-    i,
-    j;
-
   SegmentInfo
     bounding_box;
 
@@ -149,11 +149,9 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryType);
   if (status == False)
     WriterExit(FileOpenWarning,"Unable to open file",image);
-  compression=image_info->compression;
-#if defined(HasLZW)
-  if (compression == UndefinedCompression)
-    compression=LZWCompression;
-#endif
+  compression=image->compression;
+  if (image_info->compression != UndefinedCompression)
+    compression=image_info->compression;
   page=1;
   scene=0;
   do
@@ -168,13 +166,17 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     height=image->rows;
     x=0;
     y=text_size;
+    FormatString(geometry,"%ux%u",image->columns,image->rows);
     if (image_info->page != (char *) NULL)
-      (void) ParseImageGeometry(image_info->page,&x,&y,&width,&height);
+      (void) strcpy(geometry,image_info->page);
     else
-      if (image->page != (char *) NULL)
-        (void) ParseImageGeometry(image->page,&x,&y,&width,&height);
+      if ((image->page_info.width != 0) && (image->page_info.height != 0))
+        (void) FormatString(geometry,"%ux%u%+d%+d",image->page_info.width,
+	  image->page_info.height,image->page_info.x,image->page_info.y);
       else
-        (void) ParseImageGeometry(PSPageGeometry,&x,&y,&width,&height);
+        if (Latin1Compare(image_info->magick,"PDF") == 0)
+          (void) strcpy(geometry,PSPageGeometry);
+    (void) ParseImageGeometry(geometry,&x,&y,&width,&height);
     /*
       Scale relative to dots-per-inch.
     */
@@ -264,16 +266,56 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlob(image,strlen(buffer),buffer);
     if (compression != NoCompression)
       {
-        (void) sprintf(buffer,"/%s filter ",
-          compression == ZipCompression ? "FlateDecode" :
-          compression == LZWCompression ? "LZWDecode" : "RunLengthDecode");
+        switch (compression)
+        {
+          case JPEGCompression: sprintf(buffer,CFormat,"DCTDecode"); break;
+          case LZWCompression: sprintf(buffer,CFormat,"LZWDecode"); break;
+          case ZipCompression: sprintf(buffer,CFormat,"FlateDecode"); break;
+          default: sprintf(buffer,CFormat,"RunLengthDecode"); break;
+        }
         (void) WriteBlob(image,strlen(buffer),buffer);
       }
     (void) strcpy(buffer,"/ReusableStreamDecode filter\n");
     (void) WriteBlob(image,strlen(buffer),buffer);
-    p=image->pixels;
     switch (compression)
     {
+      case JPEGCompression:
+      {
+        char
+          filename[MaxTextExtent];
+
+        FILE
+          *file;
+
+        Image
+          *jpeg_image;
+
+        int
+          c;
+
+        /*
+          Write image to temporary file in JPEG format.
+        */
+        TemporaryFilename(filename);
+        jpeg_image=CloneImage(image,image->columns,image->rows,True);
+        if (jpeg_image == (Image *) NULL)
+          WriterExit(DelegateWarning,"Unable to clone image",image);
+        (void) strcpy(jpeg_image->filename,filename);
+        status=WriteJPEGImage(image_info,jpeg_image);
+        DestroyImage(jpeg_image);
+        if (status == False)
+          WriterExit(DelegateWarning,"Unable to write image",image);
+        file=fopen(filename,ReadBinaryType);
+        if (file == (FILE *) NULL)
+          WriterExit(FileOpenWarning,"Unable to open file",image);
+        Ascii85Initialize();
+        for (c=fgetc(file); c != EOF; c=fgetc(file))
+          Ascii85Encode(image,c);
+        Ascii85Flush(image);
+        (void) fclose(file);
+        (void) remove(filename);
+        break;
+      }
       case RunlengthEncodedCompression:
       default:
       {
@@ -287,25 +329,27 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         pixels=(unsigned char *)
           AllocateMemory(number_packets*sizeof(unsigned char));
         if (pixels == (unsigned char *) NULL)
-          WriterExit(ResourceLimitWarning,"Memory allocation failed",
-            image);
+          WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
         /*
           Dump Packbit encoded pixels.
         */
         q=pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          for (x=0; x < (int) image->columns; x++)
           {
-              *q++=DownScale(p->index);
-              *q++=DownScale(p->red);
-              *q++=DownScale(p->green);
-              *q++=DownScale(p->blue);
+            *q++=DownScale(p->opacity);
+            *q++=DownScale(p->red);
+            *q++=DownScale(p->green);
+            *q++=DownScale(p->blue);
+            p++;
           }
-          p++;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->packets))
-              ProgressMonitor(SaveImageText,i,image->packets);
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         if (compression == ZipCompression)
           status=
@@ -320,7 +364,7 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
             CloseBlob(image);
             return(False);
           }
-        FreeMemory((char *) pixels);
+        FreeMemory(pixels);
         break;
       }
       case NoCompression:
@@ -329,19 +373,22 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
           Dump uncompressed DirectColor packets.
         */
         Ascii85Initialize();
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          for (x=0; x < (int) image->columns; x++)
           {
-            Ascii85Encode(image,DownScale(p->index));
+            Ascii85Encode(image,DownScale(p->opacity));
             Ascii85Encode(image,DownScale(p->red));
             Ascii85Encode(image,DownScale(p->green));
             Ascii85Encode(image,DownScale(p->blue));
+            p++;
           }
-          p++;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,image->packets))
-              ProgressMonitor(SaveImageText,i,image->packets);
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
         Ascii85Flush(image);
         break;
@@ -433,8 +480,7 @@ Export unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlob(image,strlen(buffer),buffer);
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
   if (image_info->adjoin)
