@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,32 +60,32 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P a c k b i t s D e c o d e I m a g e                                     %
+%   D e c o d e I m a g e                                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method PackbitsDecodeImage uncompresses an image via Macintosh Packbits
-%  encoding specific to the Adobe Photoshop image format.
+%  Method DecodeImage uncompresses an image via Macintosh encoding specific to
+%  the Adobe Photoshop image format.
 %
-%  The format of the PackbitsDecodeImage routine is:
+%  The format of the DecodeImage method is:
 %
-%      status=PackbitsDecodeImage(image,channel)
+%      unsigned int DecodeImage(Image *image,const int channel)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method PackbitsDecodeImage return True if the image is
+%    o status: Method DecodeImage return True if the image is
 %      decoded.  False is returned if there is an error occurs.
 %
-%    o image: The address of a structure of type Image.
+%    o image,image: The address of a structure of type Image.
 %
 %    o channel:  Specifies which channel: red, green, blue, or index to
 %      decode the pixel values into.
 %
 %
 */
-static unsigned int PackbitsDecodeImage(Image *image,const int channel)
+static unsigned int DecodeImage(Image *image,const int channel)
 {
   int
     count,
@@ -94,12 +95,13 @@ static unsigned int PackbitsDecodeImage(Image *image,const int channel)
     length;
 
   register int
-    i;
+    i,
+    x;
 
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
-  q=image->pixels;
+  x=0;
   length=image->columns*image->rows;
   while (length > 0)
   {
@@ -110,38 +112,44 @@ static unsigned int PackbitsDecodeImage(Image *image,const int channel)
       {
         if (count == -128)
           continue;
-        count=(-count+1);
         pixel=ReadByte(image);
-        for ( ; count > 0; count--)
+        for (count=(-count+1); count > 0; count--)
         {
+          q=SetPixelCache(image,x % image->columns,x/image->columns,1,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           switch (channel)
           {
             case 0:
             {
-              q->red=(Quantum) pixel;
+              q->red=pixel;
               if (image->class == PseudoClass)
-                q->index=(unsigned short) pixel;
+                {
+                  *image->indexes=pixel;
+                  *q=image->colormap[pixel];
+                }
               break;
             }
             case 1:
             {
-              q->green=(Quantum) pixel;
+              q->green=pixel;
               break;
             }
             case 2:
             {
-              q->blue=(Quantum) pixel;
+              q->blue=pixel;
               break;
             }
             case 3:
             default:
             {
-              q->index=(unsigned short) pixel;
+              q->opacity=pixel;
               break;
             }
           }
-          q->length=0;
-          q++;
+          if (!SyncPixelCache(image))
+            break;
+          x++;
           length--;
         }
         continue;
@@ -150,34 +158,41 @@ static unsigned int PackbitsDecodeImage(Image *image,const int channel)
     for (i=count; i > 0; i--)
     {
       pixel=ReadByte(image);
+      q=SetPixelCache(image,x % image->columns,x/image->columns,1,1);
+      if (q == (PixelPacket *) NULL)
+        break;
       switch (channel)
       {
         case 0:
         {
-          q->red=(Quantum) pixel;
+          q->red=pixel;
           if (image->class == PseudoClass)
-            q->index=(unsigned short) pixel;
+            {
+              *image->indexes=pixel;
+              *q=image->colormap[pixel];
+            }
           break;
         }
         case 1:
         {
-          q->green=(Quantum) pixel;
+          q->green=pixel;
           break;
         }
         case 2:
         {
-          q->blue=(Quantum) pixel;
+          q->blue=pixel;
           break;
         }
         case 3:
         default:
         {
-          q->index=(unsigned short) pixel;
+          q->opacity=pixel;
           break;
         }
       }
-      q->length=0;
-      q++;
+      if (!SyncPixelCache(image))
+        break;
+      x++;
       length--;
     }
   }
@@ -205,6 +220,45 @@ static unsigned int PackbitsDecodeImage(Image *image,const int channel)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s P S D                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method IsPSD returns True if the image format type, identified by the
+%  magick string, is PSD.
+%
+%  The format of the ReadPSDImage method is:
+%
+%      unsigned int IsPSD(const unsigned char *magick,
+%        const unsigned int length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsPSD returns True if the image format type is PSD.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+Export unsigned int IsPSD(const unsigned char *magick,const unsigned int length)
+{
+  if (length < 4)
+    return(False);
+  if (strncmp((char *) magick,"8BPS",4) == 0)
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e a d P S D I m a g e                                                   %
 %                                                                             %
 %                                                                             %
@@ -215,7 +269,7 @@ static unsigned int PackbitsDecodeImage(Image *image,const int channel)
 %  It allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
-%  The format of the ReadPSDImage routine is:
+%  The format of the ReadPSDImage method is:
 %
 %      image=ReadPSDImage(image_info)
 %
@@ -300,11 +354,14 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
   Image
     *image;
 
+  int
+    j,
+    y;
+
   LayerInfo
     *layer_info;
 
   long
-    count,
     length,
     size;
 
@@ -313,16 +370,19 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
 
   register int
     i,
-    j,
-    k;
+    x;
 
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
   short int
     number_layers;
 
+  unsigned char
+    *scanline;
+
   unsigned int
+    packet_size,
     status;
 
   unsigned short
@@ -373,10 +433,10 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
         Create colormap.
       */
       image->class=PseudoClass;
-      image->colors=1 << image->depth;
-      image->colormap=(ColorPacket *)
-        AllocateMemory(image->colors*sizeof(ColorPacket));
-      if (image->colormap == (ColorPacket *) NULL)
+      image->colors=256;
+      image->colormap=(PixelPacket *)
+        AllocateMemory(image->colors*sizeof(PixelPacket));
+      if (image->colormap == (PixelPacket *) NULL)
         ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
       for (i=0; i < (int) image->colors; i++)
       {
@@ -390,42 +450,30 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
             Read PSD raster colormap.
           */
           for (i=0; i < (int) image->colors; i++)
-            image->colormap[i].red=ReadByte(image);
+            image->colormap[i].red=UpScale(ReadByte(image));
           for (i=0; i < (int) image->colors; i++)
-            image->colormap[i].green=ReadByte(image);
+            image->colormap[i].green=UpScale(ReadByte(image));
           for (i=0; i < (int) image->colors; i++)
-            image->colormap[i].blue=ReadByte(image);
+            image->colormap[i].blue=UpScale(ReadByte(image));
         }
     }
   length=MSBFirstReadLong(image);
-  while (length > 0)
-  {
-    /*
-      Read image resource block.
-    */
-    status=ReadBlob(image,4,(char *) type);
-    if ((status == False) || (strncmp(type,"8BIM",4) != 0))
-      ReaderExit(CorruptImageWarning,"Not a PSD image file",image);
-    (void) MSBFirstReadShort(image);
-    count=ReadByte(image);
-    if (count > 0)
-      for (i=0; i < count; i++)
-        (void) ReadByte(image);
-    if (!(count & 0x01))
-      {
-        (void) ReadByte(image);
-        length--;
-      }
-    size=MSBFirstReadLong(image);
-    for (i=0; i < size; i++)
-      (void) ReadByte(image);
-    length-=(count+size+11);
-    if (size & 0x01)
-      {
-        (void) ReadByte(image);
-        length--;
-      }
-  }
+  if (length > 0)
+    {
+      unsigned char
+        *data;
+
+      data=(unsigned char *)
+        AllocateMemory(length*sizeof(unsigned char));
+      if (data == (unsigned char *) NULL)
+        ReaderExit(ResourceLimitWarning,
+          "8BIM resource memory allocation failed",image);
+      status=ReadBlob(image,length,(char *) data);
+      if ((status == False) || (strncmp((char *) data,"8BIM",4) != 0))
+        ReaderExit(CorruptImageWarning,"Not a PSD image file",image);
+      image->iptc_profile.info=data;
+      image->iptc_profile.length=length;
+    }
   if (image_info->ping)
     {
       CloseBlob(image);
@@ -473,63 +521,34 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
         /*
           Allocate layered image.
         */
-        layer_info[i].image=AllocateImage(image_info);
+        layer_info[i].image=
+          CloneImage(image,layer_info[i].width,layer_info[i].height,True);
         if (layer_info[i].image == (Image *) NULL)
           {
             for (j=0; j < i; j++)
               DestroyImage(layer_info[j].image);
-            ReaderExit(ResourceLimitWarning,"Memory allocation failed",
-              image);
+            ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
           }
-        layer_info[i].image->file=image->file;
-        layer_info[i].image->class=image->class;
         if (psd_header.mode == CMYKMode)
           layer_info[i].image->colorspace=CMYKColorspace;
         else
           layer_info[i].image->matte=layer_info[i].channels >= 4;
-        if (image->colormap != (ColorPacket *) NULL)
-          {
-            /*
-              Convert pixels to Runlength encoded.
-            */
-            layer_info[i].image->colormap=(ColorPacket *)
-              AllocateMemory(image->colors*sizeof(ColorPacket));
-            if (layer_info[i].image->colormap == (ColorPacket *) NULL)
-              {
-                for (j=0; j < i; j++)
-                  DestroyImage(layer_info[j].image);
-                ReaderExit(ResourceLimitWarning,"Memory allocation failed",
-                  image);
-              }
-          }
-        layer_info[i].image->columns=layer_info[i].width;
-        layer_info[i].image->rows=layer_info[i].height;
-        layer_info[i].image->packets=
-          layer_info[i].image->columns*layer_info[i].image->rows;
-        layer_info[i].image->pixels=(RunlengthPacket *) AllocateMemory(
-          (layer_info[i].image->packets+256)*sizeof(RunlengthPacket));
-        if (layer_info[i].image->pixels == (RunlengthPacket *) NULL)
-          {
-            for (j=0; j < i; j++)
-              DestroyImage(layer_info[j].image);
-            ReaderExit(ResourceLimitWarning,"Memory allocation failed",
-              image);
-          }
-        SetImage(layer_info[i].image);
       }
       /*
         Read pixel data for each layer.
       */
       for (i=0; i < number_layers; i++)
       {
+        layer_info[i].image->file=image->file;
+        layer_info[i].image->blob_info=image->blob_info;
         for (j=0; j < (int) layer_info[i].channels; j++)
         {
           compression=MSBFirstReadShort(layer_info[i].image);
           if (compression != 0)
             {
-              for (k=0; k < (int) layer_info[i].image->rows; k++)
-                (void) MSBFirstReadShort(image);
-              (void) PackbitsDecodeImage(layer_info[i].image,
+              for (y=0; y < (int) layer_info[i].image->rows; y++)
+                (void) MSBFirstReadShort(layer_info[i].image);
+              (void) DecodeImage(layer_info[i].image,
                 layer_info[i].channel_info[j].type);
             }
           else
@@ -537,70 +556,112 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
               /*
                 Read uncompressed pixel data as separate planes.
               */
-              q=layer_info[i].image->pixels;
-              for (k=0; k < (int) layer_info[i].image->packets; k++)
+              packet_size=1;
+              if (layer_info[i].image->class == PseudoClass)
+                {
+                  if (layer_info[i].image->colors > 256)
+                    packet_size++;
+                }
+              else
+                if (layer_info[i].image->depth > 8)
+                  packet_size++;
+              scanline=(unsigned char *) AllocateMemory((packet_size*
+                layer_info[i].image->columns+1)*sizeof(unsigned char));
+              if (scanline == (unsigned char *) NULL)
+                ReaderExit(ResourceLimitWarning,"Memory allocation failed",
+                  image);
+              for (y=0; y < (int) layer_info[i].image->rows; y++)
               {
+                q=SetPixelCache(layer_info[i].image,0,y,
+                  layer_info[i].image->columns,1);
+                if (q == (PixelPacket *) NULL)
+                  break;
+                (void) ReadBlob(layer_info[i].image,packet_size*
+                  layer_info[i].image->columns,(char *) scanline);
                 switch (layer_info[i].channel_info[j].type)
                 {
                   case 0:
                   {
-                    ReadQuantumFile(q->red);
-                    q->index=q->red;
+                    if (layer_info[i].image->class == PseudoClass)
+                      (void) ReadPixelCache(layer_info[i].image,IndexQuantum,
+                        scanline);
+                    else
+                      (void) ReadPixelCache(layer_info[i].image,RedQuantum,
+                        scanline);
                     break;
                   }
                   case 1:
                   {
-                    ReadQuantumFile(q->green);
+                    (void) ReadPixelCache(layer_info[i].image,GreenQuantum,
+                      scanline);
                     break;
                   }
                   case 2:
                   {
-                    ReadQuantumFile(q->blue);
+                    (void) ReadPixelCache(layer_info[i].image,BlueQuantum,
+                      scanline);
                     break;
                   }
                   case 3:
                   default:
                   {
-                    ReadQuantumFile(q->index);
+                    (void) ReadPixelCache(layer_info[i].image,OpacityQuantum,
+                      scanline);
                     break;
                   }
                 }
-                q->length=0;
-                q++;
+                if (!SyncPixelCache(layer_info[i].image))
+                  break;
               }
+              FreeMemory(scanline);
             }
         }
-        if (layer_info[i].image->class == PseudoClass)
-          SyncImage(layer_info[i].image);
-        else
-          if (layer_info[i].image->colorspace == CMYKColorspace)
+        image->file=layer_info[i].image->file;
+        image->blob_info=layer_info[i].image->blob_info;
+        if (layer_info[i].image->colorspace == CMYKColorspace)
+          {
+            /*
+              Correct CMYK levels.
+            */
+            for (y=0; y < (int) layer_info[i].image->rows; y++)
             {
-              /*
-                Correct CMYK levels.
-              */
-              q=layer_info[i].image->pixels;
-              for (k=0; k < (int) layer_info[i].image->packets; k++)
+              q=SetPixelCache(layer_info[i].image,0,y,
+                layer_info[i].image->columns,1);
+              if (q == (PixelPacket *) NULL)
+                break;
+              for (x=0; x < (int) layer_info[i].image->columns; x++)
               {
                 q->red=MaxRGB-q->red;
                 q->green=MaxRGB-q->green;
                 q->blue=MaxRGB-q->blue;
-                q->index=MaxRGB-q->index;
+                q->opacity=MaxRGB-q->opacity;
                 q++;
               }
+              if (!SyncPixelCache(layer_info[i].image))
+                break;
             }
-          else
-            if (layer_info[i].opacity != Opaque)
+          }
+        else
+          if (layer_info[i].opacity != Opaque)
+            {
+              /*
+                Correct for opacity level.
+              */
+              for (y=0; y < (int) layer_info[i].image->rows; y++)
               {
-                /*
-                  Correct for opacity level.
-                */
-                q=layer_info[i].image->pixels;
-                for (k=0; k < (int) layer_info[i].image->packets; k++)
+                q=GetPixelCache(layer_info[i].image,0,y,
+                  layer_info[i].image->columns,1);
+                if (q == (PixelPacket *) NULL)
+                  break;
+                for (x=0; x < (int) layer_info[i].image->columns; x++)
                 {
-                  q->index=(int) (q->index*layer_info[i].opacity)/Opaque;
+                  q->opacity=(int) (q->opacity*layer_info[i].opacity)/Opaque;
                   q++;
                 }
+                if (!SyncPixelCache(layer_info[i].image))
+                  break;
               }
+            }
         layer_info[i].image->file=(FILE *) NULL;
       }
       for (i=0; i < 4; i++)
@@ -610,11 +671,6 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
     Convert pixels to Runlength encoded.
   */
   compression=MSBFirstReadShort(image);
-  image->packets=image->columns*image->rows;
-  image->pixels=(RunlengthPacket *)
-    AllocateMemory((image->packets+256)*sizeof(RunlengthPacket));
-  if (image->pixels == (RunlengthPacket *) NULL)
-    ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
   SetImage(image);
   if (compression != 0)
     {
@@ -624,81 +680,110 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
       for (i=0; i < (int) (image->rows*psd_header.channels); i++)
         (void) MSBFirstReadShort(image);
       for (i=0; i < (int) psd_header.channels; i++)
-        (void) PackbitsDecodeImage(image,i);
+        (void) DecodeImage(image,i);
     }
   else
     {
+      int
+        channel_map[24];
+
       /*
         Read uncompressed pixel data as separate planes.
       */
+      packet_size=1;
+      if (image->class == PseudoClass)
+        {
+          if (image->colors > 256)
+            packet_size++;
+        }
+      else
+        if (image->depth > 8)
+          packet_size++;
+      for (i=0; i < psd_header.channels; i++)
+        channel_map[i]=!image->matte ? i : i-1;
+      scanline=(unsigned char *)
+        AllocateMemory(packet_size*image->columns*sizeof(unsigned char));
+      if (scanline == (unsigned char *) NULL)
+        ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
       for (i=0; i < (int) psd_header.channels; i++)
       {
-        q=image->pixels;
-        for (j=0; j < (int) image->packets; j++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          switch (i)
+          q=GetPixelCache(image,0,y,image->columns,1);
+          status=ReadBlob(image,packet_size*image->columns,(char *) scanline);
+          if ((status == False) || (q == (PixelPacket *) NULL))
+            break;
+          switch (channel_map[i])
           {
             case 0:
             {
-              ReadQuantumFile(q->red);
-              q->index=q->red;
+              if (image->class == PseudoClass)
+                (void) ReadPixelCache(image,IndexQuantum,scanline);
+              else
+                (void) ReadPixelCache(image,RedQuantum,scanline);
               break;
             }
             case 1:
             {
-              ReadQuantumFile(q->green);
+              (void) ReadPixelCache(image,GreenQuantum,scanline);
               break;
             }
             case 2:
             {
-              ReadQuantumFile(q->blue);
+              (void) ReadPixelCache(image,BlueQuantum,scanline);
               break;
             }
             case 3:
             default:
             {
-              ReadQuantumFile(q->index);
+              (void) ReadPixelCache(image,OpacityQuantum,scanline);
               break;
             }
           }
-          q->length=0;
-          q++;
+          if (!SyncPixelCache(image))
+            break;
         }
       }
+      if (image->matte)
+        MatteImage(image,Transparent);
+      FreeMemory(scanline);
     }
-  if (image->class == PseudoClass)
-    SyncImage(image);
-  else
-    if (image->colorspace == CMYKColorspace)
+  if (image->colorspace == CMYKColorspace)
+    {
+      /*
+        Correct CMYK levels.
+      */
+      for (y=0; y < (int) image->rows; y++)
       {
-        /*
-          Correct CMYK levels.
-        */
-        q=image->pixels;
-        for (i=0; i < (int) image->packets; i++)
+        q=GetPixelCache(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (int) image->columns; x++)
         {
           q->red=MaxRGB-q->red;
           q->green=MaxRGB-q->green;
           q->blue=MaxRGB-q->blue;
-          q->index=MaxRGB-q->index;
+          q->opacity=MaxRGB-q->opacity;
           q++;
         }
+        if (!SyncPixelCache(image))
+          break;
       }
+    }
   for (i=0; i < number_layers; i++)
   {
     /*
       Composite layer onto image.
     */
     if ((layer_info[i].width != 0) && (layer_info[i].height != 0))
-      CompositeImage(image,OverCompositeOp,layer_info[i].image,
-        layer_info[i].x,layer_info[i].y);
-    layer_info[i].image->colormap=(ColorPacket *) NULL;
+      CompositeImage(image,OverCompositeOp,layer_info[i].image,layer_info[i].x,
+        layer_info[i].y);
     DestroyImage(layer_info[i].image);
   }
   image->matte=False;
   if (image->colorspace != CMYKColorspace)
     image->matte=psd_header.channels >= 4;
-  CondenseImage(image);
+  CloseBlob(image);
   return(image);
 }
 
@@ -716,9 +801,9 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
 %  Method WritePSDImage writes an image in the Adobe Photoshop encoded image
 %  format.
 %
-%  The format of the WritePSDImage routine is:
+%  The format of the WritePSDImage method is:
 %
-%      status=WritePSDImage(image_info,image)
+%      unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -734,14 +819,17 @@ Export Image *ReadPSDImage(const ImageInfo *image_info)
 */
 Export unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
 {
-  register int
-    i,
-    j;
+  int
+    y;
 
-  register RunlengthPacket
-    *p;
+  register int
+    i;
+
+  unsigned char
+    *pixels;
 
   unsigned int
+    packet_size,
     status;
 
   /*
@@ -750,6 +838,13 @@ Export unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryType);
   if (status == False)
     WriterExit(FileOpenWarning,"Unable to open file",image);
+  packet_size=image->depth > 8 ? 6 : 3;
+  if (image->matte)
+    packet_size+=image->depth > 8 ? 2 : 1;
+  pixels=(unsigned char *)
+    AllocateMemory(packet_size*image->columns*sizeof(PixelPacket));
+  if (pixels == (unsigned char *) NULL)
+    WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
   (void) WriteBlob(image,4,"8BPS");
   MSBFirstWriteShort(image,1);  /* version */
   (void) WriteBlob(image,6,"      ");  /* reserved */
@@ -759,7 +854,7 @@ Export unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
     MSBFirstWriteShort(image,image->matte ? 4 : 3);
   MSBFirstWriteLong(image,image->rows);
   MSBFirstWriteLong(image,image->columns);
-  MSBFirstWriteShort(image,image->depth);
+  MSBFirstWriteShort(image,image->class == PseudoClass ? 8 : image->depth);
   if (((image_info->colorspace != UndefinedColorspace) ||
        (image->colorspace != CMYKColorspace)) &&
        (image_info->colorspace != CMYKColorspace))
@@ -800,57 +895,65 @@ Export unsigned int WritePSDImage(const ImageInfo *image_info,Image *image)
   /*
     Write uncompressed pixel data as separate planes.
   */
-  p=image->pixels;
   if (image->class == PseudoClass)
-    for (i=0; i < (int) image->packets; i++)
+    for (y=0; y < (int) image->rows; y++)
     {
-      for (j=0; j <= ((int) p->length); j++)
-        WriteQuantumFile(p->index);
-      p++;
+      if (!GetPixelCache(image,0,y,image->columns,1))
+        break;
+      (void) WritePixelCache(image,IndexQuantum,pixels);
+      (void) WriteBlob(image,image->columns,pixels);
     }
   else
     {
-      for (i=0; i < (int) image->packets; i++)
-      {
-        for (j=0; j <= ((int) p->length); j++)
-          if (image->colorspace != CMYKColorspace)
-            WriteQuantumFile(p->red)
-          else
-            WriteQuantumFile(MaxRGB-p->red);
-        p++;
-      }
-      p=image->pixels;
-      for (i=0; i < (int) image->packets; i++)
-      {
-        for (j=0; j <= ((int) p->length); j++)
-          if (image->colorspace != CMYKColorspace)
-            WriteQuantumFile(p->green)
-          else
-            WriteQuantumFile(MaxRGB-p->green);
-        p++;
-      }
-      p=image->pixels;
-      for (i=0; i < (int) image->packets; i++)
-      {
-        for (j=0; j <= ((int) p->length); j++)
-          if (image->colorspace != CMYKColorspace)
-            WriteQuantumFile(p->blue)
-          else
-            WriteQuantumFile(MaxRGB-p->blue);
-        p++;
-      }
-      p=image->pixels;
-      if (image->matte || (image->colorspace == CMYKColorspace))
-        for (i=0; i < (int) image->packets; i++)
+      packet_size=image->depth > 8 ? 2 : 1;
+      if (image->matte)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
-            if (image->colorspace != CMYKColorspace)
-              WriteQuantumFile(p->index)
-            else
-              WriteQuantumFile(MaxRGB-p->index);
-          p++;
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,OpacityQuantum,pixels);
+          (void) WriteBlob(image,packet_size*image->columns,pixels);
+        }
+      for (y=0; y < (int) image->rows; y++)
+      {
+        if (!GetPixelCache(image,0,y,image->columns,1))
+          break;
+        if (image->colorspace == CMYKColorspace)
+          (void) WritePixelCache(image,CyanQuantum,pixels);
+        else
+          (void) WritePixelCache(image,RedQuantum,pixels);
+        (void) WriteBlob(image,packet_size*image->columns,pixels);
+      }
+      for (y=0; y < (int) image->rows; y++)
+      {
+        if (!GetPixelCache(image,0,y,image->columns,1))
+          break;
+        if (image->colorspace == CMYKColorspace)
+          (void) WritePixelCache(image,YellowQuantum,pixels);
+        else
+          (void) WritePixelCache(image,GreenQuantum,pixels);
+        (void) WriteBlob(image,packet_size*image->columns,pixels);
+      }
+      for (y=0; y < (int) image->rows; y++)
+      {
+        if (!GetPixelCache(image,0,y,image->columns,1))
+          break;
+        if (image->colorspace == CMYKColorspace)
+          (void) WritePixelCache(image,MagentaQuantum,pixels);
+        else
+          (void) WritePixelCache(image,BlueQuantum,pixels);
+        (void) WriteBlob(image,packet_size*image->columns,pixels);
+      }
+      if (image->colorspace == CMYKColorspace)
+        for (y=0; y < (int) image->rows; y++)
+        {
+          if (!GetPixelCache(image,0,y,image->columns,1))
+            break;
+          (void) WritePixelCache(image,BlackQuantum,pixels);
+          (void) WriteBlob(image,packet_size*image->columns,pixels);
         }
     }
+  FreeMemory(pixels);
   CloseBlob(image);
   return(True);
 }
