@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,6 +60,45 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s S G I                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method IsSGI returns True if the image format type, identified by the
+%  magick string, is SGI.
+%
+%  The format of the ReadSGIImage method is:
+%
+%      unsigned int IsSGI(const unsigned char *magick,
+%        const unsigned int length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsSGI returns True if the image format type is SGI.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+Export unsigned int IsSGI(const unsigned char *magick,const unsigned int length)
+{
+  if (length < 2)
+    return(False);
+  if (strncmp((char *) magick,"\001\332",2) == 0)
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e a d S G I I m a g e                                                   %
 %                                                                             %
 %                                                                             %
@@ -69,9 +109,9 @@
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
-%  The format of the ReadSGIImage routine is:
+%  The format of the ReadSGIImage method is:
 %
-%      image=ReadSGIImage(image_info)
+%      Image *ReadSGIImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -151,7 +191,7 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
     y,
     z;
 
-  register RunlengthPacket
+  register PixelPacket
     *q;
 
   register unsigned char
@@ -243,7 +283,7 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
             }
           }
         }
-        FreeMemory((char *) scanline);
+        FreeMemory(scanline);
       }
     else
       {
@@ -338,34 +378,31 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
     image->matte=iris_header.depth == 4;
     image->columns=iris_header.columns;
     image->rows=iris_header.rows;
-    image->packets=image->columns*image->rows;
-    image->pixels=(RunlengthPacket *)
-      AllocateMemory(image->packets*sizeof(RunlengthPacket));
-    if (image->pixels == (RunlengthPacket *) NULL)
-      ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
-    SetImage(image);
     /*
-      Convert SGI raster image to runlength-encoded packets.
+      Convert SGI raster image to pixel packets.
     */
-    q=image->pixels;
     if (image->class == DirectClass)
       {
         /*
-          Convert SGI image to DirectClass runlength-encoded packets.
+          Convert SGI image to DirectClass pixel packets.
         */
         for (y=0; y < (int) image->rows; y++)
         {
           p=iris_pixels+((image->rows-1)-y)*(image->columns*4);
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
             q->red=UpScale(*p);
             q->green=UpScale(*(p+1));
             q->blue=UpScale(*(p+2));
-            q->index=UpScale(*(p+3));
-            q->length=0;
+            q->opacity=UpScale(*(p+3));
             p+=4;
             q++;
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
@@ -379,9 +416,9 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
         /*
           Create grayscale map.
         */
-        image->colormap=(ColorPacket *)
-          AllocateMemory(image->colors*sizeof(ColorPacket));
-        if (image->colormap == (ColorPacket *) NULL)
+        image->colormap=(PixelPacket *)
+          AllocateMemory(image->colors*sizeof(PixelPacket));
+        if (image->colormap == (PixelPacket *) NULL)
           ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
         for (i=0; i < (int) image->colors; i++)
         {
@@ -390,27 +427,29 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
           image->colormap[i].blue=(Quantum) UpScale(i);
         }
         /*
-          Convert SGI image to PseudoClass runlength-encoded packets.
+          Convert SGI image to PseudoClass pixel packets.
         */
         for (y=0; y < (int) image->rows; y++)
         {
+          if (!SetPixelCache(image,0,y,image->columns,1))
+            break;
           p=iris_pixels+((image->rows-1)-y)*(image->columns*4);
           for (x=0; x < (int) image->columns; x++)
           {
             index=(unsigned short) (*p);
-            q->index=index;
-            q->length=0;
+            image->indexes[x]=index;
             p+=4;
             q++;
           }
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
         }
         SyncImage(image);
       }
-    FreeMemory((char *) iris_pixels);
-    CondenseImage(image);
+    FreeMemory(iris_pixels);
     /*
       Proceed to next image.
     */
@@ -430,8 +469,7 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
             return((Image *) NULL);
           }
         image=image->next;
-        ProgressMonitor(LoadImagesText,(unsigned int) TellBlob(image),
-          (unsigned int) image->filesize);
+        ProgressMonitor(LoadImagesText,TellBlob(image),image->filesize);
       }
   } while (iris_header.magic == 0x01DA);
   while (image->previous != (Image *) NULL)
@@ -453,9 +491,9 @@ Export Image *ReadSGIImage(const ImageInfo *image_info)
 %
 %  Method WriteSGIImage writes an image in SGI RGB encoded image format.
 %
-%  The format of the WriteSGIImage routine is:
+%  The format of the WriteSGIImage method is:
 %
-%      status=WriteSGIImage(image_info,image)
+%      unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -546,17 +584,18 @@ Export unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
       filler[492];
   } SGIHeader;
 
+  int
+    y,
+    z;
+
   SGIHeader
     iris_header;
 
   register int
     i,
-    j,
-    x,
-    y,
-    z;
+    x;
 
-  register RunlengthPacket
+  register PixelPacket
     *p;
 
   register unsigned char
@@ -624,32 +663,25 @@ Export unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     if (iris_pixels == (unsigned char *) NULL)
       WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
     /*
-      Convert runlength-encoded packets to uncompressed SGI pixels.
+      Convert pixel packets to uncompressed SGI pixels.
     */
-    x=0;
-    y=0;
-    p=image->pixels;
-    q=iris_pixels+(iris_header.rows-1)*(iris_header.columns*4);
-    for (i=0; i < (int) image->packets; i++)
+    for (y=0; y < (int) image->rows; y++)
     {
-      for (j=0; j <= ((int) p->length); j++)
+      p=GetPixelCache(image,0,y,image->columns,1);
+      if (p == (PixelPacket *) NULL)
+        break;
+      q=iris_pixels+((iris_header.rows-1)-y)*(iris_header.columns*4);
+      for (x=0; x < (int) image->columns; x++)
       {
         *q++=DownScale(p->red);
         *q++=DownScale(p->green);
         *q++=DownScale(p->blue);
-        *q++=(unsigned char) p->index;
-        x++;
-        if (x == (int) image->columns)
-          {
-            if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                ProgressMonitor(SaveImageText,y,image->rows);
-            y++;
-            q=iris_pixels+((iris_header.rows-1)-y)*(iris_header.columns*4);
-            x=0;
-          }
+        *q++=(unsigned char) p->opacity;
+        p++;
       }
-      p++;
+      if (image->previous == (Image *) NULL)
+        if (QuantumTick(y,image->rows))
+          ProgressMonitor(SaveImageText,y,image->rows);
     }
     if (image_info->compression == NoCompression)
       {
@@ -734,8 +766,7 @@ Export unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     FreeMemory(iris_pixels);
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
   if (image_info->adjoin)
