@@ -18,7 +18,8 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,6 +60,45 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s S U N                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method IsSUN returns True if the image format type, identified by the
+%  magick string, is SUN.
+%
+%  The format of the ReadSUNImage method is:
+%
+%      unsigned int IsSUN(const unsigned char *magick,
+%        const unsigned int length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsSUN returns True if the image format type is SUN.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+Export unsigned int IsSUN(const unsigned char *magick,const unsigned int length)
+{
+  if (length < 4)
+    return(False);
+  if (strncmp((char *) magick,"\131\246\152\225",4) == 0)
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   D e c o d e I m a g e                                                     %
 %                                                                             %
 %                                                                             %
@@ -68,10 +108,11 @@
 %  Method DecodeImage unpacks the packed image pixels into
 %  runlength-encoded pixel packets.
 %
-%  The format of the DecodeImage routine is:
+%  The format of the DecodeImage method is:
 %
-%      status=DecodeImage(compressed_pixels,number_columns,number_rows,
-%        pixels)
+%      unsigned int DecodeImage(const unsigned char *compressed_pixels,
+%        const unsigned int number_columns,const unsigned int number_rows,
+%        unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
@@ -152,9 +193,9 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
 %  the memory necessary for the new Image structure and returns a pointer to
 %  the new image.
 %
-%  The format of the ReadSUNImage routine is:
+%  The format of the ReadSUNImage method is:
 %
-%      image=ReadSUNImage(image_info)
+%      Image *ReadSUNImage(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
@@ -191,13 +232,15 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
   Image
     *image;
 
-  register int
+  int
     bit,
-    i,
-    x,
     y;
 
-  register RunlengthPacket
+  register int
+    i,
+    x;
+
+  register PixelPacket
     *q;
 
   register unsigned char
@@ -269,11 +312,10 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
             /*
               Create linear color ramp.
             */
-            image->colormap=(ColorPacket *)
-              AllocateMemory(image->colors*sizeof(ColorPacket));
-            if (image->colormap == (ColorPacket *) NULL)
-              ReaderExit(ResourceLimitWarning,"Memory allocation failed",
-                image);
+            image->colormap=(PixelPacket *)
+              AllocateMemory(image->colors*sizeof(PixelPacket));
+            if (image->colormap == (PixelPacket *) NULL)
+              ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
             for (i=0; i < (int) image->colors; i++)
             {
               image->colormap[i].red=(MaxRGB*i)/(image->colors-1);
@@ -291,11 +333,11 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
         /*
           Read SUN raster colormap.
         */
-        image->colormap=(ColorPacket *)
-          AllocateMemory(image->colors*sizeof(ColorPacket));
+        image->colormap=(PixelPacket *)
+          AllocateMemory(image->colors*sizeof(PixelPacket));
         sun_colormap=(unsigned char *)
           AllocateMemory(image->colors*sizeof(unsigned char));
-        if ((image->colormap == (ColorPacket *) NULL) ||
+        if ((image->colormap == (PixelPacket *) NULL) ||
             (sun_colormap == (unsigned char *) NULL))
           ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
         (void) ReadBlob(image,image->colors,(char *) sun_colormap);
@@ -307,7 +349,7 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
         (void) ReadBlob(image,image->colors,(char *) sun_colormap);
         for (i=0; i < (int) image->colors; i++)
           image->colormap[i].blue=UpScale(sun_colormap[i]);
-        FreeMemory((char *) sun_colormap);
+        FreeMemory(sun_colormap);
         break;
       }
       case RMT_RAW:
@@ -324,12 +366,11 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
           ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
         (void) ReadBlob(image,(unsigned int) sun_header.maplength,
           (char *) sun_colormap);
-        FreeMemory((char *) sun_colormap);
+        FreeMemory(sun_colormap);
         break;
       }
       default:
-        ReaderExit(CorruptImageWarning,"Colormap type is not supported",
-          image);
+        ReaderExit(CorruptImageWarning,"Colormap type is not supported",image);
     }
     sun_data=(unsigned char *)
       AllocateMemory(sun_header.length*sizeof(unsigned char));
@@ -354,7 +395,7 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
         if (sun_pixels == (unsigned char *) NULL)
           ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
         (void) DecodeImage(sun_data,bytes_per_line,height,sun_pixels);
-        FreeMemory((char *) sun_data);
+        FreeMemory(sun_data);
       }
     /*
       Initialize image structure.
@@ -362,44 +403,31 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
     image->matte=(sun_header.depth == 32);
     image->columns=(unsigned int) sun_header.width;
     image->rows=(unsigned int) sun_header.height;
-    image->packets=image->columns*image->rows;
-    image->pixels=(RunlengthPacket *)
-      AllocateMemory(image->packets*sizeof(RunlengthPacket));
-    if (image->pixels == (RunlengthPacket *) NULL)
-      ReaderExit(ResourceLimitWarning,"Memory allocation failed",image);
     /*
-      Convert SUN raster image to runlength-encoded packets.
+      Convert SUN raster image to pixel packets.
     */
     p=sun_pixels;
-    q=image->pixels;
     if (sun_header.depth == 1)
       for (y=0; y < (int) image->rows; y++)
       {
-        /*
-          Convert bitmap scanline to runlength-encoded color packets.
-        */
-        for (x=0; x < (int) (image->columns >> 3); x++)
+        if (!SetPixelCache(image,0,y,image->columns,1))
+          break;
+        for (x=0; x < ((int) image->columns-7); x+=8)
         {
           for (bit=7; bit >= 0; bit--)
-          {
-            q->index=((*p) & (0x01 << bit) ? 0x00 : 0x01);
-            q->length=0;
-            q++;
-          }
+            image->indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x00 : 0x01);
           p++;
         }
         if ((image->columns % 8) != 0)
           {
             for (bit=7; bit >= (int) (8-(image->columns % 8)); bit--)
-            {
-              q->index=((*p) & (0x01 << bit) ? 0x00 : 0x01);
-              q->length=0;
-              q++;
-            }
+              image->indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x00 : 0x01);
             p++;
           }
         if ((((image->columns/8)+(image->columns % 8 ? 1 : 0)) % 2) != 0)
           p++;
+        if (!SyncPixelCache(image))
+          break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
             ProgressMonitor(LoadImageText,y,image->rows);
@@ -408,17 +436,14 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
       if (image->class == PseudoClass)
         for (y=0; y < (int) image->rows; y++)
         {
-          /*
-            Convert PseudoColor scanline to runlength-encoded color packets.
-          */
+          if (!SetPixelCache(image,0,y,image->columns,1))
+            break;
           for (x=0; x < (int) image->columns; x++)
-          {
-            q->index=(*p++);
-            q->length=0;
-            q++;
-          }
+            image->indexes[x]=(*p++);
           if ((image->columns % 2) != 0)
             p++;
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
@@ -426,14 +451,13 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
       else
         for (y=0; y < (int) image->rows; y++)
         {
-          /*
-            Convert DirectColor scanline to runlength-encoded color packets.
-          */
+          q=SetPixelCache(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
           for (x=0; x < (int) image->columns; x++)
           {
-            q->index=0;
             if (image->matte)
-              q->index=UpScale(*p++);
+              q->opacity=UpScale(*p++);
             if (sun_header.type == RT_STANDARD)
               {
                 q->blue=UpScale(*p++);
@@ -452,19 +476,19 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
                 q->green=image->colormap[q->green].green;
                 q->blue=image->colormap[q->blue].blue;
               }
-            q->length=0;
             q++;
           }
           if (((image->columns % 2) != 0) && (image->matte == False))
             p++;
+          if (!SyncPixelCache(image))
+            break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               ProgressMonitor(LoadImageText,y,image->rows);
         }
-    FreeMemory((char *) sun_pixels);
     if (image->class == PseudoClass)
       SyncImage(image);
-    CondenseImage(image);
+    FreeMemory(sun_pixels);
     /*
       Proceed to next image.
     */
@@ -484,8 +508,7 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
             return((Image *) NULL);
           }
         image=image->next;
-        ProgressMonitor(LoadImagesText,(unsigned int) TellBlob(image),
-          (unsigned int) image->filesize);
+        ProgressMonitor(LoadImagesText,TellBlob(image),image->filesize);
       }
   } while (sun_header.magic == 0x59a66a95);
   while (image->previous != (Image *) NULL)
@@ -507,9 +530,9 @@ Export Image *ReadSUNImage(const ImageInfo *image_info)
 %
 %  Method WriteSUNImage writes an image in the SUN rasterfile format.
 %
-%  The format of the WriteSUNImage routine is:
+%  The format of the WriteSUNImage method is:
 %
-%      status=WriteSUNImage(image_info,image)
+%      unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -545,14 +568,13 @@ Export unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
   } SUNHeader;
 
   int
-    x,
     y;
 
   register int
     i,
-    j;
+    x;
 
-  register RunlengthPacket
+  register PixelPacket
     *p;
 
   SUNHeader
@@ -626,7 +648,6 @@ Export unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to SUN raster pixels.
     */
-    p=image->pixels;
     x=0;
     y=0;
     if (!IsPseudoClass(image) && !IsGrayImage(image))
@@ -637,46 +658,39 @@ Export unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
         unsigned char
           *pixels;
 
-        unsigned short
-          value;
-
         /*
           Allocate memory for pixels.
         */
         pixels=(unsigned char *)
-          AllocateMemory(image->columns*sizeof(RunlengthPacket));
+          AllocateMemory(image->columns*sizeof(PixelPacket));
         if (pixels == (unsigned char *) NULL)
           WriterExit(ResourceLimitWarning,"Memory allocation failed",image);
         /*
           Convert DirectClass packet to SUN RGB pixel.
         */
-        q=pixels;
-        for (i=0; i < (int) image->packets; i++)
+        for (y=0; y < (int) image->rows; y++)
         {
-          for (j=0; j <= ((int) p->length); j++)
+          p=GetPixelCache(image,0,y,image->columns,1);
+          if (p == (PixelPacket *) NULL)
+            break;
+          q=pixels;
+          for (x=0; x < (int) image->columns; x++)
           {
             if (image->matte)
-              *q++=DownScale(p->index);
+              *q++=DownScale(p->opacity);
             *q++=DownScale(p->red);
             *q++=DownScale(p->green);
             *q++=DownScale(p->blue);
-            x++;
-            if (x == (int) image->columns)
-              {
-                if (image->columns & 0x01)
-                  WriteQuantum(0,q);  /* pad scanline */
-                (void) WriteBlob(image,q-pixels,(char *) pixels);
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows))
-                    ProgressMonitor(SaveImageText,y,image->rows);
-                q=pixels;
-                x=0;
-                y++;
-              }
+            p++;
           }
-          p++;
+          if (image->columns & 0x01)
+            *q++=0;  /* pad scanline */
+          (void) WriteBlob(image,q-pixels,(char *) pixels);
+          if (image->previous == (Image *) NULL)
+            if (QuantumTick(y,image->rows))
+              ProgressMonitor(SaveImageText,y,image->rows);
         }
-        FreeMemory((char *) pixels);
+        FreeMemory(pixels);
       }
     else
       if (IsMonochromeImage(image))
@@ -693,14 +707,16 @@ Export unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           if (image->colors == 2)
             polarity=
               Intensity(image->colormap[0]) > Intensity(image->colormap[1]);
-          bit=0;
-          byte=0;
-          for (i=0; i < (int) image->packets; i++)
+          for (y=0; y < (int) image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            if (!GetPixelCache(image,0,y,image->columns,1))
+              break;
+            bit=0;
+            byte=0;
+            for (x=0; x < (int) image->columns; x++)
             {
               byte<<=1;
-              if (p->index == polarity)
+              if (image->indexes[x] == polarity)
                 byte|=0x01;
               bit++;
               if (bit == 8)
@@ -709,26 +725,16 @@ Export unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
                   bit=0;
                   byte=0;
                 }
-              x++;
-              if (x == (int) image->columns)
-                {
-                  /*
-                    Advance to the next scanline.
-                  */
-                  if (bit != 0)
-                    (void) WriteByte(image,byte << (8-bit));
-                  if ((((image->columns/8)+
-                      (image->columns % 8 ? 1 : 0)) % 2) != 0)
-                    (void) WriteByte(image,0);  /* pad scanline */
-                  if (image->previous == (Image *) NULL)
-                    if (QuantumTick(y,image->rows))
-                      ProgressMonitor(SaveImageText,y,image->rows);
-                  bit=0;
-                  byte=0;
-                  x=0;
-               }
+              p++;
             }
-            p++;
+            if (bit != 0)
+              (void) WriteByte(image,byte << (8-bit));
+            if ((((image->columns/8)+
+                (image->columns % 8 ? 1 : 0)) % 2) != 0)
+              (void) WriteByte(image,0);  /* pad scanline */
+            if (image->previous == (Image *) NULL)
+              if (QuantumTick(y,image->rows))
+                ProgressMonitor(SaveImageText,y,image->rows);
           }
         }
       else
@@ -745,29 +751,25 @@ Export unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           /*
             Convert PseudoClass packet to SUN colormapped pixel.
           */
-          for (i=0; i < (int) image->packets; i++)
+          for (y=0; y < (int) image->rows; y++)
           {
-            for (j=0; j <= ((int) p->length); j++)
+            if (!GetPixelCache(image,0,y,image->columns,1))
+              break;
+            for (x=0; x < (int) image->columns; x++)
             {
-              (void) WriteByte(image,p->index);
-              x++;
-              if (x == (int) image->columns)
-                {
-                  if (image->columns & 0x01)
-                    (void) WriteByte(image,0);  /* pad scanline */
-                  if (image->previous == (Image *) NULL)
-                    if (QuantumTick(y,image->rows))
-                      ProgressMonitor(SaveImageText,y,image->rows);
-                  x=0;
-                }
+              (void) WriteByte(image,image->indexes[x]);
+              p++;
             }
-            p++;
+            if (image->columns & 0x01)
+              (void) WriteByte(image,0);  /* pad scanline */
+            if (image->previous == (Image *) NULL)
+              if (QuantumTick(y,image->rows))
+                ProgressMonitor(SaveImageText,y,image->rows);
           }
         }
     if (image->next == (Image *) NULL)
       break;
-    image->next->file=image->file;
-    image=image->next;
+    image=GetNextImage(image);
     ProgressMonitor(SaveImagesText,scene++,GetNumberScenes(image));
   } while (image_info->adjoin);
   if (image_info->adjoin)
