@@ -18,7 +18,8 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -33,15 +34,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -190,12 +191,19 @@ Export char *BaseFilename(const char *name)
     *p;
 
   static char
-    basename[MaxTextExtent];
+    *basename = (char *) NULL;
 
   /*
     Get basename of client.
   */
   assert(name != (char *) NULL);
+  if (basename == (char *) NULL)
+    {
+      basename=(char *) malloc(MaxTextExtent*sizeof(char *));
+      if (basename == (char *) NULL)
+        MagickError(ResourceLimitError,"Unable to get base filename",
+          "Memory allocation failed");
+    }
   (void) strcpy(basename,name);
   p=basename+(Extent(basename)-1);
   while (p > basename)
@@ -362,11 +370,11 @@ Export unsigned short *ConvertTextToUnicode(const char *text,int *count)
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short*) NULL);
+    return((unsigned short *) NULL);
   unicode=(unsigned short *)
-    AllocateMemory(strlen(text)*sizeof(unsigned short *));
+    AllocateMemory((strlen(text)+1)*sizeof(unsigned short));
   if (unicode == (unsigned short *) NULL)
-    return((unsigned short*) NULL);
+    return((unsigned short *) NULL);
   p=text;
   q=unicode;
   while (*p != '\0')
@@ -378,7 +386,7 @@ Export unsigned short *ConvertTextToUnicode(const char *text,int *count)
         value=InterpretUnicode(p,4);
         if (value < 0)
           {
-            FreeMemory((char *) unicode);
+            FreeMemory(unicode);
             return((unsigned short *) NULL);
           }
         *q=(unsigned short) value;
@@ -557,13 +565,7 @@ Export unsigned int ExpandFilenames(int *argc,char ***argv)
     ExpandFilename(filename);
     if (!IsGlob(filename))
       {
-        /*
-          Silently skip directories.
-        */
-        if (IsDirectory(filename))
-          count--;
-        else
-          expanded=True;
+        expanded=True;
         continue;
       }
     /*
@@ -592,8 +594,8 @@ Export unsigned int ExpandFilenames(int *argc,char ***argv)
     if (j == number_files)
       {
         for (j=0; j < number_files; j++)
-          FreeMemory((char *) filelist[j]);
-        FreeMemory((char *) filelist);
+          FreeMemory(filelist[j]);
+        FreeMemory(filelist);
         continue;
       }
     /*
@@ -612,7 +614,7 @@ Export unsigned int ExpandFilenames(int *argc,char ***argv)
     {
       if (IsDirectory(filelist[j]))
         {
-          FreeMemory((char *) filelist[j]);
+          FreeMemory(filelist[j]);
           continue;
         }
       expanded=True;
@@ -623,21 +625,21 @@ Export unsigned int ExpandFilenames(int *argc,char ***argv)
           MagickWarning(ResourceLimitWarning,"Unable to expand filenames",
             (char *) NULL);
           for ( ; j < number_files; j++)
-            FreeMemory((char *) filelist[j]);
-          FreeMemory((char *) filelist);
+            FreeMemory(filelist[j]);
+          FreeMemory(filelist);
           return(False);
         }
       FormatString(vector[count],"%.*s%.1024s",(int) (p-filename),filename,
         filelist[j]);
-      FreeMemory((char *) filelist[j]);
+      FreeMemory(filelist[j]);
       count++;
     }
-    FreeMemory((char *) filelist);
+    FreeMemory(filelist);
   }
   (void) chdir(home_directory);
   if (!expanded)
     {
-      FreeMemory((char *) vector);
+      FreeMemory(vector);
       return(False);
     }
   *argc=count;
@@ -821,7 +823,7 @@ Export int GetGeometry(const char *image_geometry,int *x,int *y,
 Export int GlobExpression(const char *expression,const char *pattern)
 {
   ImageInfo
-    image_info;
+    *image_info;
 
   unsigned int
     done,
@@ -839,12 +841,13 @@ Export int GlobExpression(const char *expression,const char *pattern)
   /*
     Determine if pattern is a subimage, i.e. img0001.pcd[2].
   */
-  GetImageInfo(&image_info);
-  (void) strcpy(image_info.filename,pattern);
-  SetImageInfo(&image_info,True);
-  exempt=(Latin1Compare(image_info.magick,"VID") == 0) ||
-    (image_info.subimage &&
-    (Latin1Compare(expression,image_info.filename) == 0));
+  image_info=CloneImageInfo((ImageInfo *) NULL);
+  (void) strcpy(image_info->filename,pattern);
+  SetImageInfo(image_info,True);
+  exempt=(Latin1Compare(image_info->magick,"VID") == 0) ||
+    (image_info->subimage &&
+    (Latin1Compare(expression,image_info->filename) == 0));
+  DestroyImageInfo(image_info);
   if (exempt)
     return(False);
   /*
@@ -1098,8 +1101,7 @@ Export unsigned int IsAccessible(const char *filename)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Method IsDirectory returns True if the file as defined by filename is
-%  a directory.  Once MetroWerks write a stat(2) function, we can remove the
-%  chdir(2) function.
+%  a directory.
 %
 %  The format of the IsAccessible method is:
 %
@@ -1120,24 +1122,16 @@ Export unsigned int IsDirectory(const char *filename)
   int
     status;
 
-#if !defined(WIN32)
   struct stat
     file_info;
 
+  assert(filename != (const char *) NULL);
+  if (*filename == '\0')
+    return(False);
   status=stat(filename,&file_info);
   if (status != 0)
     return(False);
   return(S_ISDIR(file_info.st_mode));
-#else
-  char
-    current_directory[MaxTextExtent];
-
-  (void) getcwd(current_directory,MaxTextExtent-1);
-  status=chdir(filename);
-  if (status == 0)
-    (void) chdir(current_directory);
-  return(status == 0);
-#endif
 }
 
 /*
@@ -1621,20 +1615,20 @@ Export int MultilineCensus(const char *label)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ParseImageGeometry parses a geometry specification and returns the
+%  Method ParseGeometry parses a geometry specification and returns the
 %  width, height, x, and y values.  It also returns flags that indicates
 %  which of the four values (width, height, xoffset, yoffset) were located
 %  in the string, and whether the x and y values are negative.  In addition,
 %  there are flags to report any meta characters (%, !, <, and >).
 %
-%  The format of the ParseImageGeometry method is:
+%  The format of the ParseGeometry method is:
 %
 %      int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
 %        unsigned int *height)
 %
 %  A description of each parameter follows:
 %
-%    o flags:  Method ParseImageGeometry returns a bitmask that indicates
+%    o flags:  Method ParseGeometry returns a bitmask that indicates
 %      which of the four values were located in the geometry string.
 %
 %    o image_geometry:  Specifies a character string representing the geometry
@@ -1649,12 +1643,12 @@ Export int MultilineCensus(const char *label)
 %
 */
 
-static int ReadInteger(const char *p,char **q)
+static double ReadReal(const char *p,char **q)
 {
   int
     sign;
 
-  register int
+  register double
     value;
 
   value=0;
@@ -1669,6 +1663,22 @@ static int ReadInteger(const char *p,char **q)
       }
   for ( ; (*p >= '0') && (*p <= '9'); p++)
     value=(value*10)+(*p-'0');
+  if (*p == '.')
+    {
+      double
+        fraction;
+
+      register int
+        i;
+
+      p++;
+      fraction=0;
+      for (i=10; (*p >= '0') && (*p <= '9'); p++)
+      {
+        fraction=fraction+(*p-'0')/i;
+        i*=10;
+      }
+    }
   *q=(char *) p;
   if (sign >= 0)
     return(value);
@@ -1697,7 +1707,7 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
       /*
         Parse width.
       */
-      bounds.width=ReadInteger(geometry,&q);
+      bounds.width=ReadReal(geometry,&q);
       if (geometry == q)
         return(0);
       geometry=q;
@@ -1709,7 +1719,7 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
         Parse height.
       */
       geometry++;
-      bounds.height=ReadInteger(geometry,&q);
+      bounds.height=ReadReal(geometry,&q);
       if (geometry == q)
         return(0);
       geometry=q;
@@ -1723,16 +1733,16 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
       if (*geometry == '-')
         {
           geometry++;
-          bounds.x=(-ReadInteger(geometry,&q));
+          bounds.x=(-ReadReal(geometry,&q));
           if (geometry == q)
-            return (0);
+            return(0);
           geometry=q;
           mask|=XNegative;
         }
       else
         {
           geometry++;
-          bounds.x=ReadInteger(geometry,&q);
+          bounds.x=ReadReal(geometry,&q);
           if (geometry == q)
             return(0);
           geometry=q;
@@ -1746,7 +1756,7 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
           if (*geometry == '-')
             {
               geometry++;
-              bounds.y=(-ReadInteger(geometry,&q));
+              bounds.y=(-ReadReal(geometry,&q));
               if (geometry == q)
                 return(0);
               geometry=q;
@@ -1755,7 +1765,7 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
           else
             {
               geometry++;
-              bounds.y=ReadInteger(geometry,&q);
+              bounds.y=ReadReal(geometry,&q);
               if (geometry == q)
                 return(0);
               geometry=q;
@@ -1773,7 +1783,7 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
     *width=bounds.width;
   if (mask & HeightValue)
     *height=bounds.height;
-  return (mask);
+  return(mask);
 }
 
 /*
@@ -1792,7 +1802,7 @@ Export int ParseGeometry(const char *geometry,int *x,int *y,unsigned int *width,
 %
 %  The format of the PostscriptGeometry method is:
 %
-%      void DestroyPostscriptGeometry(char *geometry)
+%      void PostscriptGeometry(const char *page)
 %
 %  A description of each parameter follows.
 %
@@ -2261,9 +2271,6 @@ Export void Strip(char *data)
     *p,
     *q;
 
-  register int
-    i;
-
   assert(data != (char *) NULL);
   if (*data == '\0')
     return;
@@ -2275,9 +2282,8 @@ Export void Strip(char *data)
     q--;
   count=q-p+1;
   q=data;
-  for (i=0; i < count; i++)
-    *q++=(*p++);
-  *q='\0';
+  (void) memcpy(q,p,count*sizeof(char));
+  *(q+count)='\0';
 }
 
 /*
@@ -2317,7 +2323,7 @@ Export int SystemCommand(const unsigned int verbose,const char *command)
 
 #if !defined(vms) && !defined(macintosh) && !defined(WIN32)
   status=system(command);
-#else
+#endif
 #if defined(vms)
   status=!system(command);
 #endif
@@ -2326,11 +2332,10 @@ Export int SystemCommand(const unsigned int verbose,const char *command)
 #endif
 #if defined(WIN32)
   status=NTSystemCommand(command);
-#endif
 #endif
   if (verbose)
-    MagickWarning(UndefinedWarning,command,
-      status ? strerror(errno) : (char *) NULL);
+    MagickWarning(UndefinedWarning,command,status ? strerror(errno) :
+      (char *) NULL);
   return(status);
 }
 
@@ -2358,18 +2363,13 @@ Export int SystemCommand(const unsigned int verbose,const char *command)
 %   o  filename:  Specifies a pointer to an array of characters.  The unique
 %      file name is returned in this array.
 %
-%
 */
 Export void TemporaryFilename(char *filename)
 {
-  register int
-    i;
-
   assert(filename != (char *) NULL);
   *filename='\0';
-  for (i=0; i < 256; i++)
-  {
 #if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+  {
     register char
       *p;
 
@@ -2379,20 +2379,17 @@ Export void TemporaryFilename(char *filename)
         (void) strcpy(filename,p);
         free((char *) p);
       }
+  }
 #else
 #if defined(WIN32)
-    (void) NTTemporaryFilename(filename);
+  (void) NTTemporaryFilename(filename);
 #else
 #if defined(macintosh)
-    (void) getcwd(filename,MaxTextExtent >> 1);
+  (void) getcwd(filename,MaxTextExtent >> 1);
 #endif
-    (void) tmpnam(filename+strlen(filename));
+  (void) tmpnam(filename+strlen(filename));
 #endif
 #endif
-    if ((strchr(filename,'%') == (char *) NULL) &&
-        (strchr(filename,'.') == (char *) NULL))
-      break;
-  }
 }
 
 /*
@@ -2505,7 +2502,7 @@ Export char *TranslateText(const ImageInfo *image_info,const Image *image,
       MagickWarning(ResourceLimitWarning,"Unable to translate text",
         "Memory allocation failed");
       if (indirection)
-        FreeMemory((char *) text);
+        FreeMemory(text);
       return((char *) NULL);
     }
   local_info=CloneImageInfo(image_info);
@@ -2514,8 +2511,8 @@ Export char *TranslateText(const ImageInfo *image_info,const Image *image,
       MagickWarning(ResourceLimitWarning,"Unable to translate text",
         "Memory allocation failed");
       if (indirection)
-        FreeMemory((char *) text);
-      FreeMemory((char *) translated_text);
+        FreeMemory(text);
+      FreeMemory(translated_text);
       return((char *) NULL);
     }
   /*
@@ -2659,7 +2656,7 @@ Export char *TranslateText(const ImageInfo *image_info,const Image *image,
       }
       case 'n':
       {
-        FormatString(q,"%u",local_info->subrange);
+        FormatString(q,"%u",GetNumberScenes(image));
         q=translated_text+Extent(translated_text);
         break;
       }
@@ -2745,6 +2742,6 @@ Export char *TranslateText(const ImageInfo *image_info,const Image *image,
   *q='\0';
   DestroyImageInfo(local_info);
   if (indirection)
-    FreeMemory((char *) text);
+    FreeMemory(text);
   return(translated_text);
 }
