@@ -59,6 +59,7 @@ extern "C" {
 */
 #define MaxNumberPens  11
 #define MaxNumberFonts  11
+#define MaxIconSize  96
 
 /*
   Enumeration declarations.
@@ -277,10 +278,8 @@ typedef struct _XResourceInfo
     magnify;
 
   char
+    *map_type,
     *matte_color,
-    *map_type;
-
-  char
     *name;
 
   unsigned int
@@ -502,7 +501,7 @@ extern Export unsigned int
   XMakeImage(Display *,const XResourceInfo *,XWindowInfo *,Image *,
     unsigned int,unsigned int),
   XMakePixmap(Display *,const XResourceInfo *,XWindowInfo *),
-  XQueryColorDatabase(const char *,ColorPacket *);
+  XQueryColorDatabase(const char *,XColor *);
 
 extern Export void
   XAnimateBackgroundImage(Display *,XResourceInfo *,Image *),
@@ -522,7 +521,7 @@ extern Export void
   XGetAnnotateInfo(XAnnotateInfo *),
   XGetImportInfo(XImportInfo *),
   XGetMapInfo(const XVisualInfo *,const Colormap,XStandardColormap *),
-  XGetPixelInfo(Display *,const XVisualInfo *,const XStandardColormap *,
+  XGetPixelPacket(Display *,const XVisualInfo *,const XStandardColormap *,
     const XResourceInfo *,Image *,XPixelInfo *),
   XGetResourceInfo(XrmDatabase,char *,XResourceInfo *),
   XGetWindowInfo(Display *,XVisualInfo *,XStandardColormap *,XPixelInfo *,
@@ -535,7 +534,7 @@ extern Export void
     XStandardColormap *,XPixelInfo *),
   XMakeWindow(Display *,Window,char **,int,XClassHint *,XWMHints *,
     XWindowInfo *),
-  XProgressMonitor(const char *,const unsigned int,const unsigned int),
+  XProgressMonitor(const char *,const off_t,const off_t),
   XQueryPosition(Display *,const Window,int *,int *),
   XRefreshWindow(Display *,const XWindowInfo *,const XEvent *),
   XRemoteCommand(Display *,const char *,const char *),
@@ -543,7 +542,7 @@ extern Export void
   XUserPreferences(XResourceInfo *),
   XSetCursorState(Display *,XWindows *,const unsigned int),
   XSignalHandler(int),
-  XWarning(const unsigned int,const char *,const char *);
+  XWarning(const WarningType,const char *,const char *);
 
 extern Export Window
   XClientWindow(Display *,Window),
