@@ -19,7 +19,8 @@
 %                            December 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright (C) 2000 ImageMagick Studio, a non-profit organization dedicated %
+%  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -34,15 +35,15 @@
 %  The software is provided "as is", without warranty of any kind, express or %
 %  implied, including but not limited to the warranties of merchantability,   %
 %  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
+%  ImageMagick Studio be liable for any claim, damages or other liability,    %
+%  whether in an action of contract, tort or otherwise, arising from, out of  %
+%  or in connection with ImageMagick or the use or other dealings in          %
+%  ImageMagick.                                                               %
 %                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Except as contained in this notice, the name of the ImageMagick Studio     %
+%  shall not be used in advertising or otherwise to promote the sale, use or  %
+%  other dealings in ImageMagick without prior written authorization from the %
+%  ImageMagick Studio.                                                        %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -59,6 +60,7 @@
 %    -border geometry     surround image with a border of color
 %    -bordercolor color   border color
 %    -box color           color for annotation bounding box
+%    -cache threshold     megabytes of memory available to the pixel cache
 %    -charcoal factor     simulate a charcoal drawing
 %    -colorize value      colorize the image with the pen color
 %    -colors value        preferred number of colors in the image
@@ -97,6 +99,7 @@
 %    -loop iterations     add Netscape loop extension to your GIF animation
 %    -map filename        transform image colors to match this set of colors
 %    -matte               store matte channel if the image has one
+%    -median radius       apply a median filter to the image
 %    -modulate value      vary the brightness, saturation, and hue
 %    -monochrome          transform image to black and white
 %    -negate              replace every pixel with its complementary color 
@@ -113,6 +116,7 @@
 %    -roll geometry       roll an image vertically or horizontally
 %    -rotate degrees      apply Paeth rotation to the image
 %    -sample geometry     scale image with pixel sampling
+%    -scale geometry      resize image
 %    -scene value         image scene number
 %    -seed value          pseudo-random number generator seed value
 %    -segment values      segment an image
@@ -144,13 +148,8 @@
 /*
   Include declarations.
 */
-#if !defined(macintosh)
 #include "magick/magick.h"
 #include "magick/defines.h"
-#else
-#include "magick.h"
-#include "defines.h"
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -187,6 +186,7 @@ static void Usage(const char *client_name)
       "-border geometry     surround image with a border of color",
       "-bordercolor color   border color",
       "-box color           color for annotation bounding box",
+      "-cache threshold     megabytes of memory available to the pixel cache",
       "-charcoal factor     simulate a charcoal drawing",
       "-colorize value      colorize the image with the pen color",
       "-colors value        preferred number of colors in the image",
@@ -225,6 +225,7 @@ static void Usage(const char *client_name)
       "-loop iterations     add Netscape loop extension to your GIF animation",
       "-map filename        transform image colors to match this set of colors",
       "-matte               store matte channel if the image has one",
+      "-median radius       apply a median filter to the image",
       "-modulate value      vary the brightness, saturation, and hue",
       "-monochrome          transform image to black and white",
       "-negate              replace every pixel with its complementary color ",
@@ -241,6 +242,7 @@ static void Usage(const char *client_name)
       "-roll geometry       roll an image vertically or horizontally",
       "-rotate degrees      apply Paeth rotation to the image",
       "-sample geometry     scale image with pixel sampling",
+      "-scale geometry      resize image",
       "-scene number        image scene number",
       "-seed value          pseudo-random number generator seed value",
       "-segment values      segment an image",
@@ -309,7 +311,7 @@ int main(int argc,char **argv)
     sans;
 
   ImageInfo
-    image_info;
+    *image_info;
 
   Image
     *image;
@@ -343,7 +345,7 @@ int main(int argc,char **argv)
   format=(char *) NULL;
   global_colormap=False;
   image=(Image *) NULL;
-  GetImageInfo(&image_info);
+  image_info=CloneImageInfo((ImageInfo *) NULL);
   scene=0;
   status=False;
   /*
@@ -359,7 +361,7 @@ int main(int argc,char **argv)
         {
           if (strncmp("antialias",option+1,3) == 0)
             {
-              image_info.antialias=(*option == '-');
+              image_info->antialias=(*option == '-');
               break;
             }
           MagickError(OptionError,"Unrecognized option",option);
@@ -369,13 +371,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("background",option+1,5) == 0)
             {
-              image_info.background_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing background color",option);
-                  (void) CloneString(&image_info.background_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],
+                    &image_info->background_color);
                 }
               break;
             }
@@ -401,25 +403,24 @@ int main(int argc,char **argv)
             }
           if (strncmp("bordercolor",option+1,7) == 0)
             {
-              image_info.border_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing border color",option);
-                  (void) CloneString(&image_info.border_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],&image_info->border_color);
                 }
               break;
             }
           if (strncmp("box",option+1,3) == 0)
             {
-              image_info.box=(char *) NULL;
+              image_info->box=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing box color",option);
-                  (void) CloneString(&image_info.box,argv[i]);
+                  (void) CloneString(&image_info->box,argv[i]);
                 }
               break;
             }
@@ -428,6 +429,17 @@ int main(int argc,char **argv)
         }
         case 'c':
         {
+          if (strncmp("cache",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+                    MagickError(OptionError,"Missing threshold",option);
+                }
+              SetCacheThreshold(atoi(argv[i]));
+              break;
+            }
           if (strncmp("charcoal",option+1,3) == 0)
             {
               if (*option == '-')
@@ -466,32 +478,32 @@ int main(int argc,char **argv)
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.colorspace=UndefinedColorspace;
+                  image_info->colorspace=UndefinedColorspace;
                   if (Latin1Compare("cmyk",option) == 0)
-                    image_info.colorspace=CMYKColorspace;
+                    image_info->colorspace=CMYKColorspace;
                   if (Latin1Compare("gray",option) == 0)
-                    image_info.colorspace=GRAYColorspace;
+                    image_info->colorspace=GRAYColorspace;
                   if (Latin1Compare("ohta",option) == 0)
-                    image_info.colorspace=OHTAColorspace;
+                    image_info->colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
-                    image_info.colorspace=RGBColorspace;
+                    image_info->colorspace=RGBColorspace;
                   if (Latin1Compare("srgb",option) == 0)
-                    image_info.colorspace=sRGBColorspace;
+                    image_info->colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
-                    image_info.colorspace=TransparentColorspace;
+                    image_info->colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
-                    image_info.colorspace=XYZColorspace;
+                    image_info->colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
-                    image_info.colorspace=YCbCrColorspace;
+                    image_info->colorspace=YCbCrColorspace;
                   if (Latin1Compare("ycc",option) == 0)
-                    image_info.colorspace=YCCColorspace;
+                    image_info->colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
-                    image_info.colorspace=YIQColorspace;
+                    image_info->colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
-                    image_info.colorspace=YPbPrColorspace;
+                    image_info->colorspace=YPbPrColorspace;
                   if (Latin1Compare("yuv",option) == 0)
-                    image_info.colorspace=YUVColorspace;
-                  if (image_info.colorspace == UndefinedColorspace)
+                    image_info->colorspace=YUVColorspace;
+                  if (image_info->colorspace == UndefinedColorspace)
                     MagickError(OptionError,"Invalid colorspace type",option);
                 }
               break;
@@ -508,31 +520,31 @@ int main(int argc,char **argv)
             }
           if (strncmp("compress",option+1,5) == 0)
             {
-              image_info.compression=NoCompression;
+              image_info->compression=NoCompression;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.compression=UndefinedCompression;
+                  image_info->compression=UndefinedCompression;
                   if (Latin1Compare("None",option) == 0)
-                    image_info.compression=NoCompression;
+                    image_info->compression=NoCompression;
                   if (Latin1Compare("BZip",option) == 0)
-                    image_info.compression=BZipCompression;
+                    image_info->compression=BZipCompression;
                   if (Latin1Compare("Fax",option) == 0)
-                    image_info.compression=FaxCompression;
+                    image_info->compression=FaxCompression;
                   if (Latin1Compare("Group4",option) == 0)
-                    image_info.compression=Group4Compression;
+                    image_info->compression=Group4Compression;
                   if (Latin1Compare("JPEG",option) == 0)
-                    image_info.compression=JPEGCompression;
+                    image_info->compression=JPEGCompression;
                   if (Latin1Compare("LZW",option) == 0)
-                    image_info.compression=LZWCompression;
+                    image_info->compression=LZWCompression;
                   if (Latin1Compare("RunlengthEncoded",option) == 0)
-                    image_info.compression=RunlengthEncodedCompression;
+                    image_info->compression=RunlengthEncodedCompression;
                   if (Latin1Compare("Zip",option) == 0)
-                    image_info.compression=ZipCompression;
-                  if (image_info.compression == UndefinedCompression)
+                    image_info->compression=ZipCompression;
+                  if (image_info->compression == UndefinedCompression)
                     MagickError(OptionError,"Invalid compression type",option);
                 }
               break;
@@ -566,37 +578,37 @@ int main(int argc,char **argv)
         {
           if (strncmp("delay",option+1,3) == 0)
             {
-              image_info.delay=(char *) NULL;
+              image_info->delay=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing delay",option);
-                  image_info.delay=PostscriptGeometry(argv[i]);
+                  image_info->delay=PostscriptGeometry(argv[i]);
                 }
               break;
             }
           if (strncmp("density",option+1,3) == 0)
             {
-              image_info.density=(char *) NULL;
+              image_info->density=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  (void) CloneString(&image_info.density,argv[i]);
+                  (void) CloneString(&image_info->density,argv[i]);
                 }
               break;
             }
           if (strncmp("depth",option+1,3) == 0)
             {
-              image_info.depth=QuantumDepth;
+              image_info->depth=QuantumDepth;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing image depth",option);
-                  image_info.depth=atoi(argv[i]);
+                  image_info->depth=atoi(argv[i]);
                 }
               break;
             }
@@ -604,31 +616,31 @@ int main(int argc,char **argv)
             break;
           if (Latin1Compare("display",option+1) == 0)
             {
-              image_info.server_name=(char *) NULL;
+              image_info->server_name=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing server name",option);
-                  (void) CloneString(&image_info.server_name,argv[i]);
+                  (void) CloneString(&image_info->server_name,argv[i]);
                 }
               break;
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=(char *) NULL;
+              image_info->dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing method",option);
-                  image_info.dispose=PostscriptGeometry(argv[i]);
+                  image_info->dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
           if (strncmp("dither",option+1,3) == 0)
             {
-              image_info.dither=(*option == '-');
+              image_info->dither=(*option == '-');
               break;
             }
           if (strncmp("draw",option+1,2) == 0)
@@ -720,13 +732,13 @@ int main(int argc,char **argv)
             break;
           if (strncmp("font",option+1,3) == 0)
             {
-              image_info.font=(char *) NULL;
+              image_info->font=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing font name",option);
-                  (void) CloneString(&image_info.font,argv[i]);
+                  (void) CloneString(&image_info->font,argv[i]);
                 }
               break;
             }
@@ -739,10 +751,10 @@ int main(int argc,char **argv)
                   if (i == argc)
                     MagickError(OptionError,"Missing image type",option);
                   (void) CloneString(&format,argv[i]);
-                  (void) strcpy(image_info.filename,format);
-                  (void) strcat(image_info.filename,":");
-                  SetImageInfo(&image_info,False);
-                  if (*image_info.magick == '\0')
+                  (void) strcpy(image_info->filename,format);
+                  (void) strcat(image_info->filename,":");
+                  SetImageInfo(image_info,False);
+                  if (*image_info->magick == '\0')
                     MagickError(OptionError,"Unrecognized image format",format);
                 }
               break;
@@ -759,13 +771,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("fuzz",option+1,2) == 0)
             {
-              image_info.fuzz=0;
+              image_info->fuzz=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing distance",option);
-                  image_info.fuzz=atoi(argv[i]);
+                  image_info->fuzz=atoi(argv[i]);
                 }
               break;
             }
@@ -850,23 +862,23 @@ int main(int argc,char **argv)
             }
           if (strncmp("interlace",option+1,3) == 0)
             {
-              image_info.interlace=NoInterlace;
+              image_info->interlace=NoInterlace;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.interlace=UndefinedInterlace;
+                  image_info->interlace=UndefinedInterlace;
                   if (Latin1Compare("None",option) == 0)
-                    image_info.interlace=NoInterlace;
+                    image_info->interlace=NoInterlace;
                   if (Latin1Compare("Line",option) == 0)
-                    image_info.interlace=LineInterlace;
+                    image_info->interlace=LineInterlace;
                   if (Latin1Compare("Plane",option) == 0)
-                    image_info.interlace=PlaneInterlace;
+                    image_info->interlace=PlaneInterlace;
                   if (Latin1Compare("Partition",option) == 0)
-                    image_info.interlace=PartitionInterlace;
-                  if (image_info.interlace == UndefinedInterlace)
+                    image_info->interlace=PartitionInterlace;
+                  if (image_info->interlace == UndefinedInterlace)
                     MagickError(OptionError,"Invalid interlace type",option);
                 }
               break;
@@ -913,25 +925,25 @@ int main(int argc,char **argv)
             }
           if (strncmp("linewidth",option+1,2) == 0)
             {
-              image_info.linewidth=0;
+              image_info->linewidth=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing size",option);
-                  image_info.linewidth=atoi(argv[i]);
+                  image_info->linewidth=atoi(argv[i]);
                 }
               break;
             }
           if (strncmp("loop",option+1,2) == 0)
             {
-              image_info.iterations=(char *) NULL;
+              image_info->iterations=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing iterations",option);
-                  image_info.iterations=PostscriptGeometry(argv[i]);
+                  image_info->iterations=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -955,13 +967,12 @@ int main(int argc,char **argv)
             break;
           if (strncmp("mattecolor",option+1,6) == 0)
             {
-              image_info.matte_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing matte color",option);
-                  (void) CloneString(&image_info.matte_color,argv[i]);
+                  (void) QueryColorDatabase(argv[i],&image_info->matte_color);
                 }
               break;
             }
@@ -977,7 +988,7 @@ int main(int argc,char **argv)
             }
           if (strncmp("monochrome",option+1,4) == 0)
             {
-              image_info.monochrome=(*option == '-');
+              image_info->monochrome=(*option == '-');
               break;
             }
           MagickError(OptionError,"Unrecognized option",option);
@@ -1029,13 +1040,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("page",option+1,3) == 0)
             {
-              image_info.page=(char *) NULL;
+              image_info->page=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing page geometry",option);
-                  image_info.page=PostscriptGeometry(argv[i]);
+                  image_info->page=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -1051,25 +1062,25 @@ int main(int argc,char **argv)
             }
           if (strncmp("pen",option+1,2) == 0)
             {
-              image_info.pen=(char *) NULL;
+              image_info->pen=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing pen color",option);
-                  (void) CloneString(&image_info.pen,argv[i]);
+                  (void) CloneString(&image_info->pen,argv[i]);
                 }
               break;
             }
           if (strncmp("pointsize",option+1,2) == 0)
             {
-              image_info.pointsize=12;
+              image_info->pointsize=12;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing size",option);
-                  image_info.pointsize=atoi(argv[i]);
+                  image_info->pointsize=atof(argv[i]);
                 }
               break;
             }
@@ -1080,13 +1091,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("quality",option+1,2) == 0)
             {
-              image_info.quality=75;
+              image_info->quality=75;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing quality",option);
-                  image_info.quality=atoi(argv[i]);
+                  image_info->quality=atoi(argv[i]);
                 }
               break;
             }
@@ -1147,6 +1158,16 @@ int main(int argc,char **argv)
                 }
               break;
             }
+          if (strncmp("scale",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !IsGeometry(argv[i]))
+                    MagickError(OptionError,"Missing geometry",option);
+                }
+              break;
+            }
           if (strncmp("scene",option+1,3) == 0)
             {
               scene=0;
@@ -1209,13 +1230,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("size",option+1,2) == 0)
             {
-              image_info.size=(char *) NULL;
+              image_info->size=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  (void) CloneString(&image_info.size,argv[i]);
+                  (void) CloneString(&image_info->size,argv[i]);
                 }
               break;
             }
@@ -1256,13 +1277,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("texture",option+1,5) == 0)
             {
-              image_info.texture=(char *) NULL;
+              image_info->texture=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing filename",option);
-                  (void) CloneString(&image_info.texture,argv[i]);
+                  (void) CloneString(&image_info->texture,argv[i]);
                 }
               break;
             }
@@ -1303,18 +1324,18 @@ int main(int argc,char **argv)
         {
           if (strncmp("units",option+1,3) == 0)
             {
-              image_info.units=UndefinedResolution;
+              image_info->units=UndefinedResolution;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.units=UndefinedResolution;
+                  image_info->units=UndefinedResolution;
                   if (Latin1Compare("PixelsPerInch",option) == 0)
-                    image_info.units=PixelsPerInchResolution;
+                    image_info->units=PixelsPerInchResolution;
                   if (Latin1Compare("PixelsPerCentimeter",option) == 0)
-                    image_info.units=PixelsPerCentimeterResolution;
+                    image_info->units=PixelsPerCentimeterResolution;
                 }
               break;
             }
@@ -1325,18 +1346,18 @@ int main(int argc,char **argv)
         {
           if (strncmp("verbose",option+1,2) == 0)
             {
-              image_info.verbose=(*option == '-');
+              image_info->verbose=(*option == '-');
               break;
             }
           if (strncmp("view",option+1,3) == 0)
             {
-              image_info.view=(char *) NULL;
+              image_info->view=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing view transform",option);
-                  (void) CloneString(&image_info.view,argv[i]);
+                  (void) CloneString(&image_info->view,argv[i]);
                 }
               break;
             }
@@ -1371,8 +1392,8 @@ int main(int argc,char **argv)
         /*
           Option is a file name: begin by reading image from specified file.
         */
-        (void) strcpy(image_info.filename,argv[i]);
-        image=ReadImage(&image_info);
+        (void) strcpy(image_info->filename,argv[i]);
+        image=ReadImage(image_info);
         if (image == (Image *) NULL)
           {
             if (*option == '-')
@@ -1398,24 +1419,24 @@ int main(int argc,char **argv)
               (void) strcpy(p,format);
             else
               {
-                FormatString(image_info.filename,"%.1024s:%.1024s",format,
+                FormatString(image_info->filename,"%.1024s:%.1024s",format,
                   image->filename);
-                (void) strcpy(image->filename,image_info.filename);
+                (void) strcpy(image->filename,image_info->filename);
               }
           }
         /*
           Transmogrify image as defined by the image processing options.
         */
-        MogrifyImages(&image_info,i,argv,&image);
+        MogrifyImages(image_info,i,argv,&image);
         if (global_colormap)
-          (void) MapImages(image,(Image *) NULL,image_info.dither);
+          (void) MapImages(image,(Image *) NULL,image_info->dither);
         /*
           Write transmogrified image to disk.
         */
         if (format != (char *) NULL)
           AppendImageFormat(format,image->filename);
         else
-          if (strcmp(image_info.filename,"-") != 0)
+          if (strcmp(image_info->filename,"-") != 0)
             {
               (void) strcpy(filename,image->filename);
               LocaleFilename(image->filename);
@@ -1425,19 +1446,19 @@ int main(int argc,char **argv)
           (void) strcpy(p->filename,image->filename);
           p->scene=scene++;
         }
-        SetImageInfo(&image_info,True);
+        SetImageInfo(image_info,True);
         for (p=image; p != (Image *) NULL; p=p->next)
         {
-          status=WriteImage(&image_info,p);
-          if ((status == False) || image_info.adjoin)
+          status=WriteImage(image_info,p);
+          if ((status == False) || image_info->adjoin)
             break;
         }
         for (p=image; p != (Image *) NULL; p=p->next)
           scene--;
-        if (image_info.verbose)
-          DescribeImage(image,(FILE *) NULL,False);
+        if (image_info->verbose)
+          DescribeImage(image,stderr,False);
         if ((format == (char *) NULL) && (status != False))
-          if (Latin1Compare(image_info.filename,"-") != 0)
+          if (Latin1Compare(image_info->filename,"-") != 0)
             {
               (void) remove(filename);
               (void) rename(image->filename,filename);
@@ -1447,7 +1468,10 @@ int main(int argc,char **argv)
     }
   if (image == (Image *) NULL)
     MagickError(OptionError,"Missing an image file name",(char *) NULL);
+  DestroyImageInfo(image_info);
   DestroyDelegateInfo();
+  DestroyMagickInfo();
+  FreeMemory(argv);
   Exit(status ? 0 : errno);
   return(False);
 }
